<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use ArrayAccess;
use Exception;
use Ron\DB\ArrayCursor;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Utils\Attributes;


/**
 * Zajistuje pristup k datum na radku. Umoznuje indexovani pomoci ciselnych indexu, ale take pomoci kodu poli v browseru.
 */
class BrowserRowArray implements ArrayAccess, \Countable
{
    /** @var array */
    protected $data = null;
    /** @var BrowserField[] */
    protected $browserFields;


    /**
     * BrowserRowArray constructor.
     * @param BrowserField[] $browserFields
     */
    public function __construct($browserFields)
    {
        //
        $this ->browserFields = [];
        foreach ($browserFields as $field) {
            if ($field ->code) {
                $this->browserFields[$field->code] = $field;
            }
        }
    }


    /**
     * @param array $data
     */
    public function setData($data)
    {
        $this->data = $data;
    }


    /**
     * Vraci index datoveho pole se zadanym offsetem.
     * Offset muze byt primo index (integer, nebo string obsahujici integer),
     * nebo kod sloupce v browseru.
     * @param $offset
     * @return int|null
     */
    private function getIndex($offset)
    {
        if (!$this ->data) {
            return null;
        }

        if (is_integer($offset)) {
            return ($offset < 0)
                ? count($this ->data) + $offset
                : $offset;
        }

        if (is_string($offset) && $offset !== '' &&  $offset[0] >= '0'  &&  $offset[0] <= '9') {
            return ((int)$offset < 0)
                ? count($this ->data) + (int)$offset
                : (int)$offset;
        }

        $field = isset($this ->browserFields[$offset]) ? $this ->browserFields[$offset] : null;
        if ($field) {
            return $field ->sqlFieldIndex < 0
                ? count($this ->data) + $field ->sqlFieldIndex
                : $field ->sqlFieldIndex;
        }

        return null;
    }


    public function offsetExists($offset): bool
    {
        $index = $this ->getIndex($offset);
        return isset($index)  &&  $index >= 0  &&  $index < count($this ->data)  &&  isset($this ->data[$index]);
    }


    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        if (is_string($offset)  &&  $offset !== ''  &&  $offset[0] === '@') {
            // pokud index zacina znakem @, tak vola getViewText pole se zadanym indexem (bez uvodniho znaku @)
            $fieldOffset = substr($offset, 1);

            $field = isset($this ->browserFields[$fieldOffset]) ? $this ->browserFields[$fieldOffset] : null;

            // zjisti hodnotu datoveho pole
            $index = $this ->getIndex($fieldOffset);

            if ($field  &&  $field ->visible) {
                return $field ->getViewText(isset($index) ? $this ->data[$index] : null, null, null, $this);
//                return (isset($index)  &&  $index >= 0  &&  $index < count($this ->data))
//                    ? $field ->getViewText($this ->data[$index], null, null, $this)
//                    : null;
            }

            return null;
        }

        $index = $this ->getIndex($offset);

        return (isset($index)  &&  $index >= 0  &&  $index < count($this ->data))
            ? $this ->data[$index]
            : null;
    }

    public function offsetSet($offset, $value): void
    {
        $index = $this ->getIndex($offset);

        if (isset($index)  &&  $index >= 0  &&  $index < count($this ->data)) {
            $this ->data[$index] = $value;
        }
    }

    public function offsetUnset($offset): void
    {
        $index = $this ->getIndex($offset);

        if (isset($index)  &&  $index >= 0  &&  $index < count($this ->data)) {
            unset($this ->data[$index]);
        }
    }

    public function count(): int
    {
        return $this ->data
            ? count($this ->data)
            : 0;
    }
}


/**
 * Zajistuje nacteni dat z databaze a renderovani tabulky se seznamem. Umoznuje konfiguraci zobrazenych sloupcu atd.
 */
class Browser implements ViewObject, ConfigurableObject
{
    // Mody zobrazeni tabulky - tabulka, nebo jednotlive divy (napr. dlazdice)
    const MODE_TABLE = 'table';
    const MODE_TILES = 'tiles';
    const MODE_LIST = 'list';

    /** @var  Database */
    public $database;

    /** @var string     Kod browseru - slouzi pro ukladani nastaveni do SESSION */
    public $code;
    /** @var string     Nadpis browseru */
    public $title;
    /** @var string     Id tabulky v html strance */
    public $tableId;

    /** @var BrowserField[] */
    public $fields;

    private $mode = self::MODE_TABLE;

    /** @var string|SqlStatement     Select cast vyrazu (jmena poli) - pokud neni zadana, sestavi se automaticky z poli */
    public $sqlSelectPart;
    /** @var string|SqlStatement     From cast sql vyrazu */
    public $sqlFromPart;
    /** @var string|SqlStatement  Where cast sql vyrazu */
    public $sqlWherePart;
    /** @var string     Pokud neni zadano, radi se podle $defaultSort */
    public $sqlOrderPart;
    /** @var true|string     GROUP BY cast SQL dotazu. Pokud je nastaveno na true (doporuceno), tak se sestavi automaticky - to umoznuje i pridavani poli v XML konfiguraci. */
    public $sqlGroupByPart;
    /** @var bool       Priznak, jestli se ma do selektu pridavat DISTINCT */
    public $distinct;

    /** @var int        Implicitni razeni v browseru - cislo pole (zaporne pro sestupne razeni) */
    public $defaultSort;
    /** @var int        Maximalni pocet zaznamu na strance - pokud je zadan, pokusi se strankovat (nemusi fungovat na vsech databazich!) */
    public $maxPageRows = null;
    /** @var int        Index zobrazene stranky */
    public $pageIndex = null;
    //public $maxLinesCount; // maximalni pocet radku, ktery se zobrazi. Pokud jich bude vice, tak se strankovat

    /** @var  int|string    Index primarniho pole v seznamu poli - slouzi pro identifikaci zaznamu pro editaci */
    public $primaryField;
    /** @var  bool      Nastavit na true, pokud se nema zobrazovat header */
    public $noHeader;
    /** @var  string    Odkaz na akci, ktera se provede po kliknuti na radek */
    public $rowClickLink;
    /** @var  bool      Jestli se ma zobrazovat paticka seznamu */
    public $showFooter;
    /** @var bool       Jestli se maji zobrazit zpravy MessageManageru */
    public $showMessages;

    /** @var int        Nastavi se v load pomoci getOrderFieldIndex */
    public $orderFieldIndex;
    /** @var int        Pomocna promenna - pocet sloupcu - urci se pri vykreslovani hlavicky */
    public $columnCount;

    /** @var string     css trida pro zobrazeni, defaultne "browser" */
    public $cssClass;

    /** @var string     css trida radku */
    public $rowCssClass = '';

    /** @var string     css trida dlazdice */
    public $tileCssClass = 'tile';

    /** @var string|string[]     Url adresa pro zobrazeni tohoto browseru - pouzije se pro sestaveni odkazu pro razeni atd. Pokud obsahuje pole tak jej pouzije jako jednotlive parametry a metoda getBrowserUrl z nich sestavi retezec */
    public $browserUrl;

    /** @var Cursor  Data nactena pomoci load */
    public $cursor;

    /** @var string     CSS, ktery se vyrenderuje pred seznamem. Lze nastavit napriklad uzivatelskou konfiguraci. */
    public $cssStyle;

    /** @var string[]   Asociativni pole HTML parametru */
    public $tableHtmlParams = [];

    /** @var ObjectConfiguration    Konfiguracni objekt - umoznuje nacist konfiguraci z XML */
    public $objectConfiguration;

    /** @var bool       Priznak, ze se maji data nacist rovnou do pameti do ArrayCursoru */
    public $loadDataToArray = false;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
        $this ->cssClass = 'browser rowOver';
        $this ->showMessages = true;
    }


    /**
     * @param string $mode
     * @return Browser
     */
    public function setMode(string $mode, $updateClasses = true): Browser
    {
        $this->mode = $mode;

        if ($updateClasses) {
            switch ($mode) {
                case self::MODE_TILES :
                    $this ->cssClass = 'row';
                    $this ->rowCssClass = 'col padding-all';
                    break;

                case self::MODE_TABLE :
                    $this ->cssClass = 'browser rowOver';
                    $this ->rowCssClass = '';
                    break;

                case self::MODE_LIST :
                    $this ->cssClass = '';
                    $this ->rowCssClass = '';
                    break;
            }
        }

        return $this;
    }

    /**
     * @param string $code
     * @return Browser
     */
    public function setCode(string $code): Browser
    {
        $this->code = $code;
        return $this;
    }

    /**
     * @return string|string[]|null
     */
    public function getBrowserUrl()
    {
        return $this->browserUrl;
    }


    /**
     * @param string $browserUrl
     * @return Browser
     */
    public function setBrowserUrl(string $browserUrl): Browser
    {
        $this->browserUrl = $browserUrl;
        return $this;
    }


    /**
     * @param string $title
     * @return Browser
     */
    public function setTitle(string $title): Browser
    {
        $this->title = TextManager::_($title);
        return $this;
    }

    /**
     * @param string $tableId
     * @return Browser
     */
    public function setTableId(string $tableId): Browser
    {
        $this->tableId = $tableId;
        return $this;
    }

    /**
     * @param string $cssClass
     * @return Browser
     */
    public function setCssClass(string $cssClass): Browser
    {
        $this->cssClass = $cssClass;
        return $this;
    }

    /**
     * @param string $rowCssClass
     * @return Browser
     */
    public function setRowCssClass(string $rowCssClass): Browser
    {
        $this->rowCssClass = $rowCssClass;
        return $this;
    }

    /**
     * @param string $tileCssClass
     * @return Browser
     */
    public function setTileCssClass(string $tileCssClass): Browser
    {
        $this->tileCssClass = $tileCssClass;
        return $this;
    }


    /**
     * Prida sloupec. Pokud je zadan primo objekt BrowserField, tak jej vlozi. Jinak vytvori novy objekt zadaneho typu nebo se zadanymi parametry
     * @param BrowserField|array|string $field  Objekt BrowserField nebo asociativni pole podle ktereho se vytvori novy objekt nebo typ pole
     * @return BrowserField
     */
    public function addField($field = null)
    {
        if (is_array($field)) {
            $field = new BrowserField($this ->database, $field);
        } else if (!$field  ||  !($field instanceof BrowserField)) {
            $newField = new BrowserField($this ->database);
            $newField ->setType($field ?: BrowserField::FIELD_TYPE_DEFAULT);
            $field = $newField;
        }

        $this ->fields[] = $field;

        return $field;
    }


    public function visibleFieldsCount()
    {
        $result = 0;

        foreach ($this ->fields as $field) {
            if ($field ->visible  &&  $field ->type !== BrowserField::FIELD_TYPE_HIDDEN  &&  $field ->type !== BrowserField::FIELD_TYPE_COMMENT) {
                $result++;
            }
        }

        return $result;
    }

    /**
     * Najde pole se zadanym Id
     * @param string $fieldId
     * @return int
     */
    public function findFieldIndex($fieldId)
    {
        foreach ($this ->fields as $index => $field) {
            if ($field ->getId() === $fieldId) {
                return $index;
            }
        }

        return null;
    }

    /**
     * Vraci pole se zadanym kodem
     * @param $code
     * @return null|int
     */
    public function findFieldIndexByCode($code)
    {
        foreach ($this ->fields as $index => $field) {
            if ($field ->code === $code) {
                return $index;
            }
        }

        return null;
    }


    /**
     * Vraci pole se zadanym kodem, nebo null, kdyz jej nenajde
     * @param string $code
     * @return BrowserField|null
     */
    public function findFieldByCode($code)
    {
        $index = $this ->findFieldIndexByCode($code);
        return isset($index)
            ? $this ->fields[$index]
            : null;
    }

    /**
     * Prida BrowserFields ze zadaneho pole $fields - to mohou byt primo objekty BrowserField nebo pole pro init()
     * @param array $fields
     */
    public function addFields($fields)
    {
        foreach($fields as $field) {
            $this ->addField($field);
        }
    }


    /**
     * Presune zadane pole ($fieldIndex) pred pole s kodem $before, nebo za pole s kodem $after
     * @param int $fieldIndex
     * @param string $before
     * @param string $after
     */
    public function moveField($fieldIndex, $before, $after)
    {
        if ($before) {
            $field2Index = $this ->findFieldIndexByCode($before);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex + 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                array_splice($this ->fields, $field2Index, 0, $removed);
                return;
            }
        }

        if ($after) {
            $field2Index = $this ->findFieldIndexByCode($after);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex - 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);
                array_splice($this ->fields, $field2Index + 1, 0, $removed);
                return;
            }
        }
    }


    public function createNewField($colSettings)
    {
        if (!isset($colSettings['sqlFieldName'])) {
            return null;
        }

        $field = $this ->addField(isset($colSettings['type']) ? $colSettings['type'] : BrowserField::FIELD_TYPE_DEFAULT);
        $field ->loadConfigurationArray($colSettings);

        return count($this->fields) - 1;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'style' :
                $this ->cssStyle = ($this ->cssStyle ?: '') . $valueText;
                break;

            case 'showFooter' :
            case 'noHeader' :
            case 'distinct' :
            case 'defaultSort' :
            case 'maxPageRows' :
            case 'pageIndex' :
            case 'showMessages' :
            case 'loadDataToArray' :
                $this ->{$key} = (int)$valueText;
                break;

            case 'tableId' :
            case 'sqlSelectPart' :
            case 'sqlFromPart' :
            case 'sqlWherePart' :
            case 'sqlOrderPart' :
            case 'sqlGroupByPart' :
            case 'rowClickLink' :
            case 'cssClass' :
            case 'rowCssClass' :
            case 'tileCssClass' :
            case 'browserUrl' :
                $this ->{$key} = $valueText;
                break;

            case 'sqlFilter' :
                if ($valueText) {
                    $this ->sqlWherePart .= ($this ->sqlWherePart ? ' AND ' : '') . $valueText;
                }
                break;

            case 'title' :
                $this ->{$key} = TextManager::_($valueText);
                break;

            case 'column' :
                $this ->loadColumnsConfiguration($value);
                break;
        }
    }

    /**
     * @param ArrayAccess $colSettings
     */
    public function loadColumnConfiguration($colSettings)
    {
        $fieldIndex = isset($colSettings['code'])
            ? $this ->findFieldIndexByCode($colSettings['code'])
            : null;

        if (isset($fieldIndex)) {
            $this->fields[$fieldIndex]->loadConfigurationArray($colSettings);
        } else {
            // zkusi vytvorit uplne nove pole
            $fieldIndex = $this ->createNewField($colSettings);
        }

        if (isset($fieldIndex)) {
            if ($colSettings['before']  ||  $colSettings['after']) {
                $this ->moveField($fieldIndex, $colSettings['before'], $colSettings['after']);
            }
        }
    }

    /**
     * @param ArrayAccess $columnsConfArray
     */
    public function loadColumnsConfiguration($columnsConfArray)
    {
        if (!$columnsConfArray) {
            return;
        }

        foreach ($columnsConfArray as $colSettings) {
            $this ->loadColumnConfiguration($colSettings);
        }
    }


    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        $this ->objectConfiguration = new ObjectConfiguration($this);
        $this ->objectConfiguration ->loadConfigurationXml($xmlText);
    }


    /**
     * Vraci ORDER BY klauzuli
     * @return string
     */
    public function getOrderSQLPart()
    {
        if ($this ->sqlOrderPart) {
            return $this->sqlOrderPart;
        }

        if (!$this ->orderFieldIndex) {
            return '';
        } else if ($this ->orderFieldIndex > count($this->fields)  ||  $this ->orderFieldIndex < -count($this->fields)) {
            return '';
        }

        $field = $this ->fields[abs($this ->orderFieldIndex) - 1];
        if ($this ->orderFieldIndex < 0) {
            if ($field ->sortSQLDesc) {
                return $field ->sortSQLDesc;
            }
            if ($field ->sortSQL) {
                return implode(' DESC,', explode(',', $field ->sortSQL)) . ' DESC';
                //return $field ->sortSQL . ' DESC';
            }
            if ($field ->sqlFieldName) {
                return $field ->sqlFieldName . ' DESC';
            }
        } else {
            if ($field ->sortSQL) {
                return $field ->sortSQL;
            }

            return $field ->sqlFieldName;
        }

        return '';
    }


    /**
     * Vraci cast retezce, ktera se vlozi do WHERE casti dotazu
     * muze vracet napriklad filtr podle zvolenych filtru (viz renderFilter)
     * @return string|SqlStatement
     */
    public function getSQLFilter()
    {
        return '';
    }


    /**
     * @param BrowserField $field
     */
    public function setNewFieldSqlIndex($field)
    {
        if (!isset($field ->sqlFieldIndex)  &&  $field ->sqlFieldName) {
            // na konec bylo pridano nove pole => posune indexy vsech ostatnich poli, ktera maji zaporny index
            foreach ($this ->fields as $f) {
                if ($f ->sqlFieldIndex < 0) {
                    $f ->sqlFieldIndex--;
                }
            }

            $field ->sqlFieldIndex = -1; // aktualne je to posledni pole odzadu
        }
    }


    /**
     * Vraci jmena poli pro SQL dotaz (tj. SELECT cast dotazu) - pokud neni primo zadana, tak ji sestavi z nazvu poli
     * @return string|SqlStatement
     */
    public function getSelectSQLPart()
    {
        $result = $this ->sqlSelectPart ?: '';

        $sqlFieldIndex = 0;

        foreach ($this ->fields as $i => $field) {
            if ($field ->sqlFieldName  &&  $field ->visible  &&  !isset($field ->sqlFieldIndex)) {
                // prida nazev pole do SQL dotazu
                $result = SqlStatement::concatenate(
                    [$result, $field ->castWideMemo ? $this ->database ->castWideMemo($field ->sqlFieldName) : $field ->sqlFieldName],
                    ', ', false);

//                // prida nazev pole do SQL dotazu
//                $result .=
//                    ($result ? ', ' : '') .
//                    ($field ->castWideMemo ? $this ->database ->castWideMemo($field ->sqlFieldName) : $field ->sqlFieldName);

                // a nastavi index
                if ($this ->sqlSelectPart) {
                    // seznam ma nastaven select part => nova pole pridavam na konec a cisluji zapornym indexem odzadu
                    // ... vsechna ostatni pole se zapornym indexem se musi posunout
                    $this ->setNewFieldSqlIndex($field);
                } else {
                    // seznam nema nastaven select - muzu cislovat normalne od nuly
                    $this ->fields[$i] ->sqlFieldIndex = $sqlFieldIndex;
                }

                $sqlFieldIndex++;
            }
        }

        return $result;
    }


    /**
     * Vraci SELECT cast SQL dotazu
     * @return string|SqlStatement
     */
    public function getSqlFromPart()
    {
        return $this ->sqlFromPart;
    }


    /**
     * Vraci GROUP BY cast SQL dotazu
     * @return string
     */
    public function getSqlGroupByPart()
    {
        if ($this ->sqlGroupByPart === true) {
            if ($this ->sqlSelectPart instanceof SqlStatement) {
                throw new Exception('GROUP BY neni podporovan pro browser s SqlStatment v sqlSelectPart!');
            }
            // pokud je true, tak se sestavi automaticky stejne jako SELECT part, ale vynecha pole, ktera maji nastaveno excludeFromGroupBy
            $result = $this ->sqlSelectPart ?: '';

            foreach ($this ->fields as $i => $field) {
                if ($field ->sqlFieldName  &&  $field ->visible  &&  !$field ->excludeFromGroupBy) {
                    if ($field ->sqlFieldName instanceof SqlStatement) {
                        throw new Exception('GROUP BY neni podporovan pro browser s SqlStatment v sqlSelectPart!');
                    }

                    // prida nazev pole do SQL dotazu
                    $result .= ($result ? ', ' : '') . $field ->sqlFieldName;
                }
            }

            return $result;
        }

        return $this ->sqlGroupByPart;
    }


    /**
     * @param string|SqlStatement   $sqlFilter   Dodatecny sql filtr
     * @return string|SqlStatement
     */
    public function getSQL($sqlFilter = '')
    {
        $orderBy = $this ->getOrderSQLPart();

        $wherePart = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter()]);

        // TODO: PRIDAT Omezeni poctu radku ?
        //$sqlText .= ' OFFSET 5 ROWS FETCH NEXT 5 ROWS ONLY'; - funguje na MSSQL a na ORACLE

        return SQLUtils::createSql(
            $this ->getSelectSQLPart(), $this ->getSqlFromPart(),
            $wherePart, $orderBy, $this ->distinct, $this ->getSqlGroupByPart());
    }



    /**
     * Vraci index pole, podle ktereho se ma radit
     * @return int
     */
    public function getOrderFieldIndex()
    {
        $orderFieldCode = $_GET['sort'] ?? null;

        if (!$orderFieldCode && $this->code) {
            // razeni neni nastaveno v URL
            if (isset($_SESSION['BROWSERSSORT'][$this->code])) {
                // uz ulozeno v session
                $orderFieldCode = $_SESSION['BROWSERSSORT'][$this->code];
            } else {
                // zkusi nacist uzivatelske nastaveni
                $orderFieldCode = App::getInstance()->getUserSettings('browser_' . $this->code . '_sort');
            }
        }

        if ($orderFieldCode) {
            // nastaveno v URL nebo v session nebo v uzivatelskem nastaveni
            $minus = $orderFieldCode[0] === '-';
            if ($minus) {
                $orderFieldCode = substr($orderFieldCode, 1);
            }

            $orderFieldIndex = $this ->findFieldIndexByCode($orderFieldCode);

            if (isset($orderFieldIndex)) {
                if ($this ->fields[$orderFieldIndex] ->visible  &&  $this ->fields[$orderFieldIndex] ->ordering  &&  $this ->fields[$orderFieldIndex] ->type !== BrowserField::FIELD_TYPE_HIDDEN) {
                    if ($this->code  &&  isset($_GET['sort'])) {
                        App::getInstance()->storeUserSettings(
                            'browser_' . $this->code . '_sort', ($minus ? '-' : '') . $orderFieldCode, $this ->defaultSort);
                    }
                    return ($minus ? -1 : 1) * ($orderFieldIndex + 1);  // cislovani od 1 kvuli moznosti sestupneho razeni
                }
            }
        }

        if (!$this ->defaultSort) {
            return null;
        }

        if (is_integer($this ->defaultSort)) {
            global $development;
            if ($development) {
                MessageManager::addWarning('Nastaveni vychoziho razeni v browseru (defaultSort) na integer je zastarale! Pouzijte kod pole!');
            }

            return $this ->defaultSort;
        }

        $minus = $this ->defaultSort[0] === '-';
        $orderFieldCode = $minus
            ? substr($this ->defaultSort, 1)
            : $this ->defaultSort;

        $orderFieldIndex = $this ->findFieldIndexByCode($orderFieldCode);

        if (isset($orderFieldIndex)) {
            if ($this ->fields[$orderFieldIndex] ->visible) {
                return ($minus ? -1 : 1) * ($orderFieldIndex + 1); // cislovani od 1 (kvuli moznosti sestupneho razeni
            }
        }

        return null;

//        $orderFieldIndex = $this->defaultSort;
//        $defaultSortStr = $this ->defaultSort
//            ? (($this ->defaultSort < 0 ? '-' : '') . $this ->fields[abs($this ->defaultSort) - 1] ->getId())
//            : '';
//
//        if (isset($_GET['sort'])) {
//            // v URL nastaveno nove razeni
//            $newFieldIndex = $_GET['sort']; // pozor, cislovani od 1 (aby bylo mozne rozlisit sestupne/vzestupne)
//            if ($newFieldIndex  &&  abs($newFieldIndex) <= count($this ->fields)  &&  $this ->fields[abs($newFieldIndex) - 1] ->ordering) {
//                // OK, pole existuje a je mozne podle nej radit
//                $orderFieldIndex = $newFieldIndex;
//
//                if ($this ->code) {
//                    // zapise do nastaveni uzivatele
//                    App::getInstance() ->storeUserSettings(
//                        'browser_' . $this->code . '_sort',
//                        ($newFieldIndex < 0 ? '-' : '') . $this ->fields[abs($newFieldIndex) - 1] ->getId(),
//                        $defaultSortStr );
//
//                    // a do session
//                    $_SESSION['BROWSERSSORT'][$this->code] = $newFieldIndex;
//                }
//            }
//        } else if ($this->code) {
//            if (isset($_SESSION['BROWSERSSORT'][$this->code])) {
//                // uz ulozeno v session
//                $orderFieldIndex = $_SESSION['BROWSERSSORT'][$this->code];
//            } else {
//                // jeste neni v session - nacte z nastaveni uzivatele
//                $sortStr = App::getInstance()->getUserSettings('browser_' . $this->code . '_sort', $defaultSortStr);
//
//                $minus = false;
//                if ($sortStr && $sortStr[0] === '-') {
//                    $minus = true;
//                    $sortStr = substr($sortStr, 1);
//                }
//                $newFieldIndex = $this->findFieldIndex($sortStr); // pozor, cislovani od 0
//                if (isset($newFieldIndex)) {
//                    $orderFieldIndex = $newFieldIndex + 1;
//                    if ($minus) {
//                        $orderFieldIndex = -$orderFieldIndex;
//                    }
//                }
//            }
//        }
//
//        return $orderFieldIndex;
    }


    /**
     * Vraci true, pokud ma byt radek zobrazen
     * @param ArrayAccess|array $values
     * @return bool
     */
    public function enableLine($values)
    {
        return true;
    }


    /**
     * Nacte data
     * @param string $sql
     * @throws Exception
     */
    public function loadSQL($sql) // , $firstRow  * @param int  $firstRow          Prvni radek, ktery se ma zobrazit
    {
        MessageManager::addDebugLog('Browser::loadSQL ' . ($this ->code ? "({$this ->code}): " : ': ') . ($sql instanceof SqlStatement ? $sql ->sqlText : $sql));

        $this ->cursor = $this ->database ->select($sql);

        if ($this ->loadDataToArray) {
            $this ->cursor = new ArrayCursor($this ->cursor);
        }
    }



    /**
     * Nacte data
     * @param string|SqlStatement $sqlFilter     Nepovinny filtr, ktery se prida do where pomoci AND
     * @throws Exception
     */
    public function load($sqlFilter = '')
    {
        $this ->orderFieldIndex = $this ->getOrderFieldIndex();
        $sqlText = $this ->getSQL($sqlFilter);

        $this ->loadSQL($sqlText);
    }


    /**
     * Zobrazi nad seznamem filtry (napr vyber typu, stavu, ...)
     */
    public function renderFilter()
    {
        if ($this ->showMessages) {
            MessageManager::getInstance()->showMessages(is_string($this ->showMessages) ? $this ->showMessages : null);
        }
    }


    /**
     */
    public function renderHeader()
    {
        if ($this ->cssStyle) {
            echo "<style>{$this ->cssStyle}</style>\n";
        }

        echo "<div class='app-banner'>\n";

        if ($this ->title) {
            echo "<h1>{$this ->title}</h1>";
        }

        if ($this ->objectConfiguration) {
            echo $this->objectConfiguration ->getHtmlContent(['head', 'hlavicka']);
        }

        $this ->renderFilter();

        echo "</div>\n";
    }


    /**
     * Zobrazi hlavicku tabulky
     */
    public function renderTableHeader()
    {
        switch ($this ->mode) {
            case self::MODE_LIST :
            case self::MODE_TILES :
                break;

            case self::MODE_TABLE :
            default :
                echo '<thead class="floating-table-header">';
                echo '<tr>';

                $this->columnCount = 0;
                $colSpan = 0;
                $baseUrl = StringUtils::appendSuffix($this ->getBrowserUrl() ?? '', '&');

                foreach ($this->fields as $i => $field) {
                    if ($field->visible) {
                        if ($colSpan > 1) {
                            // tento header nezobrazuje, protoze predchozi byl pres vice sloupcu a zasahuje i do tohoto sloupce
                            $colSpan--;
                        } else {
                            $poms = $field->getHeader($i, $this->orderFieldIndex, $baseUrl);
                            if ($poms) {
                                echo $poms;

                                $colSpan = (int)$field->headerColSpan;
                                $this->columnCount += ($colSpan > 1) ? $colSpan : 1;
                            }
                        }
                    }
                }
                echo "</tr>\n";
                echo "</thead>\n";
        }
    }


    /**
     */
    public function renderTableFooter()
    {
        switch ($this ->mode) {
            case self::MODE_TILES :
            case self::MODE_LIST :
                break;

            case self::MODE_TABLE :
            default :
                echo '<tr>';

                $colspan = 0;
                foreach ($this->fields as $i => $field) {
                    if ($field->visible) {
                        if ($colspan > 1) {
                            // tento footer nezobrazuje, protoze predchozi byl pres vice sloupcu a zasahuje i do tohoto sloupce
                            $colspan--;
                        } else {
                            echo $field->getFooter($i, $this->getBrowserUrl());
                            $colspan = -(int)$field->footerColSpan;
                        }
                    }
                }

                echo "</tr>\n";
        }
    }


    public function getTableAttributes()
    {
        $attrs = new Attributes(['data-max-page-rows' => $this ->maxPageRows ?: '']);

        if ($this ->cssClass) {
            $attrs['class'] = $this ->cssClass;
        }
        if ($this ->tableId) {
            $attrs['id'] = $this ->tableId;
        }
        if ($this ->code) {
            $attrs['data-code'] = $this ->code;

            $dataCols = '';
            foreach ($this ->fields as $field) {
                // doplni seznam skrytych poli - aby se daly v HTML dohledat
                if ($field ->code  &&  !$field ->visible) {
                    $dataCols .= ($dataCols ? ' ' : '') . $field ->code;
                }
            }

            if ($dataCols) {
                $attrs['data-invisible-fields'] = $dataCols;
            }
        }

        if ($GLOBALS['development']) {
            $className = get_class($this);
            $attrs['data-class-name'] = $className;
            $attrs['data-sourcefile-name'] = (new \ReflectionClass($className)) ->getFileName();
        }

        if ($this ->tableHtmlParams) {
            $attrs ->append($this ->tableHtmlParams);
        }

        return $attrs;
    }

    /**
     * @param int $orderFieldIndex
     */
    public function renderTableStart($orderFieldIndex = 0)
    {
        switch ($this ->mode) {
            case self::MODE_TILES :
                echo '<div ' . $this->getTableAttributes() . ">\n";
                break;

            case self::MODE_LIST :
                echo '<ul ' . $this->getTableAttributes() . ">\n";
                break;

            case self::MODE_TABLE :
            default :
                echo '<table ' . $this->getTableAttributes() . ">\n";
        }

        if (!$this->noHeader) {
            $this->renderTableHeader();
        }
    }


    /**
     *
     */
    public function renderTableEnd()
    {
        switch ($this ->mode) {
            case self::MODE_TILES :
                echo "</div>\n";
                break;

            case self::MODE_LIST :
                echo "</ul>\n";
                break;

            case self::MODE_TABLE :
            default :
                echo "</table>\n";
        }
    }


    /**
     * Zobrazeni separatoru (radek s vodorovnou linkou)
     */
    public function renderSeparator()
    {
        if ($this ->mode === self::MODE_TABLE) {
            echo '<tr class="browserseparator"><td colspan="' . $this->columnCount . '"></td></tr>';
        }
    }


//    /**
//     * @param int   $index
//     * @param BrowserField $field
//     * @param mixed $value
//     * @param int   $rowNumber
//     * @param mixed $recordId
//     * @param array $values
//     */
//    public function renderFieldEdit($index, $field, $value, $rowNumber, $recordId, $values)
//    {
//        echo $field ->getFieldCellText($value, $rowNumber, $recordId, $values);
//    }


    /**
     * @param int   $index
     * @param BrowserField $field
     * @param mixed $value
     * @param int   $rowNumber
     * @param mixed $recordId
     * @param BrowserRowArray $values
     */
    public function renderField($index, $field, $value, $rowNumber, $recordId, $values)
    {
        $tag = $this ->mode === self::MODE_TABLE
            ? 'td'
            : 'div';

        $attrs = $field ->getAttributes($value, $rowNumber, $recordId, $values);

        $headerText = '';
        if ($this ->mode === self::MODE_TILES  &&  $field ->header) {
            $headerText =
                "<span class='{$field ->headerClass}'>{$field ->header}</span> ";
        }

        switch ($field ->type) {
            case BrowserField::FIELD_TYPE_HIDDEN :
            case BrowserField::FIELD_TYPE_COMMENT :
                break;

            case BrowserField::FIELD_TYPE_LINE :
                echo
                    "<$tag " . $attrs . '>' .
                    ($rowNumber > 0 ? $rowNumber : '') .
                    "</$tag>";
                break;

            case BrowserField::FIELD_TYPE_DEFAULT :
            case BrowserField::FIELD_TYPE_TEXT :
            case BrowserField::FIELD_TYPE_SELECT :
            default :
                echo
                    "<$tag " . $attrs . '>' .
                    $headerText .
                    $field ->getViewText($value, $rowNumber, $recordId, $values) .
                    "</$tag>";
        }
    }


    /**
     * @return int   return the index of primary field in sql query
     */
    public function getPrimaryFieldSQLIndex()
    {
        if (is_string($this ->primaryField)) {
            // primarni pole zadano pomoci
            return $this ->primaryField;
        }

        if ($this ->primaryField < 0) {
            return -1;
        }

        if ($this ->sqlSelectPart) {
            // je primo zadana sqlSelectPart => index musi byt primo index pole v SELECT casti sql dotazu
            return $this ->primaryField;
        }

        $result = 0;
        for ($i = 0; $i < $this ->primaryField - 1; $i++) {
            if ($this->fields[$i]->sqlFieldName) {
                $result++;
            }
        }

        return $result;
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @return int|string
     */
    public function getClickLink($values)
    {
        if ($this ->rowClickLink && $this ->primaryField >= 0) {
            $primaryIndex = $this ->getPrimaryFieldSQLIndex();
            if ($primaryIndex < 0) {
                return '';
            }

            return $this ->rowClickLink . '&record=' . $values[$primaryIndex];
        }

        return 0;
    }


    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    public function getRowAttributes($values, $className, $rowNumber)
    {
        $result = new Attributes();

        $link = $this ->getClickLink($values);
        if ($link) {
            $className .= ($className ? ' ' : '') . 'clickHref';
            $result['data-ron-href'] = $link;
        }

        if ($this ->rowCssClass) {
            $className .= ($className ? ' ' : '') . $this ->rowCssClass;
        }

        if ($className) {
            $result['class'] = $className;
        }

        return $result;
    }


    public function getTileAttributes($values)
    {
        return new Attributes(['class' => $this ->tileCssClass]);
    }


    public function renderComments($values, $className, $rowNumber, $recordId)
    {
        // TODO: MODE_DIV

        foreach ($this ->fields as $i => $field) {
            if ($field ->type === BrowserField::FIELD_TYPE_COMMENT  &&  $field ->visible) {
                $value = $field ->getViewText($values[$field ->sqlFieldIndex], $rowNumber, $recordId, $values);

                if ($value) {
                    echo "\n<tr class='$className'>";

                    // header
                    $headerText = $field ->getHeaderText($i, 0);

                    $cols = $this ->columnCount;
                    if ($field ->headerColSpan) {
                        // header
                        $cols -= abs($field ->headerColSpan);

                        echo
                            '<td class="' . $field ->getHeaderClass(0) . '"' .
                            ((abs($field ->headerColSpan) > 1) ? ' colspan="' . abs($field ->headerColSpan) . '"' : '') .
                            '>' .
                            ($field ->headerColSpan > 0 ? $headerText : '') .  // zobrazi header, pokud ma byt zobrazeny v samostatne bunce
                            '</td>';
                    }

                    if ($field ->footerColSpan) {
                        $cols -= $field ->footerColSpan;
                    }

                    echo
                        '<td colspan="' . $cols . '" class="' . $field ->getFieldClass() . '"' . ($field ->code ? " data-code='{$field ->code}'" : '') .'>' .
                        (($headerText  &&  (int)$field ->headerColSpan <= 0)
                            ? '<span class="' . $field ->getHeaderClass(0) . '">' . $headerText . '</span>'
                            : '') .
                        $value .
                        '</td>';

                    if ($field ->footerColSpan) {
                        echo '<td colspan="' . $field ->footerColSpan . '"></td>';
                    }

                    echo
                        '</tr>';
                }
            }
        }
    }


    public function renderFields($values, $rowNumber)
    {
        $primaryIndex = $this ->getPrimaryFieldSQLIndex();
        $recordId = ($values  &&  !is_null($primaryIndex) && isset($values[$primaryIndex]))
            ? $values[$primaryIndex]
            : '';

        foreach ($this ->fields as $i => $field) {
            if ($field ->visible) {
                $value = null;

                if (isset($field ->sqlFieldIndex)) {
                    $value = $values[$field ->sqlFieldIndex];
                }
                $this->renderField($i, $field, $value, $rowNumber, $recordId, $values);
            }
        }
    }


    /**
     * @param ArrayAccess|array  $values
     * @param string $className
     * @param int    $rowNumber
     * @param bool   $edit   Zobrazeni jednoho radku
     */
    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $rowTag = $this ->mode === self::MODE_TILES
            ? 'div'
            : ($this ->mode === self::MODE_LIST ? 'li' : 'tr');

        $rowAttrs = $this ->getRowAttributes($values, $this ->mode === self::MODE_TILES ? '' : $className, $rowNumber);
        echo "<$rowTag $rowAttrs>";

        if ($this ->mode === self::MODE_TILES) {
            echo '<div ' . $this ->getTileAttributes($values) . '>';
        }

        $this ->renderFields($values, $rowNumber);

        if ($this ->mode === self::MODE_TILES) {
            echo '</div>';
        }

        echo "</$rowTag>\n";

        $this ->renderComments($values, $className, $rowNumber, $edit);
    }


    /**
     * @param Cursor $cursor
     * @throws Exception
     */
    public function renderCursor($cursor)
    {
        if (!$cursor) {
            throw new Exception('Chyba v metode browser::renderCursor - nejsou nactena data!');
        }

        $i = 1;

        $browserRowArray = new BrowserRowArray($this ->fields);

        while ($row = $cursor ->fetch()) {
            $browserRowArray ->setData($row);

            if ($this ->enableLine($browserRowArray)) {
                // Pozor! Pokud enableLine vypne nektere radky a bude zapnuto strankovani, tak se mohou nektere radky opakovat!
                $this ->renderLine($browserRowArray, ($i % 2) ? 'browsercolor1' : 'browsercolor2', $i);

                $i++;
                if ($this ->maxPageRows  &&  isset($this ->pageIndex)) {
                    if ($i > $this->maxPageRows) {
                        break;
                    }
                }
            }
        }

        $this ->cursor ->close();
    }


    /**
     * Zobrazi odkazy na dalsi stranky browseru
     * @param $rowsCount
     * @param $sort
     */
    public function renderPages($rowsCount, $sort)
    {
        if (!$rowsCount  ||  !$this ->maxPageRows) {
            return;
        }

        echo '<div class="pages">';
        echo '<ul class="linkslist wrap">';

        $minIndex = (int)max($this->pageIndex-5,1);
        $maxIndex = (int)min((int)($rowsCount / $this ->maxPageRows) + 1, $this->pageIndex + 5);

        $baseUrl = StringUtils::appendSuffix($this ->getBrowserUrl(), '&');

        if ($minIndex > 1) {
            $rozsah = 1 . '..' . $this->maxPageRows;
            $url = $baseUrl . 'page=1' . ($sort ? "&sort=" . $sort : '');
            echo "<li><a href='$url'><span title='$rozsah'>1</span></a>" .  ($minIndex > 2 ? ' ...' : '')  . "</li>";
        }

        for ($i = $minIndex; $i <= $maxIndex; $i++) {
            $url = $baseUrl . 'page=' . $i;
            $classStr = ($this ->pageIndex === $i) ? ' class="selectedlink"' : '';
            echo "<li$classStr><a href='$url" . ($sort ? "&sort=" . $sort : '') . "'>";

            $first = ($i - 1) * $this ->maxPageRows + 1;
            $last = $i * $this ->maxPageRows;
            if ($last > $rowsCount) {
                $last = $rowsCount;
            }

            $rozsah = $first . '..' . $last;
            echo "<span title='$rozsah'>" .  $i . "</span>" . '</a></li>';
        }

        $lastPage = (int)(($rowsCount - 1) / $this ->maxPageRows) + 1;

        if ($maxIndex < $lastPage) {
            $rozsah = ($lastPage * $this ->maxPageRows) . '..' . ($rowsCount);
            $url = $baseUrl . 'page=' . $lastPage . ($sort ? "&sort=" . $sort : '');
            echo "<li>" . (($maxIndex < $lastPage - 1) ? '... ' : '') . "<a href='$url'><span title='$rozsah'>$lastPage</span></a></li>";
        }


        echo '</ul>';

        $urlStr = $this->getBrowserUrl();

        if (strpos($urlStr, '?') === 0) {
            $urlStr = substr($urlStr, 1);
        }

        parse_str($urlStr, $urlArr);

        echo "<form method='get' class='browserpagenumber'>";

        foreach ($urlArr as $urlKey => $urValue) {
            echo "<input type='hidden' name='$urlKey' value='$urValue' >";
        }

        echo
            '<span class="margin-left-3 margin-right-2">' .
            TextManager::_('Go_to_page') .
            '</span>' .
            "<input type='number' name='page' min='1' max='$lastPage' >" .
            "<input type='submit' class='submitbutton' value='" . TextManager::_('OK') .
            "'>";

        echo "</form>";

        echo "</div>\n";
    }


    /**
     * Zobrazi data z nactenych dat
     * @throws Exception
     */
    public function renderData() // , $firstRow)
    {
        $this ->renderCursor($this ->cursor);
    }


    /**
     * @throws Exception
     */
    public function renderTable()
    {
        $this ->renderTableStart();
        echo "<tbody>\n";

        $this ->renderData(); // , $firstRow

        if ($this ->showFooter) {
            $this ->renderTableFooter();
        }

        echo '</tbody>';
        $this ->renderTableEnd();
    }


    /**
     * Vraci index stranky, ktera se ma zobrazi (standardne podle hodnoty url parametru "page")
     * @param $rowsCount
     * @return float|int
     */
    public function getPageIndex($rowsCount)
    {
        if (!$rowsCount) {
            return null;
        }

        $pageIndex = isset($_GET['page']) ? (int)$_GET['page'] : 1;

        if ($pageIndex * $this ->maxPageRows > $rowsCount) {
            // je za posledni strankou - presune se na posledni
            $pageIndex = (int)($rowsCount/$this ->maxPageRows)  + 1;
        }

        return $pageIndex;
    }


    /**
     * @throws Exception
     */
    public function renderContent()
    {
        if ($this ->objectConfiguration) {
            echo $this ->objectConfiguration ->getHtmlContent(['top', '']);
        }

        $rowsCount = 0;
        if ($this ->maxPageRows) {
            $rowsCount = $this ->cursor ->numRows();

            if ($rowsCount === false) {
                $this ->pageIndex = null;
                echo '<span class="d-none">Strankovani neni v ovladaci databaze podporovano!</span>';
            } else if ($rowsCount > $this ->maxPageRows) {
                $this ->pageIndex = $this ->getPageIndex($rowsCount);
                $this ->renderPages($rowsCount, null);

                if ($this ->pageIndex > 1) {
                    $this->cursor->seek(($this->pageIndex - 1) * $this->maxPageRows);
                }
            } else {
                $this ->pageIndex = 1;
            }
        }

        $this ->renderTable();

        if ($this ->maxPageRows  &&  $rowsCount > $this ->maxPageRows) {
            $this ->renderPages($rowsCount, null);
        }

        if ($this ->objectConfiguration) {
            echo $this ->objectConfiguration ->getHtmlContent(['behind', 'bottom', 'dole']);
        }
    }


    /**
     * @throws Exception
     */
    public function render()
    {
        $this ->renderHeader();
        $this ->renderContent();
    }

}
