<?php
/**
 * stanislav.elbl, 26.04.2022 14:48
 */

namespace Ron\Controls;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;

class OptionLoader
{
    /** @var Database */
    private $database;

    /** @var string|SqlStatement */
    public $sql;
    /** @var Parameters */
    public $sqlParams;

    /** @var int    Index pole pro nacteni hodnoty */
    public $keyIndex = 0;
    /** @var int    Index pole pro nacteni popisu */
    public $captionIndex = 1;
    /** @var string Format pole s popisem */
    public $format = null;

    /** @var string Text pro prazdnou hodnotu */
    public $emptyValue = null;

    /** @var int    Index pole, ze ktereho se nacte title */
    public $titleIndex = null;
    /** @var int    Index pole, ze ktereho se nacte cssClass */
    public $cssClassIndex = null;
    /** @var int[]  Asociativni pole index datovych atributu. Klic je nazev atributu */
    public $attrIndexes = null;
    /** @var mixed[]    Pole "preferovanych" hodnot - nactou se na zacatek (mimo poradi z sql) */
    public $preferredValues = null;
    /** @var callable   Funkce, ktera slouzi pro odfiltrovani nactenych radku. Pokud je zadana, tak se polozka do seznamu prida jen v pripade, ze tato funkce vrati true */
    public $filterFunction = null;

    /** @var string     Prefix pro preklad textu */
    public $translationPrefix;
    /** @var string     Prefix pro preklad titulku */
    public $titleTranslationPrefix;
    /** @var int        Index pole, ktere se pouzije jako kod pro preklad */
    public $translationValueIndex;


    /**
     * OptionLoader constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }

    /**
     * @param SqlStatement|string $sql
     * @param Parameters $sqlParams
     * @return $this
     */
    public function setSql($sql, $sqlParams = null)
    {
        $this->sql = $sql;
        $this->sqlParams = $sqlParams;

        return $this;
    }


    /**
     * @param int $keyIndex
     * @return $this
     */
    public function setKeyIndex($keyIndex)
    {
        $this->keyIndex = $keyIndex;
        return $this;
    }

    /**
     * @param int $captionIndex
     * @param string $format    Pokud je zadan, tak nastavi i format pole
     * @return $this
     */
    public function setCaptionIndex($captionIndex, $format = null)
    {
        $this->captionIndex = $captionIndex;
        if (isset($format)) {
            $this ->format = $format;
        }
        return $this;
    }

    /**
     * @param string $format
     * @return $this
     */
    public function setFormat($format)
    {
        $this->format = $format;
        return $this;
    }

    /**
     * @param string $emptyValue
     * @return $this
     */
    public function setEmptyValue($emptyValue)
    {
        $this->emptyValue = $emptyValue;
        return $this;
    }

    /**
     * @param int $titleIndex
     * @return $this
     */
    public function setTitleIndex($titleIndex)
    {
        $this->titleIndex = $titleIndex;
        return $this;
    }

    /**
     * @param int $cssClassIndex
     * @return $this
     */
    public function setCssClassIndex($cssClassIndex)
    {
        $this->cssClassIndex = $cssClassIndex;
        return $this;
    }

    /**
     * @param \int[] $attrIndexes
     * @return $this
     */
    public function setAttrIndexes($attrIndexes)
    {
        $this->attrIndexes = $attrIndexes;
        return $this;
    }

    /**
     * @param \mixed[] $preferredValues
     * @return $this
     */
    public function setPreferredValues($preferredValues)
    {
        $this->preferredValues = $preferredValues;
        return $this;
    }

    /**
     * @param callable $filterFunction
     * @return $this
     */
    public function setFilterFunction($filterFunction)
    {
        $this->filterFunction = $filterFunction;
        return $this;
    }


    /**
     * @param string $translationPrefix     Prefix retezcu pro preklady
     * @param int $translationValueIndex    Index pole v SQL dotazu, jehoz hodnota se pripoji k prefixu
     * @return $this
     */
    public function setTranslationPrefix($translationPrefix, $translationValueIndex = 0)
    {
        $this->translationPrefix = $translationPrefix;
        if (isset($translationValueIndex)) {
            $this->translationValueIndex = $translationValueIndex;
        }
        return $this;
    }

    /**
     * @param string $titleTranslationPrefix
     * @return $this
     */
    public function setTitleTranslationPrefix($titleTranslationPrefix)
    {
        $this->titleTranslationPrefix = $titleTranslationPrefix;
        return $this;
    }

    /**
     * @param int $translationValueIndex
     * @return $this
     */
    public function setTranslationValueIndex($translationValueIndex)
    {
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }




    protected function createItemObject($key, $value)
    {
        return new Option($key, $value);
    }


    protected function createItem($row)
    {
        $value = (isset($this ->translationPrefix)  &&  isset($row[$this ->translationValueIndex]))
            ? TextManager::_($this ->translationPrefix . $row[$this ->translationValueIndex], false)
            : null;

        if (!isset($value)) {
            $value = $row[$this ->captionIndex];
        }

        $option = $this ->createItemObject($row[$this ->keyIndex], $value);

        if (isset($this ->titleIndex)) {
            $title = (isset($this ->titleTranslationPrefix)  &&  isset($row[$this ->translationValueIndex]))
                ? TextManager::_($this ->titleTranslationPrefix . $row[$this ->translationValueIndex], false)
                : null;

            $option ->title = isset($title) ? $title : $row[$this ->titleIndex];
        }

        if (isset($this ->cssClassIndex)) {
            $option ->cssClass = $row[$this ->cssClassIndex];
        }

        if ($this ->attrIndexes) {
            foreach ($this ->attrIndexes as $dataParamName => $rowIndex) {
                $format = null;
                if ($pos = strpos($dataParamName, ':')) {
                    $format = substr($dataParamName, $pos + 1);
                    $dataParamName = substr($dataParamName, 0, $pos);
                }

                $option->params[$dataParamName] = $format
                    ? StringUtils::format($format, $row[$rowIndex])
                    : $row[$rowIndex];
            }

        }

        return $option;
    }


    public function loadData()
    {
        $result = [];

        if ($this ->preferredValues) {
            $preferredOptions = array_fill_keys($this ->preferredValues, false);
        } else {
            $preferredOptions = null;
        }

        if (isset($this ->emptyValue)) {
            // prida polozku pro "emptyValue"
            $emptyOption = $this ->createItemObject('', $this ->emptyValue);
            if (!$this ->preferredValues) {
                // neexistuji preddefinovane hodnoty - prida ji rovnou do vysledku
                $result[] = $emptyOption;
            } else {
                // doplni prazdnou hodnotu na zacatek preddefinovanych hodnot
                array_splice($preferredOptions, 0, 0, [$emptyOption]);
                //$preferredOptions[] = $emptyOption;
            }
        }

        $cursor = $this ->database ->getSqlResult($this ->sql, $this ->sqlParams);

        while ($row = $cursor ->fetch()) {
            $filterFunction = $this->filterFunction;
            if (!$filterFunction  ||  $filterFunction($row)) {
                $newOption = $this ->createItem($row);

                if ($this ->preferredValues && in_array($newOption->key, $this ->preferredValues)) {
                    $preferredOptions[$newOption->key] = $newOption;
                } else {
                    $result[] = $newOption;
                }
            }
        }

        $cursor ->close();

        if ($preferredOptions) {
            // spoji pole $preferredValues a $result
            return array_merge(
                array_values(array_filter($preferredOptions)), // odstrani nenalezene polozky a vyrobi standardni indexovane pole
                $result);
        }

        return $result;
    }
}
