<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

use Exception;
use Ron\DebugException;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Trida slouzi pro vkladani, editaci a mazani zaznamu v databazi. Automaticky generuje SQL dotazy a take texty pro zapis do logu.
 * Implementuje rozhrani ArrayAccess, aby bylo mozne pristupovat k jednotlivym polim pomoci zapisu $record['JmenoPole']
 */
class DBRecord implements \ArrayAccess, \IteratorAggregate
{
    /** @var Database */
    public $database;

    /**
     * Jmeno DB tabulky
     * @var string
     */
    public $tableName;

    /**
     * Podminka, ktera vybira editovany zaznam (pri select, update a delete).
     * @var string
     * Musi vybrat prave 1 zaznam! (to lze zmenit nastavenim priznaku f_check_multiple = false)
     */
    public $where;

    /**
     * SQL parametry pro where podminku
     * @var ?Parameters
     */
    public $whereParams;

    /**
     * Seznam databazovych poli.
     * @var DBRecordField[] $fields
     */
    public $fields;

    /**
     * Nastavi se v UpdateRecord - na true, pokud byl zaznam uspesne zmenen, jinak false
     * a v InsertRecord - na true v pripade uspesneho vlozeni
     * @var bool
     */
    public $recordUpdated;

    /**
     * Kontrola v LoadRecord, jestli neexistuje vice zaznamu splnujicich podminku where - defaultne zapnuta
     * @var boolean
     */
    public $checkMultiple;

    /**
     * ORDER BY cast SQL dotazu pro nacteni zaznamu - ma smysl jen kdyz je povoleno nacteni/editace vice zaznamu (nastavenim $checkMultiple=true).
     * @var string
     */
    public $orderBy;

    /**
     * Filtr, kterym byl nacten zaznam. Pokud je check_multiple true, tak pro update musi byt pouzit stejny filtr!
     * @var string
     */
    public $loadedFilter;

    /**
     * Nactena data z databaze (pomoci LoadRecords())
     * @var Cursor
     */
    protected $cursor;

    /**
     * Doplnujici text pro zapis do logu
     * @var string
     */
    public $logInfoMessage;

    public $lastError;
    public $lastErrorDescription;


    /**
     * @param Database $database
     * @param string $tableName     Jmeno databazove tabulky
     */
    public function __construct($database, $tableName)
    {
        $this->database = $database;

        $this->tableName = $tableName;
        $this->where = '';

        $this->fields = [];

        $this->checkMultiple = true;

        $this->lastError = null;
        $this->lastErrorDescription = null;
    }


    /**
     * Prida zadane pole do seznamu poli ($fields)
     * @param DBRecordField $field
     * @param ?int $index   Pokud je zadan index, tak vlozi pole na zadany index, jinak ho prida na konec
     * @return DBRecord     Vraci $this pro moznost zretezeni
     */
    public function addDBRecordField($field, $index = null)
    {
        if (!$field) {
            return null;
        }

        $field ->record = $this;

        if (isset($index)) {
            array_splice($this->fields, $index, 0, [$field]);
        } else {
            $this->fields[] = $field;
        }

        return $this;
    }


    /**
     * Prida pole
     * @param string $name
     * @param string $format
     * @param bool $readOnly
     * @param int $size
     * @param $value mixed
     * @return DBRecordField
     */
    public function addField($name, $format, $readOnly = false, $size = 0, $value = null)
    {
        $field = new DBRecordField($name, $format, $readOnly, $size);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida retezcove pole
     * @param string $name
     * @param bool $readOnly
     * @param int $size
     * @param $value mixed
     * @return DBRecordField
     */
    public function addStringField($name, $readOnly = false, $size = 0, $value = null)
    {
        $field = new DBRecordField($name, '', $readOnly, $size);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida integer pole
     * @param string $name
     * @param bool $readOnly
     * @param int|null $value
     * @return DBRecordField
     */
    public function addIntField($name, $readOnly = false, $value = null)
    {
        $field = new DBRecordField($name, 'int', $readOnly);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida float pole
     * @param string $name
     * @param bool $readOnly
     * @param float|null $value
     * @return DBRecordField
     */
    public function addFloatField($name, $readOnly = false, $value = null)
    {
        $field = new DBRecordField($name, 'float', $readOnly);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida timestamp pole
     * @param string $name
     * @param bool $readOnly
     * @param int|null $value
     * @return DBRecordField
     */
    public function addTimestampField($name, $readOnly = false, $value = null)
    {
        $field = new DBRecordField($name, 'timestamp', $readOnly);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida datumove pole
     * @param string $name
     * @param bool $readOnly
     * @param int|null $value
     * @return DBRecordField
     */
    public function addDateField($name, $readOnly = false, $value = null)
    {
        $field = new DBRecordField($name, 'date', $readOnly);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Prida memo pole
     * @param string $name
     * @param bool $readOnly
     * @param int|null $value
     * @return DBRecordField
     */
    public function addMemoField($name, $readOnly = false, $value = null)
    {
        $field = new DBRecordField($name, 'memo', $readOnly);
        $field->setValue($value);

        $this->addDBRecordField($field);

        return $field;
    }


    /**
     * Nastavi where podminku a jeji parametry.
     * @param string|SqlStatement $whereSql
     * @param Parameters [$params]
     * @return DBRecord
     */
    public function setWhere($whereSql, $params = null)
    {
        if ($whereSql instanceof SqlStatement) {
            $this ->where = $whereSql ->sqlText;
            $this ->whereParams = $whereSql ->parameters;
        } else {
            $this->where = $whereSql;
            $this->whereParams = $params;
        }

        return $this;
    }

    /**
     * Prida textove pole slouzici jen pro zapis do logu
     * @param string $text
     * @return DBRecordField
     */
    public function addLogText($text)
    {
        $field = new DBRecordField('', '', false, 0);
        $field ->setValue($text);

        $this ->addDBRecordField($field);
        return $field;
    }


    /**
     * Vraci pole se zadanym jmenem. V pripade, ze jej nenajde, vraci null.
     * @param string $name
     * @return DBRecordField|null
     */
    public function findFieldByName($name)
    {
        foreach ($this->fields as $field) {
            if (!strcasecmp($field->fieldName ?? '', $name)) {
                return $field;
            }
            if (!strcasecmp('#' . $field->fieldName, $name)) {
                return $field; // # se pouziva v XML pro puvodni hodnotu (original value)
            }

            // jeste zkusi porovnat jmeno pole bez pripadneho prefixu JmenoTabulky.
            $pomFieldName = StringUtils::strAfterLast($field->fieldName, '.');
            if ($pomFieldName !== $field->fieldName) {
                if (!strcasecmp($pomFieldName, $name)) {
                    return $field;
                }
                if (!strcasecmp('#' . $pomFieldName, $name)) {
                    return $field;
                }
            }
        }

        return null;
    }


    /**
     * @param mixed $offset
     * @return bool
     */
    public function offsetExists($offset): bool
    {
        if ($offset[0] === '#') {
            $offset = substr($offset, 1); // # se pouziva v XML pro puvodni hodnotu
        }

        return (bool)$this->findFieldByName($offset);
    }


    /**
     * @param mixed $offset
     * @return mixed
     */
    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze prozatim nastavit typy
    public function offsetGet($offset)
    {
        $originalValue = false;

        if ($offset[0] === '#') {
            $offset = substr($offset, 1);
            $originalValue = true;
        }

        $field = $this->findFieldByName($offset);
        if (!$field) {
            return null;
        }

        return $originalValue ? $field->originalValue : $field->value;
    }

    /**
     * @param mixed $offset
     * @param mixed $value
     */
    public function offsetSet($offset, $value): void
    {
        $field = $this->findFieldByName($offset);
        if (!$field) {
            return;
        }

        $field->setNewValue($value);
    }

    /**
     * @param mixed $offset
     */
    public function offsetUnset($offset): void
    {
        $this->offsetSet($offset, null);
    }


    public function getIterator(): \Traversable
    {
        return new DBRecordIterator($this);
    }

    /**
     * @return bool Vraci true, v pripade, ze pri posledni operaci loadRecord(s) doslo k chybe
     */
    public function isError()
    {
        return isset($this->lastError);
    }


    /**
     * Nacte hodnoty poli z radku nacteneho z databaze
     * @param array $row
     * @return void
     */
    public function loadRecordValues($row)
    {
        $index = 0;
        foreach ($this->fields as $field) {
            if ($field->fieldName) {
                $field->originalValue = $field->getValueFromSQLData($row[$index]);

                if ($field->readOnly) {
                    $field->value = $field->originalValue;
                } else if (!$field ->changeable  &&  isset($field ->originalValue)) {
                    $field ->value = $field ->originalValue;
                }

                $index++;
            }
        }
    }


    /**
     * Vraci jmena poli oddelena carkami pro vlozeni do SELECT casti SQL vyrazu
     * @return string
     */
    public function getSelectPartSQL()
    {
        $sql = '';

        foreach ($this->fields as $field) {
            if ($field->fieldName) { // ignoruje pole bez nazvu (napr. kvuli logovani)
                $sql .=
                    ($sql ? ', ' : '') .
                    (($field->format === 'memo')
                        ? $this->database->castWideMemo($field->fieldName)
                        : $field->fieldName
                    );
            }
        }

        return $sql;
    }


    /**
     * Vraci WHERE cast SQL dotazu pro nacteni, editaci nebo smazani zaznamu
     * @param ?string $filtr                SQL podminka, ktera se navic pripoji k $this ->where
     * @param Parameters $targetParams      Do $targetParams se pripoji parametry z whereParams a $filtrParameters
     * @param Parameters $filtrParameters   SQL parametry pro podminku $filtr
     * @return string
     * @throws Exception
     */
    public function getWherePartSQL($filtr, $targetParams = null, $filtrParameters = null)
    {
        $where = $this->where;
        if ($this ->whereParams) {
            if (!$targetParams) {
                throw new Exception('Chyba v DBRecord::getWherePartSQL - nejsou vytvoreny parametry!');
            }
            $targetParams ->appendParameters($this ->whereParams);
        }

        if ($filtr) {
            if ($where) {
                $where = '(' . $where . ') AND (' . $filtr . ')';
            } else {
                $where = $filtr;
            }

            if ($filtrParameters) {
                if (!$targetParams) {
                    throw new Exception('Chyba v DBRecord::getWherePartSQL: nelze pouzit $filtrParameters - nejsou vytvoreny parametry!');
                }
                $targetParams ->appendParameters($filtrParameters);
            }
        }

        return $where;
    }


    /**
     * Sestavi SQL vyraz pro nacteni zpracovavaneho zaznamu
     * @param ?string $filtr            SQL podminka, ktera se navic pripoji k $this ->where
     * @param Parameters $targetParams  Do $targetParams se pripoji parametry z whereParams a $filtrParameters
     * @param Parameters $filtrParams   SQL parametry pro podminku $filtr
     * @return string
     * @throws Exception
     */
    public function getSelectSQL($filtr = '', $targetParams = null, $filtrParams = null)
    {
        $sql =
            'SELECT ' . $this->getSelectPartSQL() .
            "\nFROM " . $this->tableName;

        $where = $this->getWherePartSQL($filtr, $targetParams, $filtrParams);
        if ($where) {
            $sql .= "\nWHERE " . $where;
        }

        // podminka where neni sice povinna, ale vyraz musi vratit jen jeden zaznam!
        // (pokud neni f_check_multiple = false)

        if ($this->orderBy) {
            $sql .= "\nORDER BY " . $this->orderBy;
        }

        return $sql;
    }


    /**
     * Nacte zaznam.
     * Vraci true, pokud byl zaznam nacten, false, kdyz nebyl zadny nalezen
     * @param string $filtr             Nepovinny dodatecny filtr, ktery se prida k podmince where
     * @param ?Parameters $filtrParams  Parametry filtru
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function loadRecord($filtr = '', $filtrParams = null)
    {
        $params = ($this ->whereParams || $filtrParams)
            ? $this ->database ->createParameters()
            : null;

        $sql = $this->getSelectSQL($filtr, $params, $filtrParams);

        $this->lastError = null;
        $this->lastErrorDescription = null;

        MessageManager::addDebugLog("DBRecord.loadRecord: $sql");

        try {
            $cursor = $this->database->getSqlResult($sql, $params, 'Chyba v DBRecord.loadRecord');
        } catch (DBException $e) {
            $this->lastError = 'Chyba v DBRecord.loadRecord (' . $this ->tableName . ')';
            $this->lastErrorDescription = $e ->dbMessage . "\n" . $sql;
            throw $e;
        }

        $row = $cursor->fetch();

        $result = true;

        if (!$row) {
            // zadny nacteny zaznam
            $result = false;
        } else {
            $this->loadRecordValues($row);

            if ($this->checkMultiple) {
                $this->loadedFilter = $filtr; // zapamatuje se filtr - pri update musi byt pouzit stejny

                // jeste overi, jestli nebylo vraceno vice zaznamu
                $row = $cursor->fetch();
                if ($row) {
                    // chyba - neni vybran pouze jeden zaznam!
                    $this->lastError = 'Chyba v DBRecord.loadRecord (' . $this ->tableName . ') - bylo nacteno vice zaznamu!';
                    $this->lastErrorDescription = $sql;

                    $cursor->close();

                    throw new Exception($this->lastError);
                }
            }
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Nacte data pro hromadne prochazeni (pozor! nenacita zadny zaznam! je potreba pouzit Next)
     * @param string $filtr             Nepovinny dodatecny filtr, ktery se prida k podmince where
     * @param ?Parameters $filtrParams  Parametry filtru
     * @throws DBException|Exception
     */
    public function loadRecords($filtr = '', $filtrParams = null)
    {
        $params = ($this ->whereParams || $filtrParams) ? $this ->database ->createParameters() : null;
        $sql = $this->getSelectSQL($filtr, $params, $filtrParams);

        $this->lastError = null;
        $this->lastErrorDescription = null;

        MessageManager::addDebugLog("DBRecord.loadRecords: $sql");

        $this->cursor = $this->database->select($sql, $params);
    }


    /**
     * Nacte dalsi zaznam. Vraci true v pripade uspesneho nacteni, false v pripade chyby
     * a null pokud jiz neni zadny zaznam k nacteni.
     * @param DBRecord $destRecord Pokud neni null, tak data nacita do zaznamu destRecord - $this slouzi pouze jako zdroj dat
     *                               (slouzi predevsim pro nacteni vice zaznamu napr. do pole DBRecord[])
     * @return bool|null
     * @throws Exception
     */
    public function next($destRecord = null)
    {
        if (!$this->cursor) {
            throw new DebugException('Chyba v metode DBRecord.next - nejsou nactena data!');
        }

        $row = $this->cursor->fetch();
        if ($row) {
            if ($destRecord) {
                $destRecord->loadRecordValues($row);
                $destRecord->loadedFilter = '';
            } else {
                $this->loadRecordValues($row);
            }

            return true;
        }

        // zadny dalsi radek
        $this ->cursor ->close();
        return null;
    }


    /**
     * Vraci true, pokud je nektere z datovych poli zmeneno (pole, ktera jsou jen "logovaci" ignoruje)
     * @return bool
     */
    public function isChanged()
    {
        foreach ($this->fields as $field) {
            if ($field->fieldName && (!$field->readOnly) && $field->isChanged()) {
                if (!$field ->changeable  &&  isset($field ->originalValue)) {
                    // pole nelze menit, pokud uz ma zadanou hodnotu
                } else if (!$field->updateLog) {
//                    throw new \Exception("zmena {$field ->fieldName} orig:{$field ->originalValue}-new:{$field->value}-");
                    return true;
                }
            }
        }

        return false;
    }


    /**
     * Funkce je volana po provedeni operace se zaznamem (update/insert/delete) - potomek muze funkci predefinovat
     * @param string $eventType
     * @param string $logText
     */
    public function log($eventType, $logText)
    {
    }


    /**
     * V pripade provedeni editace vraci pocet zmenenych poli, true pokud se zaznam nezmenil a v pripade chyby vyhodi vyjimku
     * @param string $filtr
     * @param Parameters $filtrParams
     * @return bool|int
     * @throws DBException
     * @throws Exception
     */
    public function updateRecord($filtr = '', $filtrParams = null)
    {
        if ($this->checkMultiple  &&  $this->loadedFilter  &&  $this->loadedFilter != $filtr) {
            // kontrola, jestli byl zadan stejny filtr jako pri load - zabrani update jinych zaznamu
            // nez byly skutecne nacteny
            throw new Exception('Chyba v UpdateRecord - neni zadan stejny filtr jako v Load()!');
        }

        $this->recordUpdated = false;

        // v pripade zmeny hodnoty nektereho pole provede update
        // zaznam uz musi byt nacten a nastaveny nove hodnoty!
        $sql = '';

        $logStr = '';
        $do_update = false;

        $cnt = 0;

        $params = $this ->database ->createParameters();

        foreach ($this->fields as $field) {
            $newValueLog = '';

            if ($field->maxLogLength) {
                $newValueLog = $field->getLogValue();

                $logStr .=
                    ($field->fieldName ? $field->fieldName . ': ' : '') .
                    $newValueLog;
            }

            if ($field->fieldName && (!$field->readOnly) && $field->isChanged()) {
                if (!$field->updateLog) {
                    // doslo ke zmene hodnoty pole - provede se update
                    // (ale logovaci pole jako napr. Update_UserName zde ignoruje)
                    $do_update = true;
                }

                $sql .=
                    ($sql ? ', ' : '') .
                    $field ->fieldName . " = " . $params ->addParameter($field ->getSQLParam());

                $cnt++;

                if ($field->maxLogLength) {
                    if (strlen($newValueLog) > 80) {
                        $logStr .= "\n";
                    }
                    $logStr .=
                        ' <old: ' . (isset($field->originalValue) ? StringUtils::substr($field->formatValue($field->originalValue), 0, $field->maxLogLength) : '(NULL)') . '>';
                }
            }

            if ($field->maxLogLength) {
                $logStr .= "\n";
            }
        }

        if ($this->logInfoMessage) {
            $logStr .= "\n{$this ->logInfoMessage}\n";
        }

        if ($do_update) {
            // doslo ke zmene alespon u jednoho pole
            $sql = 'UPDATE ' . $this->tableName . "\nSET " . $sql;

            $where = $this->getWherePartSQL($filtr, $params, $filtrParams);
            if ($where) {
                $sql .= "\nWHERE " . $where;
            }

            if (!$where  &&  $this ->checkMultiple) {
                throw new Exception('Chyba v UpdateRecord - neni zadana WHERE podminka!');
            }
        } else {
            // neni co updatovat
            return true;
        }

        MessageManager::addDebugLog("UpdateRecord: $sql");

        try {
            $this->database->sql($sql, $params);
        } catch (DBException $e) {
            $this->lastError = 'Chyba v DBRecord.updateRecords';
            $this->lastErrorDescription = $e ->dbMessage;

            $this->log('ERROR', "Chyba pri ulozeni editovaneho zaznamu!\n$logStr\n{$e ->dbMessage}\n\n$sql");

            throw new DBException($this->database, $sql, 'Chyba v DBRecord.updateRecords', null, $e ->dbMessage);
        }

        $this->recordUpdated = true;

        $this->log('UPDATE', $logStr . "\n" . $sql);
        return $cnt;
    }


    /**
     * @param string $filtr
     * @return bool|int
     * @throws DBException
     * @throws Exception
     */
    public function loadAndUpdateRecord($filtr = '')
    {
        if ($this->loadRecord($filtr)) {
            return $this->updateRecord($filtr);
        }

        throw new Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
    }


    /**
     * @param DBRecordField $field
     * @returns int
     * @throws DBException
     */
    public function getNextSerial(DBRecordField $field)
    {
        //$params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult('SELECT MAX(' . $field ->fieldName . ') FROM ' . $this ->tableName);
        $row = $cursor ->fetch();
        $cursor ->close();

        return ($row && $row[0]) ? $row[0] + 1 : 1;
    }


    /**
     * @param DBRecordField $field
     * @return int
     */
    public function getNextSequenceValue(DBRecordField $field)
    {
        return SQLUtils::nextSequenceValue($this ->database, $field ->sequenceName ?: 'SEQ_' . $this ->tableName);
    }


    /**
     * @param DBRecordField $field
     * @return int
     * @throws DBException
     */
    public function getNextSequenceOrSerial(DBRecordField $field)
    {
        if ($this ->database ->sequenceExists('SEQ_' . $this ->tableName)) {
            if ($GLOBALS['development'] ?? false) {
                throw new DebugException("Development mode: pro pole {$field ->fieldName} v tabulce {$this ->tableName} existuje sekvence! Pouzijte v XML formulari format sequence misto serial!");
            }
            return $this ->getNextSequenceValue($field);
        }

        return $this ->getNextSerial($field);
    }


    /**
     * Vlozi zaznam.
     * @return void
     * @throws DBException|Exception
     */
    public function insertRecord()
    {
        $sql =
            'INSERT INTO ' . $this->tableName . "\n(";
        $logStr = '';

        $first = true;
        foreach ($this->fields as $field) {
            if ($field->fieldName && !$field->noInsert) { // pole s priznakem noinsert (napr. Update_UserName) ignoruje
                $sql .=
                    ($first ? '' : ', ') .
                    $field->fieldName;

                $first = false;
            }
        }

        $sql .= ")\nVALUES\n(";

        $params = $this ->database ->createParameters();

        $first = true;
        foreach ($this->fields as $field) {
            if (!$field->noInsert) {
                if ($field ->format === 'serial') {
                    $field ->setValue($this ->getNextSerial($field));
                } else if ($field ->format === 'sequence') {
                    $field ->setValue($this ->getNextSequenceValue($field));
                } else if ($field ->format === 'autoserial') {
                    $field ->setValue($this ->getNextSequenceOrSerial($field));
                }

                if ($field->maxLogLength) {
                    $logStr .=
                        ($field->fieldName ? $field->fieldName . ': ' : '') . // loguje i pole bez jmena
                        $field->getLogValue() . "\n";
                }

                if ($field->fieldName) {
                    $sql .=
                        ($first ? '' : ', ') .
                        $params ->addParameter($field ->getSQLParam());
                    $first = false;
                }
            }
        }

        if ($this->logInfoMessage) {
            $logStr .= "\n{$this ->logInfoMessage}\n";
        }

        $sql .= ')';

        MessageManager::addDebugLog($sql);

        try {
            $this->database->sql($sql, $params);
        } catch (DBException $e) {
            $this->lastError = 'Chyba v DBRecord.insertRecord';
            $this->lastErrorDescription = $e ->dbMessage ?: $e ->getMessage();

            $this->log('ERROR', "Chyba pri vlozeni noveho zaznamu!\n$logStr\n{$this->lastErrorDescription}\n\n$sql");

            if ($e ->isPrimaryKeyViolation()) {
                throw new DBException($e, $sql, TextManager::_('Zaznam_s_kodem_uz_existuje'), $e, $this->lastErrorDescription);
            }
            throw new DBException($e, $sql, 'Chyba v DBRecord::insertRecord', $e, $this->lastErrorDescription);
            //throw new DBException($this->database, $sql, 'Chyba v DBRecord::insertRecord', null, 'x ' . $e ->dbMessage);
        }

        $this->recordUpdated = true;

        $this->log('INSERT', $logStr . "\n" . $sql);
    }


    /**
     * @param string $filtr             Nepovinny dodatecny filtr, ktery se prida k podmince where
     * @param Parameters $filtrParams   Parametry filtru
     * @return void
     * @throws DBException
     * @throws Exception
     */
    public function updateOrInsert($filtr = '', $filtrParams = null)
    {
        if ($this->loadRecord($filtr, $filtrParams)) {
            $this->updateRecord($filtr, $filtrParams);
        } else {
            $this->insertRecord();
        }
    }


    /**
     * Smaze zaznam - ten uz musi byt nacteny napr. pomoci loadRecord!
     * @param string $filtr
     * @param Parameters [$filtrParams]
     * @throws DBException|Exception
     */
    public function doDelete($filtr, $filtrParams = null)
    {
        // pokud jsou zadana pole, tak zaloguje hodnoty
        $logStr = '';

        foreach ($this->fields as $field) {
            if ($field->fieldName) {
                $logStr .= $field->fieldName . ': ' . $field->formatValue($field->originalValue) . "\n";
            } else {
                $logStr .= $field->value . "\n";
            }
        }

        if ($this->logInfoMessage) {
            $logStr .= "\n{$this ->logInfoMessage}\n";
        }

        if ($logStr) {
            $logStr .= "\n";
        }

        $params = ($filtrParams || $this ->whereParams) ? $this ->database ->createParameters() : null;

        // smaze zaznam
        $sql = 'DELETE FROM ' . $this->tableName;

        $where = $this->getWherePartSQL($filtr, $params, $filtrParams);
        if ($where) {
            $sql .= "\nWHERE " . $where;
        }

        MessageManager::addDebugLog("DeleteRecord: $sql");

        try {
            $this->database->sql($sql, $params);
            $this->log('DELETE', $logStr . $sql);
        } catch (DBException $e) {
            $this->log('ERROR', "Chyba pri ruseni zaznamu!\n$logStr\n{$e ->getMessage()}\n\n$sql");
            $this->lastError = "Chyba pri ruseni zaznamu!";
            $this->lastErrorDescription = $e ->getMessage();

            if ($e ->isForeignKeyViolation()) {
                throw new DBException($e, $sql, TextManager::_('Zaznam_nelze_smazat_existuje_podrizeny'), $e, $e ->dbMessage);
            }

            throw $e;
        }
    }


    /**
     * Smaze zaznam. Vraci true, pokud byl smazan, false pokud nebyl nalezen a v pripade chyby vyhodi vyjimku
     * @param string $filtr
     * @param Parameters [$filtrParams]
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function deleteRecord($filtr = '', $filtrParams = null)
    {
        if (!$this->loadRecord($filtr, $filtrParams)) {
            return false;
        }

        $this->doDelete($filtr, $filtrParams);
        return true;
    }


    /**
     * Hromadne smazani zaznamu. Zaloguje smazane zaznamy a vraci jejich pocet.
     * @param string $filtr
     * @param Parameters $filtrParams
     * @return int      Vraci pocet smazanych zaznamu
     * @throws DBException
     * @throws Exception
     */
    public function deleteRecords($filtr = '', $filtrParams = null)
    {
        $params = ($this ->whereParams || $filtrParams)
            ? $this ->database ->createParameters()
            : null;

        $sql = $this->getSelectSQL($filtr, $params, $filtrParams);

        // pokud jsou zadana pole, tak zaloguje hodnoty
        $logStr = '';

        $cursor = $this->database->select($sql, $params);
        if (!$cursor) {
            $this->lastError = 'Chyba v DBRecord.deleteRecords';
            $dbErrorText = $this->database->lastErrorText();
            $this->lastErrorDescription = $dbErrorText . "\n" . $sql;

            throw new DBException($this->database, $sql, 'Chyba v DBRecord.deleteRecords!', null, $dbErrorText);
        }

        $result = 0;

        while ($row = $cursor->fetch()) {
            $this->loadRecordValues($row);

            foreach ($this->fields as $field) {
                $logStr .= $field->fieldName . ': ' . $field->formatValue($field->originalValue) . "\n";
            }
            $logStr .= "\n";

            $result++;
        }

        $cursor ->close();

        if (!$result) {
            // neni co mazat
            return 0;
        }

        $logStr = "Smazani $result zaznamu\n" . $logStr;

        if ($this->logInfoMessage) {
            $logStr .= "\n{$this ->logInfoMessage}\n";
        }

        if ($logStr) {
            $logStr .= "\n";
        }

        // smaze zaznamy
        $params = ($this ->whereParams  ||  $filtrParams)
            ? $this ->database ->createParameters()
            : null;

        $sql = 'DELETE FROM ' . $this->tableName;

        $where = $this->getWherePartSQL($filtr, $params, $filtrParams);
        if ($where) {
            $sql .= ' WHERE ' . $where;
        }

        MessageManager::addDebugLog("DeleteRecord: $sql");

        try {
            $this->database->sql($sql, $params);
            $this->log('DELETE', $logStr . $sql);
            return $result;
        } catch (DBException $e) {
            $errorText = $this->database->lastErrorText();

            $this->log('ERROR', "Chyba pri ruseni zaznamu!" . "\n$logStr\n{$e ->getMessage()}\n\n{$e ->sql}");

            MessageManager::addDebugLog($errorText);

            throw $e;
        } catch (Exception $e) {
            $this->log('ERROR', "Chyba pri ruseni zaznamu!" . "\n$logStr\n{$e ->getMessage()}");

            MessageManager::addDebugLog($e ->getMessage());

            throw $e;
        }

    }

}
