<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace Ron\DB;


class OdbcDatabase extends Database
{

    /**
     * @var resource
     */
    private $connection;

    private $connectionString;

    private $noRecompile;
    private $persistentConnection;

    public $convertVarBinary = true;


    public function __construct($connectionString)
    {
        $this ->connectionString = $connectionString;
        $this ->functions = [
            'EXTRACTDATE' => 'CONVERT(DATE, ?)',
            'EXTRACTTIME' => 'CONVERT(TIME, ?)',
        ];
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getNoRecompile()
    {
        return $this->noRecompile;
    }

    /**
     * @param mixed $noRecompile
     */
    public function setNoRecompile($noRecompile)
    {
        $this->noRecompile = $noRecompile;
    }

    /**
     * @return mixed
     */
    public function getPersistentConnection()
    {
        return $this->persistentConnection;
    }

    /**
     * @param mixed $persistentConnection
     */
    public function setPersistentConnection($persistentConnection)
    {
        $this->persistentConnection = $persistentConnection;
    }


    /**
     * @return string
     */
    public function getDbType()
    {
        return 'odbc';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        global $db_odbc_default_lrl;
        if (!$db_odbc_default_lrl) {
            $db_odbc_default_lrl = (int)ini_get('odbc.defaultlrl');

            if ($db_odbc_default_lrl < 1048576) {
                $db_odbc_default_lrl = 1048576;
            }
        }
        ini_set('odbc.defaultlrl', $db_odbc_default_lrl);

        if ($this ->persistentConnection) {
            $this ->connection = odbc_pconnect($this ->connectionString, $user, $password);
        } else {
            $this ->connection = odbc_connect($this ->connectionString, $user, $password); // , SQL_CUR_USE_ODBC - haze chyby, nepouzivat
        }

        if (!$this ->connection) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }
    }

    /**
     * @return bool
     */
    public function close()
    {
        odbc_close($this ->connection);
        return true;
    }


    public function createParameters()
    {
        return $this ->useAdHocParams
            ? new AdHocParameters($this)
            : new PlaceholderParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params  &&  !$this ->useAdHocParams) {
            //$sqlText .= ' OPTION(RECOMPILE);'; - jen test
            //echo "<div>$sqlText</div>";
            $data = odbc_prepare($this ->connection, $sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!odbc_execute($data, $params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $recompileStr = '';

            if (substr($sqlText, -1) != ';') {
                $recompileStr = ($this->noRecompile) ? ';' : ' OPTION(RECOMPILE);';
            }

            $data = odbc_exec($this->connection, $sqlText . $recompileStr);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }

        return new OdbcCursor($this, $data);
    }

    /**
     * @param string $sqlText
     * @param Parameters $params
     * @param bool $noRecompile
     * @throws DBException
     */
    private function sqlInternal($sqlText, $params = null, $noRecompile = false)
    {
        if ($params  &&  !$this ->useAdHocParams) {
            $data = odbc_prepare($this ->connection, $sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!odbc_execute($data, $params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $recompileStr = '';

            if (substr($sqlText, -1) != ';') {
                $recompileStr = ($noRecompile || $this->noRecompile) ? ';' : ' OPTION(RECOMPILE);';
            }

            $data = odbc_exec($this->connection, $sqlText . $recompileStr);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }
    }

    /**
     * @param string $sqlText
     * @param Parameters $params
     */
    public function executeSql($sqlText, $params = null)
    {
        $this ->sqlInternal($sqlText, $params, false);
    }

    /**
     * @return string
     */
    public function lastErrorText()
    {
        if (!$this ->connection) return odbc_errormsg();

        return odbc_errormsg($this ->connection);
    }


    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        $this ->sqlInternal('BEGIN TRAN;', null, true);
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        $this ->sqlInternal('COMMIT TRAN;', null, true);
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        $this ->sqlInternal('ROLLBACK TRAN;', null, true);
    }





    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string
     */
    public function castWideMemo($text)
    {
        // return $text;
        return "CAST($text AS TEXT)";
        // upraveno 4.6.2012 - hlasilo to chybu na linuxu:
        // Unicode data in a Unicode-only collation or ntext data cannot be sent to clients using DB-Library (such as ISQL) or ODBC version 3.7 or earlier., SQL state 37000 in SQLExecDirect
    }

    /**
     * @param string $text
     * @return string
     */
    public function extractYear($text)
    {
        return "YEAR($text)";
    }

    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if (!isset($str)) return null;

        $onlyTime = strstr($str, '1899') !== false;

//        if (preg_match ("/2(\d\d\d)/", $str, $m) && ($m[1] >= 38)) {
//            // maximalni hodnota timestampu je do roku 2038 !
//            return null;
//        }

        if (!$onlyTime) {
            $result = strtotime($str);
            if ($result != -1) {
                return $result; // - $basetime;
            }
        }

        $pomarray = explode(" ", $str); // rozdeli datum a cas
        $datearray = $onlyTime ? [1970, 1, 1] : explode("-", $pomarray[0]); // rozdeli slozky data
        $timearray1 = explode(".", $pomarray[1]); // oddeli setiny od casu
        $timearray = explode(":", $timearray1[0]); // rozdeli slozky casu

        $result = mktime($timearray[0], $timearray[1], $timearray[2], $datearray[1], $datearray[2], $datearray[0]);

        return $result; //  - $basetime;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "{ts '" . date('Y-m-d H:i:s', $timestamp) . "'}";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';
        return "{d '".date('Y-m-d', $timestamp)."'}";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = (int)($timestamp / 3600);
            $timestamp = $timestamp - 3600 * $h;
            $m = (int)($timestamp / 60);
            $s = (int)$timestamp - 60 * $m;
            return "{ ts '1899-12-30 " . str_pad($h, 2, '0', STR_PAD_LEFT) . ':' . str_pad($m, 2, '0', STR_PAD_LEFT) . ':' . str_pad($s, 2, '0', STR_PAD_LEFT) . "'}";
//            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
//            $timestamp = $timestamp - 3600 * $h;
//            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
//            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
//
//            return "{ ts '1899-12-30 $h:$m:$s'}";
        }

        return "{ ts '1899-12-30 " . date('H:i:s', $timestamp) . "'}";
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('Y-m-d', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()

            $h = (int)($timestamp / 3600);
            $timestamp = $timestamp - 3600 * $h;
            $m = (int)($timestamp / 60);
            $s = (int)$timestamp - 60 * $m;
            return '1899-12-30 ' . str_pad($h, 2, '0', STR_PAD_LEFT) . ':' . str_pad($m, 2, '0', STR_PAD_LEFT) . ':' . str_pad($s, 2, '0', STR_PAD_LEFT);

//            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
//            $timestamp = $timestamp - 3600 * $h;
//            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
//            $s = str_pad((int)$timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
//
//            return "1899-12-30 $h:$m:$s";
        }

        return '1899-12-30 ' . date('H:i:s', $timestamp);
    }


    function getBlobParam($data, $fieldName)
    {
        if ($this ->useAdHocParams) {
            $datastring = unpack("H*hex", $data);
            return new AdHocParameter('0x' . $datastring['hex']);
        }

        if (!$this ->convertVarBinary) {
            return $data;
        }

        return new BlobParameter('CONVERT(varbinary(max),?)', $data);
    }

    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode(':', $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0;
    }


    protected function sequenceExistsInternal(string $sequenceName): bool
    {
        $sql =
            'SELECT name FROM sys.sequences' .
            ' WHERE UPPER(name) = ' . "'" . strtoupper($sequenceName) . "'";

        if ($sql) {
            $cursor = $this ->getSqlResult($sql);
            $row = $cursor ->fetch();
            $cursor ->close();

            return (bool)$row;
        }

        return false;
    }


}


/**
 * Implementace tridy Cursor pro cteni dat z databaze pomoci ODBC driveru.
 */
class OdbcCursor extends Cursor
{
    /**
     * @var resource
     */
    protected $data;


    protected $seekNumber; // help variable for seek()


    /**
     * @param OdbcDatabase $database
     * @param resource $data
     */
    function __construct($database, $data) {
        parent::__construct($database);
        $this ->data = $data;
    }

    /**
     * @return Database
     */
    public function getDatabase()
    {
        return $this->database;
    }

    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        $vysledek = null;

        if (isset($this ->seekNumber) && $this ->seekNumber >= 0) {
            odbc_fetch_into($this ->data, $vysledek, $this ->seekNumber + 1); // cislovani v ODBC je od 1

        } else {
            odbc_fetch_into($this ->data, $vysledek);
        }

        $this ->seekNumber = null;

        return $vysledek ?: null;   // aby nevracel prazdne pole, kdyz nenacte zadny radek
    }


//    public function fetchLongData()
//    {
//        ini_set('odbc.defaultlrl', '10485760'); // TAKTO TO NEJDE - musi byt pred select!
//        return $this ->fetch();
//
//        odbc_binmode($this ->data, ODBC_BINMODE_RETURN);
//        odbc_longreadlen($this ->data, 4096); // 65536);
//
//        if (isset($this ->seekNumber) && $this ->seekNumber >= 0) {
//            $fetchOK = odbc_fetch_row($this ->data, $this ->seekNumber);
//        } else {
//            $fetchOK = odbc_fetch_row($this ->data);
//        }
//
//        if (!$fetchOK) return null;
//
//        $result = [];
//
//        for ($i = 1;  $i <= odbc_num_fields($this ->data);  $i++) {
//            $val = odbc_result($this ->data, $i);
//
//            $max = 2;
//
//            if ( isset($val)  &&  $val != "") {
//                var_dump($val);
//                while ( $max  &&  ($chunk = odbc_result($this ->data, $i) != "") ) {
//                    var_dump($chunk);
//                    $val .= $chunk;
//                    $max--;
//                }
//            }
//            $result[] = $val;
//        }
//
//        return $result;
//    }


    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows()
    {
        return odbc_num_rows($this ->data);
    }

    /**
     * @param int $rowNumber
     * @return bool
     */
    public function seek($rowNumber)
    {
        $this ->seekNumber = $rowNumber;

        return true;
    }

    /**
     * @return bool
     */
    public function last()
    {
        $cnt = odbc_num_rows($this ->data);

        return $this ->seek($cnt - 1);
    }

}
