<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace Ron\DB;

class SqlSrvDatabase extends Database
{

    /**
     * @var resource
     */
    private $connection;

    private $connectionString;
    private $dbName;

    private $charSet;

    private $noRecompile;
    private $convertTimestamp;

    /** @var array[]    Asociativni pole voleb pro funkci sqlsrv_connect */
    private $connectionOptions = [];


    public function __construct($connectionString, $dbName)
    {
        $this->connectionString = $connectionString;
        $this->dbName = $dbName;
        $this ->functions = [
            'EXTRACTDATE' => 'CONVERT(DATE, ?)',
            'EXTRACTTIME' => 'CONVERT(TIME, ?)',
        ];
    }

    /**
     * @return resource
     */
    public function getConnection()
    {
        return $this->connection;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getDbName()
    {
        return $this->dbName;
    }


    /**
     * @return mixed
     */
    public function getCharSet()
    {
        return $this->charSet;
    }

    /**
     * @param mixed $charSet
     * @return $this
     */
    public function setCharSet($charSet)
    {
        $this->charSet = $charSet;
        return $this;
    }


    /**
     * @return mixed
     */
    public function getNoRecompile()
    {
        return $this->noRecompile;
    }

    /**
     * @param mixed $noRecompile
     * @return $this
     */
    public function setNoRecompile($noRecompile)
    {
        $this->noRecompile = $noRecompile;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getConvertTimestamp()
    {
        return $this->convertTimestamp;
    }

    /**
     * @param mixed $convertTimestamp
     * @return $this
     */
    public function setConvertTimestamp($convertTimestamp)
    {
        $this->convertTimestamp = $convertTimestamp;
        return $this;
    }


    /**
     * @return \array[]
     */
    public function getConnectionOptions()
    {
        return $this->connectionOptions;
    }

    /**
     * @param \array[] $connectionOptions
     * @return $this
     */
    public function setConnectionOptions($connectionOptions)
    {
        $this->connectionOptions = $connectionOptions;
        return $this;
    }




    /**
     * @return string
     */
    public function getDbType()
    {
        return 'sqlsrv';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        $options = $this ->connectionOptions;

        $options['Database'] = $this->dbName;
        // Encrypt => 1, TrustServerCertificate => 1

        if ( isset($user) ) {   // pro windows authentication nesmi byt jmeno a heslo zadano
            $options['UID'] = $user;
            $options['PWD'] = $password;
        }

        if ($this ->charSet) {
            $options['CharacterSet'] = $this->charSet;  // - jina moznost asi neni
        }

        $this->connection = sqlsrv_connect($this->connectionString, $options);

        if (!$this ->connection) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }
    }

    /**
     * @return bool
     */
    public function close()
    {
        return sqlsrv_close($this->connection);
    }


    public function createParameters()
    {
        if ($this ->useAdHocParams) {
            return new AdHocParameters($this);
        }

        return new PlaceholderParameters($this);
    }


    /**
     * @param $sqlText
     * @param Parameters $params
     * @return bool|resource
     * @throws DBException
     */
    private function executeSqlSrv($sqlText, $params = null)
    {
        if ($params) {
            $data = sqlsrv_query($this->connection,
                $sqlText . (substr($sqlText, -1) != ';' ? ';' : ''),
                $params->getSqlParameters(),
                ['Scrollable' => SQLSRV_CURSOR_STATIC]  // aby fungovalo numrows a seek
            );
        } else {
            $recompileStr = '';

            $sqlText = trim($sqlText);

            if (substr($sqlText, -1) != ';') {
                $recompileStr = ($this->noRecompile) ? ';' : ' OPTION(RECOMPILE);';
            }

            $data = sqlsrv_query($this->connection,
                $sqlText . $recompileStr,
                null,
                ['Scrollable' => SQLSRV_CURSOR_STATIC]  // aby fungovalo numrows a seek
            );
        }

        if (!$data) {
            throw new DBException($this, $sqlText);
            //return false;
        }

        return $data;
    }

    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return resource
     * @throws DBException
     */
    protected function executeSelect($sqlText, $params = null)
    {
        $data = $this ->executeSqlSrv($sqlText, $params);
        return new SqlSrvCursor($this, $data);
    }

    /**
     * @param string $sqlText
     * @param array $params
     * @throws DBException
     */
    protected function executeSql($sqlText, $params = null)
    {
        $this ->executeSqlSrv($sqlText, $params);
    }


    /**
     * @param \DateTime $timestamp
     * @return int|null
     */
    public static function sqlsrvConvertTimestamp($timestamp)
    {
        if (!isset($timestamp)) return NULL;

        if ($timestamp) {
            if ($timestamp->format('Y') == 1899) {
                // jedna se jen o cas - prevede ho
                $timestamp->setDate(1970, 1, 1);
            }

            return (int)$timestamp->format('U');
        } else {
            return 0;
        }
    }


    /**
     * @return string
     */
    public function lastErrorText()
    {
        $result = '';

        $errors = sqlsrv_errors();

        if ($errors) {
            foreach ($errors as $error) {
                $result .= 'SQLSTATE: ' . $error['SQLSTATE'] . '<br/>';
                $result .= 'Code: ' . $error['code'] . '<br/>';
                $result .= 'Message: ' . $error['message'] . '<br/>';
            }
        }

        return $result;
    }


    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        if (!sqlsrv_begin_transaction($this->connection)) {
            throw new DBException($this, 'begin transaction ...');
        };
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        if (!sqlsrv_commit($this->connection)) {
            throw new DBException($this, 'commit transaction ...');
        }
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        if (!sqlsrv_rollback($this->connection)) {
            throw new DBException($this, 'rollback transaction ...');
        }
    }


    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string
     */
    public function castWideMemo($text)
    {
        return "CAST($text AS TEXT)";
    }

    /**
     * @param string $text
     * @return string
     */
    public function extractYear($text)
    {
        return "YEAR($text)";
    }

    /**
     * @param int|\DateTime $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if ($this->getConvertTimestamp()) return $str; // jiz bylo zkonvertovano pri fech_row

        return $this->sqlsrvConvertTimestamp($str);
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';
        return "{ts '" . date('Y-m-d H:i:s', $timestamp) . "'}";
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';
        return "{d '" . date('Y-m-d', $timestamp) . "'}";
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "{ ts '1899-12-30 $h:$m:$s'}";
        }

        return "{ ts '1899-12-30 " . date('H:i:s', $timestamp) . "'}";
    }


    public function timestampToParam($timestamp)
    {
        if ($this ->useAdHocParams) {
            return new AdHocParameter($this ->timestampToString($timestamp));
        }

        if (!isset($timestamp)) {
            return null;
        }

        $value = new \DateTime(date('Y-m-d\TH:i:s', $timestamp), new \DateTimeZone('UTC'));
//        $value = new \DateTime('now', new \DateTimeZone('UTC'));
//        $value ->setTimezone( new \DateTimeZone('UTC') );
//        $value ->setTimestamp($timestamp);

        return $value;
    }

    public function dateToParam($timestamp)
    {
        if ($this ->useAdHocParams) {
            return new AdHocParameter($this ->dateToString($timestamp));
        }

        if (!isset($timestamp)) {
            return null;
        }

        $value = new \DateTime(date('Y-m-d\TH:i:s', $timestamp), new \DateTimeZone('UTC'));
        //$value = new \DateTime();   // toto nefunguje spravne - pravdepodobne problem s casovou zonou
        //$value ->setTimestamp($timestamp);
        $value->setTime(0, 0, 0);

        return $value;
    }

    public function timeToParam($timestamp)
    {
        if (!isset($timestamp)) {
            return null;
        }

        $value = new \DateTime();
        $value->setDate(1899, 12, 30);

        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = (int)($timestamp / 3600);
            $timestamp = $timestamp - 3600 * $h;
            $m = (int)($timestamp / 60);
            $s = $timestamp - 60 * $m;

            $value->setTime($h, $m, $s);
        } else {
            $h = date('H', $timestamp);
            $m = date('i', $timestamp);
            $s = date('s', $timestamp);

            $value->setTime($h, $m, $s);
        }

        return $value;
    }


    function getBlobParam($data, $fieldName)
    {
        //return new BlobParameter('CONVERT(VARBINARY(MAX),?)', $data);
        return new BlobParameter('?', [$data, SQLSRV_PARAM_IN, SQLSRV_PHPTYPE_STRING(SQLSRV_ENC_BINARY), SQLSRV_SQLTYPE_VARBINARY('max')]);
    }


    public function stringToSecondsOfDay($str)
    {
        if (!$str) return null;

        if ($this->getConvertTimestamp()) {
            // $str je uz timestamp (bylo zkonvertovano pri fech_row)
            return date('H', $str) * 3600 + date('i', $str) * 60 + date('s', $str);
        }

        $timestr = date_format($str, 'H:i:s');

        //  $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou
        $poma = explode(':', $timestr);

        $add = ($poma[0] == 12)
            ? (stristr($timestr, 'am') === false ? 0 : -12)
            : (stristr($timestr, 'pm') === false ? 0 : 12);

        return ($poma[0] + $add) * 3600 + $poma[1] * 60 + $poma[2];
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if (!$str) return null;
        return $this ->stringToSecondsOfDay($str) / 3600;
    }


    /**
     * @param string $sequenceName
     * @return bool
     * @throws DBException
     */
    protected function sequenceExistsInternal(string $sequenceName):bool
    {
        $sql =
            'SELECT name FROM sys.sequences' .
            ' WHERE UPPER(name) = ' . "'" . strtoupper($sequenceName) . "'";

        if ($sql) {
            $cursor = $this ->getSqlResult($sql);
            $row = $cursor ->fetch();
            $cursor ->close();

            return (bool)$row;
        }

        return false;
    }
}



class SqlSrvCursor extends Cursor
{
    /**
     * @var resource
     */
    protected $data;

    /** @var integer    Nastavuje se pri seek */
    protected $seekNumber = null;

    /**
     * @param SqlSrvDatabase $database
     * @param resource $data
     */
    function __construct($database, $data)
    {
        parent::__construct($database);
        $this->data = $data;
    }


    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }


    /**
     * @return array|bool
     */
    public function fetch()
    {
        if (isset($this ->seekNumber) && $this ->seekNumber >= 0) {
            $rowData = sqlsrv_fetch_array($this->data, SQLSRV_FETCH_NUMERIC, SQLSRV_SCROLL_ABSOLUTE, $this ->seekNumber);
            $this ->seekNumber = null;
        } else {
            $rowData = sqlsrv_fetch_array($this->data, SQLSRV_FETCH_NUMERIC);
        }

        if ($rowData && $this->database->getConvertTimestamp()) {
            foreach ($rowData as $i => $field) {
                // prevede vsechny datumy na timestamp - jinak se s nima neda procovat v XML ve <forsql>
                //if (is_object($field))
                if ($field instanceof \DateTime) {
                    $rowData[$i] = $this->database->sqlsrvConvertTimestamp($field);
                }
            }
        }

        return $rowData;
    }


    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows()
    {
        return sqlsrv_num_rows($this->data);
    }


    /**
     * @param int $rowNumber
     * @return bool
     */
    public function seek($rowNumber)
    {
        $this ->seekNumber = $rowNumber;

        return true;
    }


}
