<?php
/**
 * UTF8
 * stanislav.elbl, 09.12.2015
 */

namespace Ron;


class FileInfo
{
    /** @var string */
    var $fileName;

    /** @var bool */
    var $isDir;

    /** @var integer */
    public $size;
    /** @var integer */
    public $modificationTime;

    function __construct($fileName, $isDir = false)
    {
        $this ->fileName = $fileName;
        $this ->isDir = $isDir;
    }
}


class FileUtils
{
    /**
     * @param string $dir
     * @param bool $showSubDirs
     * @param bool $loadDetails
     * @return FileInfo[]
     */
    public static function getFileList($dir, $showSubDirs, $loadDetails = false)
    {
        $dirs = [];
        $files = [];

//        if ($showSubDirs) {
//            $dh = opendir($dir);
//
//            while (false !== ($filename = readdir($dh)))
//                if (is_dir($dir . "/" . $filename)) $result[] = $filename;
//        }

        $dh = opendir($dir);

        while (false !== ($fileName = readdir($dh))) {
            $name = StringUtils::fromUTF8($fileName);

            $fullFileName = $dir . "/" . $fileName;

            if (is_dir($fullFileName)) {
                if ($showSubDirs) {
                    $fileInfo = new FileInfo($name, true);
                    if ($loadDetails) {
                        $fileInfo ->modificationTime = filemtime($fullFileName);
                    }
                    $dirs[] = $fileInfo;
                }
            } else {
                $fileInfo = new FileInfo($name);
                if ($loadDetails) {
                    $fileInfo ->modificationTime = filemtime($fullFileName);
                    $fileInfo ->size = filesize($fullFileName);
                }
                $files[] = $fileInfo;
            }
        }

        if ($showSubDirs) {
            return array_merge($dirs, $files);
        } else {
            return $files;
        }
    }


    /**
     * Vraci jedinecne jmeno souboru v zadanem adresari. V pripade potreby prida na konec jmena souboru index.
     * @param string $dir
     * @param string $fileName
     * @param int $startIndex   Pokud je null, tak je povoleno i puvodni jmeno souboru, jinak vzdy pridava index.
     * @return string
     */
    public static function getUniqueFileName($dir, $fileName, $startIndex = null)
    {
        if ( !isset($startIndex) ) {
            if (!file_exists($dir . '/' . $fileName)) {
                return $fileName;
            }
        }

        $ext = pathinfo($fileName, PATHINFO_EXTENSION);
        $baseName = pathinfo($fileName, PATHINFO_FILENAME);

        $index = ($startIndex ?: 2);
        while (file_exists($dir . '/' . $baseName . $index . '.' . $ext)) {
            $index++;
        }

        return $baseName . $index . '.' . $ext;
    }


    /**
     * @param string $fileName
     * @return string
     */
    static function translateFileNameToLink($fileName)
    {
        $result = str_replace('%', '%25', $fileName);
        $result = str_replace('+', '%2B', $result);
        $result = str_replace(' ', '%20', $result);
        $result = str_replace('&', '%26', $result);
        return $result;
    }


    /**
     * Zkrati jmeno souboru, aby nebylo delsi nez zadana delka. Zachova priponu.
     * @param string $fileName
     * @param int    $length
     * @return string
     */
    public static function truncateFileName($fileName, $length)
    {
        if ( strlen($fileName) <= $length )  return $fileName;

        $ext = pathinfo($fileName, PATHINFO_EXTENSION);
        $basename = pathinfo($fileName, PATHINFO_BASENAME);

        if ( strlen($ext) >= $length - 1 )  {
            // moc dlouha pripona - vraci jen jeji zacatek
            return substr($ext, 0, $length);
        }

        return substr($basename, 0, $length - 1 - strlen($ext)) . '.' . $ext;
    }


    /**
     * @param int $size
     * @return string
     */
    public static function getFileSizeText($size)
    {
        if ($size >= 2097152) {
            $velikost = ($size + 524288) / 1048576;
            $jednotka = 'MB';
        } else if ($size >= 2048) {
            $velikost = ($size + 512) / 1024;
            $jednotka = 'kB';
        } else {
            $velikost = $size;
            $jednotka = 'B';
        }

        $velikost = number_format($velikost, 0, ',', ' ');
        return str_replace(' ', '&nbsp;', $velikost) . '&nbsp;' . $jednotka;
    }


    const FILE_TYPE_IMAGE = 'image';
    const FILE_TYPE_DOCUMENT = 'document';
    const FILE_TYPE_VIDEO = 'video';
    const FILE_TYPE_AUDIO = 'audio';
    const FILE_TYPE_UNKNOWN = 'unknown';


    /**
     * @param string $fileName
     * @return string
     */
    public static function getFileType($fileName)
    {
        $ext = pathinfo($fileName, PATHINFO_EXTENSION);
        switch (strtolower($ext)) {
            case 'jpg' :
            case 'jpeg' :
            case 'png' :
            case 'gif' :
            case 'bmp' :
            case 'tif' :
            case 'svg' :
                return self::FILE_TYPE_IMAGE;

            case 'pdf' :
            case 'txt' :
            case 'doc' :
            case 'docx' :
            case 'dotx' :
            case 'xls' :
            case 'xlsx' :
            case 'rtf' :
            case 'csv' :
            case 'pps' :
            case 'ppt' :
            case 'pptx' :
            case 'xml' :
            case 'odt' :
            case 'ods' :
            case 'odp' :
                return self::FILE_TYPE_DOCUMENT;

            case 'aif' :
            case 'm4a' :
            case 'mp3' :
                return self::FILE_TYPE_AUDIO;

            case 'avi' :
            case 'mp4' :
            case 'mpg' :
            case 'wmv' :
                return self::FILE_TYPE_VIDEO;

            default :
                return self::FILE_TYPE_UNKNOWN;
        }
    }


    /**
     * @param string $fileName
     * @param string $defaultIconClass
     * @return string
     */
    public static function getIcon($fileName, $defaultIconClass = '')
    {
        $ext = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));

        switch ($ext) {
            case 'pdf' :
            case 'csv' :
                return 'fa-file-' . $ext;

            case 'doc' :
            case 'docx' :
            case 'dotx' :
                return 'fa-file-word';

            case 'xls' :
            case 'xlsx' :
                return 'fa-file-excel';

            case 'ods' :
                return 'fa-file-spreadsheet';

            case 'ppt' :
            case 'pptx' :
                return 'fa-file-powerpoint';

            case 'jpg' :
            case 'jpeg' :
            case 'gif' :
            case 'png' :
            case 'bmp' :
            case 'tif' :
            case 'svg' :
                return 'fa-image';

            case 'txt' :
            case 'odt' :
                return 'fa-file-alt';

            case 'odp' :
                return 'fa-file-chart-pie';

            case 'mp4' :
            case 'mpg' :
            case 'mpeg' :
            case 'avi' :
            case 'wmv' :
                return 'fa-file-video';

            case 'mp3' :
            case 'aif' :
            case 'm4a' :
                return 'fa-file-music';

            case 'zip' :
            case 'rar' :
            case 'gz' :
            case 'tgz' :
            case 'gzip' :
                return 'fa-file-archive';

            case 'dbs' :
                return 'fa-file-export';

            case 'nes' :
                return 'fa-code';

            case 'rmr' :
                return 'fa-file-chart-line';
        }

        return $defaultIconClass;
    }

}
