<?php
/**
 * User: stanislav.elbl
 * Date: 11.7.15 2:03
 * Zakladni trida  webove aplikace
 */

namespace Ron\Http;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\LogWriter;
use Ron\MessageManager;
use Ron\MVC;
use Ron\MVC\Model;
use Ron\Profiler;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Utils\AppTracyPanel;
use Ron\Utils\ScreenTracyPanel;
use RonPortal\RonPortalSession;
use Tracy\Debugger;


abstract class App
{

    protected $originalLang;
    protected $translationLang;

    protected $sessionName;

    /** @var  Database */
    protected $database;

    protected $defaultView;

    protected static $instance;

    /** @var string     Ladici info - zobrazi se jako nadpis v TracyBaru */
    private $debugInfoCaption = '';
    /** @var string[]   Ladici info - zobrazi se jako panel v TracyBaru */
    private $debugInfo = [];


    public function __construct()
    {
        //error_reporting(E_ALL & ~E_NOTICE);
        global $development;
        if (!$development) {
            error_reporting(E_ERROR);
            //error_reporting(E_ALL & ~E_DEPRECATED & ~E_STRICT & ~E_NOTICE & ~E_WARNING);
        }
        self::$instance = $this;

        global $enableTracy;
        if ($enableTracy) {
            Debugger::enable(Debugger::Detect, 'C:/PHP/log');
            Debugger::getBar()->addPanel(new AppTracyPanel());
            Debugger::getBar()->addPanel(new ScreenTracyPanel());
        }
    }

    /**
     * @return App
     */
    public static function getInstance()
    {
        return self::$instance;
    }


    /**
     * @return LogWriter
     */
    public abstract function getLogWriter() : LogWriter;


    function initStrings()
    {
        $language = '';

        // nastaveni podle aplikace
        if (isset($_SESSION['language'])  &&  $_SESSION['language']) {
            // jazyk uz je nastaveny v session
            $language = $_SESSION['language'];
        } else {
            // jazyk jeste neni nastaven (start aplikace)
            global $g_language;
            if (isset($g_language) && $g_language) {
                $language = $g_language;
            } else if (isset($_COOKIE['ron_language']) && $_COOKIE['ron_language']) {
                $language = $_COOKIE['ron_language'];
            } else {
                global $defaultlang;
                if ($defaultlang) {
                    $language = $defaultlang;
                }
            }
        }

        $langDir = TextManager::getInstance() ->langDir;

        // nastaveni prekladu v url adrese
        if (isset($_GET['session@lang']) && $_GET['session@lang']) {
            if (file_exists($langDir . $_GET['session@lang'])) // overi, jestli uzivatel nezadal blbost
            {
                $language = $_GET['session@lang'];

                setcookie('ron_language', $language, time() + 30000000);
            }
        }

        if ($language && file_exists($langDir . $language)) {
            $this->translationLang = $language;
            $_SESSION['language'] = $language;
        } else {
            $this->translationLang = '';
        }

        $manager = TextManager::getInstance();
        $manager ->defaultLang = $this ->originalLang;
        $manager ->translationLang = $this->translationLang;

        $manager ->loadStrings('ron_strings.ini');
        $manager ->loadStrings('app_strings.ini');
        $manager ->loadFile('user/user_strings.ini', true); // preferred = true
        if ($this->translationLang  ||  $manager ->defaultLang) {
            $manager->loadFile('user/user_strings.' . ($this->translationLang ?: $manager ->defaultLang) . '.ini', true); // preferred = true
        }
    }


    function addDebugInfo($debugInfoCaption = null, $infoText = null)
    {
        if (isset($debugInfoCaption)) {
            $this ->debugInfoCaption = $debugInfoCaption;
        }

        if ($infoText) {
            $this ->debugInfo[] = $infoText;
        }
    }


    function getDebugInfoCaption()
    {
        return $this ->debugInfoCaption;
    }


    function getDebugInfo()
    {
        return $this ->debugInfo;
    }


    /**
     * @param Database $database
     * @return bool
     */
    function afterDBConnect($database)
    {
        return true;
    }


    function dbConnect()
    {
        global $db_connectionstring;

        if ($db_connectionstring) {
            /** @var Database $database */
            $database = \Ron\DB\createDefaultDatabase();

            if ($GLOBALS['db_use_adhoc_params'] ?? false) {
                MessageManager::addWarning("Nastaveni db_use_adhoc_params je zapnuto! Nepouzivejte adhoc parametry v produkcnim prostredi!");
                $database ->useAdHocParams = true;
            }


            $db_user = $GLOBALS['db_user'] ?? null;
            $db_password = $GLOBALS['db_password'] ?? null;

            if (!isset($db_password)  &&  ($GLOBALS['db_password_env_key'] ?? false)) {
                if (function_exists("phpinfo")) {
                    throw new \Exception("Nelze pouzit db_password_env_key. Nejdrive nastavte v php.ini: disable_functions = phpinfo");
                }
                $db_password = getenv($GLOBALS['db_password_env_key']);
            }

            $item = Profiler::addItem("App::dbConnect (connect to database) ...");
            try {
                $database->connect($db_user, $db_password);
            } catch (DBException $e) {
                echo
                    \Ron\ronText('Connection_fail') . ' <br/>' .
                    $e->getMessage() . ' <br/>' .
                    $e->dbMessage . '<br/>';
                Profiler::endItem($item);
                exit;
            }
            Profiler::endItem($item);

            $this->afterDBConnect($database);
        } else {
            $database = null;
        }

        $this->database = $database;

        return $database;
    }


    /**
     * @return Database
     */
    public function getDatabase()
    {
        return $this->database;
    }

    /**
     * @param Database $database
     */
    public function setDatabase($database)
    {
        $this->database = $database;
    }


    /**
     * Inicializace
     */
    function init()
    {
        $item = Profiler::addItem('App::init (session ...)');

        if ($this->sessionName) {
            session_name($this->sessionName);
        }
        if (file_exists('user_session.php')) {

            require_once('user_session.php');
        }
        session_start();

        $_SESSION['last_access_timestamp'] = date('j.n.Y G:i:s');

        //$this -> LoadCookies();
        $this->initStrings();

        if (isset($_GET['debug'])) {
            App::setDebugMode($_GET['debug']);
        }

        if (App::getDebugMode()) {
            ini_set('display_errors', '1');
        }

        Profiler::endItem($item);
    }


    /**
     * Inicializace singletonu HttpResponse
     */
    function initHttpResponse()
    {
        HttpResponse::setHeader(HttpHeaders::CONTENT_TYPE, "text/html; charset=" . TextManager::getInstance() ->getCharSet());
    }


    /**
     * Nastavi v Modelu "ron@defaultQueryString"
     * - z aktualniho query stringu odstrani promenne zacinajici session@ (tj. ty, ktere jsou urcene pro session)
     */
    public function setDefaultQueryString()
    {
        $urlParams = [];
        parse_str($_SERVER['QUERY_STRING'], $urlParams);

        $newParams = array();

        global $secure_get_params;

        foreach ($urlParams as $key => $value) {
            if (strpos($key, 'session@') !== 0  &&  $key != 'loginpassword'  &&  $key != 'loginname') {
                $newParams[$key] = $secure_get_params
                    ? StringUtils::secureString($value, $secure_get_params)
                    : $value;
            }
        }

        Model::setData('ron@defaultQueryString', http_build_query($newParams));
    }


    /** Volano po inicializaci, pripojeni databaze atd. */
    public function afterInit()
    {

    }


    /**
     * Pokud existuje soubor $path/$action.php, tak jej includne a vraci vysledek (mel by to byt controller), jinak vraci null
     * @param $action
     * @param string $path
     * @return MVC\Controller
     */
    public function createControllerForAction($action, $path = 'controllers')
    {
        $actionFile = $path . '/' . preg_replace('/[^A-Za-z0-9_\-]/', '', $action) . '.php';

        if ( file_exists($actionFile)) {
            /** @noinspection PhpIncludeInspection */
            $controller = require($actionFile);
            if ($controller  &&  $controller instanceof MVC\Controller) {
                return $controller;
            }
        }

        return null;
    }


    /**
     * @param string $action
     * @return MVC\Controller
     */
    public function createController($action)
    {
        return $this ->createControllerForAction($action);
    }


    /**
     * @return string
     */
    public function getAction() {
        if (isset($_GET['menuid'])) {
            return $_GET['menuid'];
        }

        return null;
    }


    /**
     * @param string $action
     * @param string $view
     */
    public function renderView($action, $view)
    {
        HttpResponse::renderHeaders();

        if ($view) {
            /** @noinspection PhpIncludeInspection */
            require($view);
        }
    }


    /**
     * @param MVC\OutputObject $outputObject
     */
    public function renderOutputObject($outputObject)
    {
        $outputObject ->output();
    }


    /**
     * @param MVC\PartialView $viewObject
     * @return void
     */
    public function renderPartialView($viewObject)
    {
        HttpResponse::renderHeaders();
        if ($viewObject) {
            $viewObject ->render();
        }
    }

    /**
     * Muze vyrenderovat paticku - volano z View na konci stranky
     */
    public function renderHtmlFooter()
    {
    }

    /**
     * @param MVC\Controller $controller
     * @param string $action
     * @throws \Exception
     */
    public function processControllerResult($controller, $action)
    {
        $finished = false;

        $view = null;

        while ($controller  &&  !$finished) { // while kvuli InternalRedirect
            $this ->addDebugInfo(substr(strrchr(get_class($controller), "\\"), 1), get_class($controller) . '::handleAction("' . $action . '")');
            $finished = true;

            try {
                $result = $controller->handleAction($action);
            } catch (\Exception $e) {
                $this ->addDebugInfo(null, ' => ' . get_class($e));
                // V controlleru doslo k vyjimce - pokud implementuje ExceptionHandler, tak ji umi osetrit
                if ($controller instanceof MVC\ExceptionHandler) {
                    $result = $controller ->handleException($e);
                    if (!$result) {
                        throw $e;
                    }
                } else {
                    throw $e;
                }
            }

            if ($result) {
                $this ->addDebugInfo(null, ' => ' . (is_string($result) || is_bool($result) ? $result : get_class($result)));
            }

            if ($result instanceof MVC\OutputObject) {
                // output object (json atd.)
                $this ->renderOutputObject($result);

                return;
            } else if ($result instanceof MVC\Redirect) {
                $result ->render();
                $controller = null;
            } else if ($result instanceof MVC\InternalRedirect) {
                $action = $result->action;
                $controller = $this->createController($action);

                $finished = false;
            } else if ($result === true) {
                $view = $this->defaultView;
            } else if ($result instanceof MVC\PartialView) {
                $this ->renderPartialView($result);
                return;
            } else if ($this ->defaultView  &&  $result instanceof MVC\ViewObject) {
                Model::addViewObject($result);
                $view = $this ->defaultView;
            } else {
                // TODO: renderView volat jen v pripade, ze $result je string?
                $view = $result;
            }
        }

        // volat i kdyz neni nastaveno $view - kvuli headers!
        $this->renderView($action, $view);
    }



    /**
     * Zpracuje nezachycenou vyjimku (napr. zapis do logu)
     * @param \Exception $e
     * @return void
     */
    public function processUnhandledException($e)
    {
        if ($e instanceof RonException  &&  $e ->getResponseCode()) {
            http_response_code($e ->responseCode);
        }
    }


    /**
     * @param \Exception $e
     */
    public function catchUnhandledException($e)
    {
        $this->processUnhandledException($e);

        require('view/errorview.php');
    }


    public function run()
    {
        try {
            $this->initHttpResponse();

            $this->setDefaultQueryString();

            $this->dbConnect();
//            \Ron\DB\setDefaultDatabase($database);

            $this->afterInit();
        }
        catch (\Exception $e) {
            die ($e ->getMessage());
        }

        try {
            $action = trim($this->getAction() ?? '');

            $controller = $this->createController($action);

            if ($controller) {
                $this->processControllerResult($controller, $action);
            }
        }
        catch (\Exception $e) {
            $this ->catchUnhandledException($e);
        }
    }


    public static function getDebugMode()
    {
        global $enabledebugmode;
        return $enabledebugmode  &&  ($_SESSION['debugmode'] ?? false);
    }


    public static function setDebugMode($value)
    {
        $_SESSION['debugmode'] = $value;
    }


    /**
     * @param string $itemName
     * @param string $itemValue
     * @param string $defaultValue
     */
    public function storeUserSettings($itemName, $itemValue, $defaultValue = null)
    {
    }

    /**
     * @param string $itemName
     * @param string $defaultValue
     * @return string
     */
    public function getUserSettings($itemName, $defaultValue = null)
    {
        return $defaultValue;
    }
}
