<?php
/**
 * stanislav.elbl, 09.08.2022 9:13
 * Formular pro editaci ukolu (splneni a hodnoceni)
 */

namespace RonPortal\Common;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\DBUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\UserLoginFormField;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\User;


/**
 * Termin splneni - zvyrazni jej, pokud je po terminu
 * Class TerminSplneniUkoluField
 */
class TerminSplneniUkoluField extends FormField
{
    /** @var Database */
    private $database;

    /**
     * TerminSplneniUkoluField constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct(FormField::FIELD_TYPE_DATE);
        $this ->database = $database;
    }

    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);

        if ($rowData  &&  !$rowData['field-stavukolu']) {
            // ukol neni splnen
            $termin = $this ->database ->stringToTimestamp($rowData['field-datumsplneni_narizene']);
            if ($termin  &&  $termin < time()) {
                $result['class'] = isset($result['class'])
                    ? $result['class'] . ' po_terminu'
                    : 'po_terminu';
            }
        }

        return $result;
    }
}

/**
 * Zobrazi datum splneni ukolu a v pripade, ze neni zadan a ukol je splnen (tj. StavUkolu je 1), tak zobrazi fajfku
 * Class DatumSplneniUkoluField
 */
class DatumSplneniUkoluField extends FormField
{
    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this ->readOnly  &&  !$value  &&  $rowData['field-stavukolu'] == 1) {
            // ukol splnen, ale nema zapsan datum splneni
            echo '<i class="fa inline-glyph glyph-check"></i>';
        }

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }
}


/**
 * Zobrazi pocet bodu ukolu (maximalni a dosazeny)
 * Class PocetBoduUkoluField
 */
class PocetBoduUkoluField extends FormField
{
    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo
            '<strong>' .
            (isset($rowData[0])
                ? StringUtils::format($this ->format, $rowData[0])   // dosazeny pocet
                : (isset($rowData[1]) ? ' - ' : '')) .
            '</strong>' .
            (isset($rowData[1])
                ? ' / ' . StringUtils::format($this ->format, $rowData[1])   // maximalni pocet
                : '');

        if (isset($rowData[0])  &&  isset($rowData[1])  &&  $rowData[1]) {
            echo ' (' . StringUtils::format('number0', $rowData[0] / $rowData[1] * 100) . '%)';
        }
    }
}


/**
 * Zobrazi odmenu - navrzenou a skutecnou
 * Class OdmenaUkoluField
 */
class OdmenaUkoluField extends FormField
{
    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo
            '<strong>' .
            (isset($rowData[2])
                ? StringUtils::format($this ->format, $rowData[2])   // skutecna odmena
                : (isset($rowData[3]) ? ' - ' : '')) .
            '</strong>' .
            (isset($rowData[3])
                ? ' / ' . StringUtils::format($this ->format, $rowData[3])   // navrzena odmena
                : '');
    }
}


/**
 * Pole pro zobrazeni / editaci dovednosti
 * Class DovednostUkoluField
 */
class DovednostUkoluField extends FormField
{
    /** @var int */
    public $kodSerialUkolu;
    /** @var string */
    public $kodDovednosti;
    /** @var int */
    public $kodUrovne;

    /**
     * DovednostUkoluField constructor.
     * @param int   $kodSerialUkolu
     * @param string $kodDovednosti
     * @param int   $kodUrovne
     */
    public function __construct($kodSerialUkolu, $kodDovednosti, $kodUrovne)
    {
        parent::__construct(FormField::FIELD_TYPE_CHECK);

        $this->kodSerialUkolu = $kodSerialUkolu;
        $this->kodDovednosti = $kodDovednosti;
        $this->kodUrovne = $kodUrovne;

        $this
            ->setFieldName('Dovednost_' . $kodDovednosti . '_' . $kodUrovne)
            ->setSqlFieldName(null)
            ->setCode('field-dovednost-' . $kodDovednosti . '_' . $kodUrovne);
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!$canEdit) {
            if ($value) {
                echo '<i class="fa inline-glyph glyph-check"></i>';
            }

            return;
        }

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }


    /**
     * @param Database $database
     * @param string|object $logDescription
     */
    public function zapisDovednost($database, $logDescription)
    {
        if ($this ->readOnly  ||  !$this ->visible) {
            return;
        }

        if (isset($_POST[$this ->fieldName])  &&  (int)$_POST[$this ->fieldName]) {
            // uzivatel zaskrtl checkbox pro danou dovednost
            $rec = new RonPortalDBRecord($database, 'DosazenaDovednostUkolu', $logDescription);

            $rec ->addIntField('KodUkolu', false, $this->kodSerialUkolu);
            $rec ->addStringField('KodDovednosti', false, 40, $this->kodDovednosti);
            $rec ->addIntField('KodUrovne', false, (int)$this->kodUrovne);

            $params = $database ->createParameters();
            $rec ->setWhere(
                'KodUkolu = ' . $params ->addParameter($this ->kodSerialUkolu) .
                ' AND KodDovednosti = ' . $params ->addParameter($this ->kodDovednosti) .
                ' AND KodUrovne = ' . $params ->addParameter($this ->kodUrovne),
                $params
            );

            $rec ->updateOrInsert();
        }
    }

}

class ResitelFormField extends OsobaFormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            $value = PracovniPomer::nactiPPOsoby($database, $value, time())[0]->idPP;
        }
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }
}


class UkolFormular extends RonPortalForm
{
    /** @var Ukol */
    private $ukol;

    /** @var DovednostUkoluField[] */
    private $fieldsDovednost = [];

    /**
     * @param Database $database
     * @param Ukol|null $ukol
     * @throws NeniPravoException
     * @throws \Exception
     */
    function __construct($database, $ukol)
    {
        parent::__construct($database, $ukol ? 'edit' : 'new');

        $this->ukol = $ukol;

        $this->htmlAction = '?menuid=ukoly' . (!is_null($ukol) ? '&record=' . $ukol->kodSerial : '');
        $this->stornoAction = '?menuid=ukoly';

        $this->title = TextManager::_('Ukol');
        $this->code = 'formvukol';

        $this->sqlTableName = 'UkolOsoby';

        $this->sqlSelectPart =
            'PocetBodu_Dosazeny, PocetBodu_Maximalni,' .    // 0,1
            ' SkutecnaOdmena, NarvzenaOdmena';  // 2,3

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('KodSerial')->setPrimary(true);

        // STAV
        if (!$this->isInsert()) {
            $this->addField(FormField::FIELD_TYPE_COMBO)
                ->setFieldName('StavUkolu')->setReadOnly(true)->setCaption('StavUkolu')
                ->setCode('field-stavukolu')->addFieldCssClass('font-weight-bold')
                ->setOptions([0 => TextManager::_('UkolProbiha'), 1 => TextManager::_('UkolSplnen'), -1 => TextManager::_('UkolZrusen')]);
        }

        // DEFINICE UKOLU
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-definice-ukolu');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('sec-definice-ukolu')
            ->setCaption('Definice_ukolu');

//        if (!$this->isInsert()) {
//            $this->addField(FormField::FIELD_TYPE_TEXT)
//                ->setCode('field-zadano-osobe')->addFieldCssClass('font-weight-bold')
//                ->setCaption('Resitel')
//                ->setText(StringUtils::htmlTextConvert($ukol->osoba_Jmeno));
//        }

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('kodKategorieUkolu')
            ->setCode('field-typ-ukolu')->addFieldCssClass('font-weight-bold')
            ->setCaption('Typ_ukolu')
            ->setChangeable(empty($ukol) || empty($ukol->RC_KdoZadal) || User::hasRC($ukol->RC_KdoZadal))
            ->setReadOnly(!(empty($ukol) || empty($ukol->RC_KdoZadal) || User::hasRC($ukol->RC_KdoZadal)))
            ->setSelectSql('SELECT kod, popis FROM KategorieUkolu WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'popis');

        if (!$this->isInsert()) {
            $this->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('PrubehUkoluOsoby')
                ->setCode('field-prubeh-ukolu')
                ->setCaption('prubeh_ukolu')
                ->setChangeable(User::hasRC($ukol->RC_Osoby))
                ->setReadOnly(!User::hasRC($ukol->RC_Osoby))
                ->setSelectSql('SELECT kod, popis FROM PrubehUkoluOsoby WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'popis');

            $this->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('PrioritaUkoluOsoby')
                ->setCode('field-priorita-ukolu')
                ->setCaption('priorita_ukolu')
                ->setChangeable(User::hasRC($ukol->RC_KdoZadal) || $ukol->uzivatelMaPovolenuKategoriiUkolu($this->database))
                ->setReadOnly(!(User::hasRC($ukol->RC_KdoZadal) || $ukol->uzivatelMaPovolenuKategoriiUkolu($this->database)))
                ->setSelectSql('SELECT kod, popis FROM PrioritaUkoluOsoby WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'popis');
        }

        $this->addField()
            ->setFieldName('ZkracenyPopis')->setReadOnly(true)->setCaption('zkraceny_popis')
            ->setCode('field-zkracenypopis')->setRequired(true);

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('PodrobnyPopis')->setReadOnly(true)->setCaption('Detailni_popis')
            ->setCode('field-podrobnypopis')->setDataColumnClass('form-col2 poznamka');

        if (!$this->isInsert()) {
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-zadavatel')->addFieldCssClass('font-weight-bold')->addDataColumnClass('margin-top-2')
                ->setCaption('Zadavatel')
                ->setText(StringUtils::htmlTextConvert($ukol->kdoZadal_Jmeno));
        }

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumZadani')->setReadOnly(true)->setCaption('Datum_zadani')->addDataColumnClass('margin-top-2')
            ->setCode('field-datumzadani')->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        $this->addField(new TerminSplneniUkoluField($this->database))
            ->setFieldName('DatumSplneni_Narizene')->setReadOnly(true)->setCaption('Termin_ukolu')->addFieldCssClass('font-weight-bold')
            ->setCode('field-datumsplneni_narizene');

        if (!$this->isInsert()) {
            $this->addField(new ResitelFormField('ukoly_nast_resitele', time(), !empty($this->ukol->RC_Osoby) ? PracovniPomer::nactiPPOsoby($database, $this->ukol->RC_Osoby)[0] : null))
                ->setFieldName('Resitel')->setCode('field-zadano-osobe')->addFieldCssClass('font-weight-bold')
                ->setCaption('Resitel')->setSqlFieldName('RC_Osoby')->setRequired(false)
                ->setReadOnly(!$ukol->uzivatelMaPovolenuKategoriiUkolu($this->database))
                ->setChangeable(($this->ukol->stavUkolu !== 1) && $ukol->uzivatelMaPovolenuKategoriiUkolu($this->database));
            //dokud neni ukol splnen a uzivatel muze menit kategorie, pujde zmenit resitele (pro pripad, ze se zmeni kategorie a bude potreba nastavit jineho resitele)

            $this->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('Vyjadreni_KdoZadal')->setReadOnly(false)->setCaption('vyjadreni_zadavatele')
                ->setCode('field-vyjadreni_zadavatele')->setDataColumnClass('form-col2 poznamka')
                ->setChangeable(User::hasRC($this->ukol->RC_KdoZadal))->setReadOnly(!User::hasRC($this->ukol->RC_KdoZadal));

            $this->addField(new DatumSplneniUkoluField(FormField::FIELD_TYPE_DATE))
                ->setFieldName('DatumSplneni_Skutecne')->setReadOnly(true)->setCaption('Datum_splneni')->addFieldCssClass('font-weight-bold')
                ->setCode('field-datumsplneni_skutecne');

            if (RonPortalSettings::getIntValue('zobrazit_body')) {
                $this->addField(new PocetBoduUkoluField())
                    ->setCaption('<strong>' . TextManager::_('dosazene_body') . '</strong> / ' . TextManager::_('max_body'))
                    ->setCode('field-body')
                    ->setFormat('number0');
            }

            if (RonPortalSettings::getIntValue('zobrazit_odmenu')) {
                $this->addField(new OdmenaUkoluField())
                    ->setCaption('<strong>' . TextManager::_('skutecna_odmena') . '</strong> / ' . TextManager::_('Navrzena'))
                    ->setCode('field-odmena')
                    ->setFormat('number2');
            }

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-vazana-osoba')->addFieldCssClass('font-weight-bold margin-top-2')
                ->setCaption($ukol->popisVazbyNaOsobu)
                ->setText(StringUtils::htmlTextConvert($ukol->vazanaOsobaJmeno));

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-hodnotitel')->addFieldCssClass('font-weight-bold')
                ->setCaption('Hodnotitel')
                ->setText(StringUtils::htmlTextConvert($ukol->hodnotitel));

            if (trim($ukol->schvalovatel1_Def)) {
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode('field-schvalovatel1')->addFieldCssClass('font-weight-bold')
                    ->setCaption('Schvalovatel_1')
                    ->setText(StringUtils::htmlTextConvert($ukol->schvalovatel1_Def));
            }

            if (trim($ukol->schvalovatel2_Def)) {
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode('field-schvalovatel2')->addFieldCssClass('font-weight-bold')
                    ->setCaption('Schvalovatel_2')
                    ->setText(StringUtils::htmlTextConvert($ukol->schvalovatel2_Def));
            }

            // VYHODNOCENI
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-vyhodnoceni-ukolu');

            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('sec-vyhodnoceni-ukolu')
                ->setCaption('Vyhodnoceni_ukolu');

            // Prekazka
            $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('Prekazka')->setReadOnly(!User::hasRC($ukol->RC_Osoby))->setCaption('Prekazka_ukolu')
                ->setCode('field-prekazka')
                ->setCaption2(
                    ($ukol->datumZapisuPrekazky
                        ? ' (' . DateUtils::fineDateString($ukol->datumZapisuPrekazky, true) . ')'
                        : '') .
                    (!$ukol->prekazka && $ukol->datumOdstraneniPrekazky
                        ? ' - ' . TextManager::_('Prekazka_odstranena') . ' ' . DateUtils::fineDateString($ukol->datumOdstraneniPrekazky, true)
                        : '')
                );

            $this->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('PoznamkaKPrekazce')->setReadOnly(!User::hasRC($ukol->RC_Osoby))->setCaption('Poznamka_k_prekazce')
                ->setCode('field-poznamkakprekazce');

            // Splneni
            $prava = Prava::vratPrava('ukoly');

            if (User::hasRC($ukol->RC_Osoby) ? $prava->priznak : 1 == $ukol->stavUkolu) {
                // muj ukol a muzu editovat cas splneni
                // nebo je to cizi ukol a je splneny
                // => zobrazim cas splneni
                $this->addField(FormField::FIELD_TYPE_DATE)
                    ->setFieldName('DatumSplneni_Skutecne')->setReadOnly(!User::hasRC($ukol->RC_Osoby))->setCaption('Datum_splneni')
                    ->setCode('field-datumsplneni_skutecne');
            } else if (User::hasRC($ukol->RC_Osoby)) {
                // nemuzu editovat datum splneni, ale je to muj ukol => zobrazim zaskrtavatko
                $this->addField(FormField::FIELD_TYPE_CHECK)
                    ->setFieldName('Splneno')->setSqlFieldName(null)->setCaption('Splneno')->setDefaultValue(1 == $ukol->stavUkolu)
                    ->setCode('field-splneno-checkbox');
            }

            $this->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('PoznamkaKeSplneni')->setReadOnly(!User::hasRC($ukol->RC_Osoby))->setCaption('Poznamka_ke_splneni')
                ->setCode('field-poznamkakesplneni');

            // Hodnoceni
            $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('Hodnoceno')->setCaption('Hodnoceno')->setReadOnly(!User::hasRC($ukol->RC_Hodnotitele_Def ?? '') || 1 != $ukol->stavUkolu)
                ->setCode('field-hodnoceno')
                ->setCaption2(1 == $ukol->stavUkolu && $ukol->datumHodnoceni
                    ? ' ' . DateUtils::fineDateString($ukol->datumHodnoceni, true)
                    : '');

            $this->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('PodrobnyPopisHodnoceni')->setCaption('Poznamka_k_hodnoceni')->setReadOnly(!User::hasRC($ukol->RC_Hodnotitele_Def ?? '') || 1 != $ukol->stavUkolu)
                ->setCode('field-podrobnypopishodnoceni');

            if ($ukol->hodnoceno && 1 == $ukol->stavUkolu && $ukol->kdoHodnotil_Jmeno) {
                $this->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode('field-hodnotil')
                    ->setCaption('Hodnotil')
                    ->setText(StringUtils::htmlTextConvert($ukol->kdoHodnotil_Jmeno));
            }

            $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
                ->setFieldName('Update_Timestamp')->setReadOnly(false)
                ->setCode('field-update_timestamp')->setUpdateLog(true);
            $this->addField(new UserLoginFormField(false))->setUpdateLog(true);

            // Dovednosti
            $this->pridejPoleProDovednosti($ukol);
        }
    }


    /**
     * Prida pole formulare pro zobrazeni a pripadne potvrzeni dosazenych dovednosti
     * @param $ukol
     */
    private function pridejPoleProDovednosti($ukol)
    {
        // Dovednosti
        $params = $this ->database ->createParameters();

        $cursor = $this ->database ->getSqlResult(
            'SELECT Dovednost.Popis, UrovenDovednosti.Popis, Dovednost.Kod, UrovenDovednosti.KodUrovne, ' . // 0 - 3
            '  DosazenaDovednostUkolu.KodUkolu AS DovednostDosazena' . // priznak, jestli byla uroven dosazena
            ' FROM UkolOsoby' .
            ' INNER JOIN DovednostUkolu ON (UkolOsoby.KodSerial = DovednostUkolu.KodUkolu) ' .
            ' INNER JOIN UrovenDovednosti ON (DovednostUkolu.KodUrovne = UrovenDovednosti.KodUrovne AND DovednostUkolu.KodDovednosti = UrovenDovednosti.KodDovednosti) ' .
            ' INNER JOIN Dovednost ON (UrovenDovednosti.KodDovednosti = Dovednost.Kod) ' .
            ' LEFT OUTER JOIN DosazenaDovednostUkolu' .
            '  ON DosazenaDovednostUkolu.KodUkolu = UkolOsoby.KodSerial' .
            '  AND DosazenaDovednostUkolu.KodDovednosti = DovednostUkolu.KodDovednosti' .
            '  AND DosazenaDovednostUkolu.KodUrovne = DovednostUkolu.KodUrovne' .
            ' WHERE UkolOsoby.KodSerial = ' . $params->addParameter($ukol ->kodSerial) .
            ' ORDER BY DosazenaDovednostUkolu.KodUkolu DESC, Dovednost.Popis, UrovenDovednosti.KodUrovne', // ziskane dovednosti jsou prvni
            $params);

        $prvni = true;
        $prvniNedosazena = true;

        $lzeEditovatDovednosti =
            $ukol ->stavUkolu == 1 // musi byt splnen
            && (User::hasRC($ukol ->RC_Hodnotitele_Def ?? '') || User::hasRC($ukol ->RC_schvalovatele1_Def ?? '') || User::hasRC($ukol ->RC_schvalovatele2_Def ?? ''));

        while ($row = $cursor ->fetch()) {
            if ($prvni) {
                $this ->addField(FormField::FIELD_TYPE_GROUP)
                    ->setCode('group-dovednosti');
            }

            if ($row[4] ? $prvni : $prvniNedosazena) {
                if ($row[4]) {
                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('sec-dovednosti')
                        ->setCaption('Ukolem_dosazene_dovednosti');
                } else {
                    $prvniNedosazena = false;

                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('sec-dalsi-dovednosti')
                        ->setCaption('Definovane_dovednosti_ukolu');
                }
            }

            $prvni = false;

            $f = new DovednostUkoluField($ukol ->kodSerial, $row[2], $row[3]);
            $f  ->setDefaultValue((bool)$row[4]) ->setReadOnly(!$lzeEditovatDovednosti || $row[4]) // nelze vypnout
                ->setCaption(StringUtils::htmlTextConvert($row[0]))     // popis
                ->setCaption2(' ' . StringUtils::htmlTextConvert($row[1]));   // uroven

            $this ->addField($f);
            $this ->fieldsDovednost[] = $f;
        }
    }


    public function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);
        $rec ->addTimestampField('CasHodnoceni', true);

        if ($this->isInsert()) {
            $rec->addStringField('RC_Vlozil', true, '15', User::getRC());
        } else {
            if (isset($_POST['Resitel']) && !empty($_POST['Resitel'])) {
                if (empty($this->ukol->RC_Osoby) && $this->ukol->uzivatelMaPovolenuKategoriiUkolu($this->database)) {
                    $rec->addStringField('RC_KdoZadal', false, '15', User::getRC());
                }

                $pp = PracovniPomer::nactiPP($this->database, (int)$_POST['Resitel']);
                $rec->addStringField('RC_Osoby', false, '15', $pp->rc);
            }
        }
    }


    /**
     * @param \Ron\DB\DBRecord $rec
     * @return bool
     */
    function beforeUpdateRecord($rec)
    {
        if ($rec['Prekazka']  &&  $rec['Prekazka'] != $rec['#Prekazka']) {
            $rec ->addTimestampField('DatumZapisuPrekazky', false, time());
        }

        if (!$rec['Prekazka']  &&  $rec['Prekazka'] != $rec['#Prekazka']) {
            $rec ->addTimestampField('DatumOdstraneniPrekazky', false, time());
        }

        $f = $this ->findFieldByName('Splneno');
        if ($f  &&  !$f ->readOnly) {
            $splneno = isset($_POST['Splneno']) && (int)$_POST['Splneno'];
            if ($splneno) {
                $rec ->findFieldByName('StavUkolu') ->setNewValue(1);

                if (!$rec['DatumSplneni_Skutecne']) {
                    $rec->findFieldByName('DatumSplneni_Skutecne')->setNewValue(time());
                }
            } else {
                $rec ->findFieldByName('StavUkolu') ->setNewValue(0);

                if ($rec['DatumSplneni_Skutecne']) {
                    $rec->findFieldByName('DatumSplneni_Skutecne')->setNewValue(null);
                }
            }
        }

        $f = $this ->findFieldByName('Hodnoceno');
        if ($f  &&  !$f ->readOnly) {
            if ($rec['Hodnoceno']  &&  $rec['Hodnoceno'] != $rec['#Hodnoceno']) {
                $rec ->addStringField('RC_KdoHodnotil', false, 0, User::getRC());
                $rec ->findFieldByName('CasHodnoceni') ->setNewValue(time());
            }
        }

//        if ($this->RC_schvalovatele1_Def == User::getRC()) {
//            $rec ->addField('Schvaleno1', 'int', false, 0, $_POST['schvaleni1'] ? 1 : 0 );
//            $rec ->addField('RC_Schvalovatele1', '', false, 0, User::getRC());
//            $rec ->addField('CasSchvaleni1', 'timestamp', false, 0, time())->updateLog = true;
//        }

//        if ( $this->RC_schvalovatele2_Def == User::getRC()) {
//            $rec ->addField('Schvaleno2', 'int', false, 0, $_POST['schvaleni2'] ? 1 : 0 );
//            $rec ->addField('RC_Schvalovatele2', '', false, 0, User::getRC());
//            $rec ->addField('CasSchvaleni2', 'timestamp', false, 0, time())->updateLog = true;
//        }

        return parent::beforeUpdateRecord($rec);
    }


    function afterUpdateRecord($rec)
    {
        if ($rec['StavUkolu'] == 1) {
            $this->zapisUkolemDosazeneDovednosti();
        }

        return parent::afterUpdateRecord($rec);
    }


    /**
     * Zapis definovanych dovednosti ukolu do tabulky DosazenaDovednostUkolu
     */
    private function zapisUkolemDosazeneDovednosti()
    {
        foreach ($this ->fieldsDovednost as $field) {
            $field ->zapisDovednost($this ->database, $this);
        }
    }
}
