<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

class MesicniVysledek
{
    public $idPP;

    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    /** @var int */
    public $mesic;
    /** @var string */
    public $typCasoveSlozkyMzdy;

    /** @var float */
    public $hodiny;
    /** @var float */
    public $dny;
    /** @var float */
    public $kalendarniDny;

    public $poznamka;

    /** @var string     Naplni se podle Update_UserName nebo Insert_UserName */
    public $lastUpdateUser;
    /** @var string     Naplni se podle Update_Timestamp nebo Insert_Timestamp */
    public $lastUpdateTimestamp;


    protected function loadDefaultData($database, $row)
    {
        $this ->idPP = (int)$row[0];
        $this ->rc = $row[1];
        $this ->osobniCislo = $row[2];
        $this ->datumNastupu = $database ->stringToTimestamp($row[3]);

        $this ->mesic = $database ->stringToTimestamp($row[4]);
        $this ->typCasoveSlozkyMzdy = $row[5];

        $this ->hodiny = isset($row[6]) ? (float)$row[6] : null;
        $this ->dny = isset($row[7]) ? (float)$row[7] : null;
        $this ->kalendarniDny = isset($row[8]) ? (float)$row[8] : null;

        $this ->poznamka = $row[9];
    }

    /**
     * @param Database $database
     * @param array $row
     */
    protected function load($database, $row)
    {
        $this ->loadDefaultData($database, $row);

        $this ->lastUpdateUser = $row[12] ?: $row[10];
        $this ->lastUpdateTimestamp = $database ->stringToTimestamp($row[12] ? $row[13] : $row[11]);
    }


    protected static function getDefaultSelectPart($tableName, $nacistPoznamku)
    {
        return
            'PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, ' . // 0..3
            " $tableName.Mesic, $tableName.TypCasoveSlozkyMzdy, " . // 4,5
            " $tableName.Hodiny, $tableName.Dny, $tableName.KalendarniDny, " . // 6..8
            ($nacistPoznamku
                ? "$tableName.Poznamka" // 9
                : "null" // 9
            );
    }


    protected static function getSelectPart($tableName)
    {
        return
            static::getDefaultSelectPart($tableName, true) .
            ", $tableName.Insert_UserName, $tableName.Insert_Timestamp," . // 10..
            " $tableName.Update_UserName, $tableName.Update_Timestamp";  // ..13
    }

    /**
     * Nacte mesicni vysledky. Pokud se nacitaji data jen pro jednu osobu a mesic a $povolitAsociativni je true, tak vraci asociativni pole indexovane typem.
     * @param Database $database
     * @param $tableName
     * @param int|int[] $idPP
     * @param int $mesic
     * @param string|string[]|null $typCasoveSlozkyMzdy
     * @param bool $povolitAsociativni
     * @return MesicniVysledek[]
     * @throws DBException
     */
    protected static function nactiZaznamy($database, $tableName, $idPP, $mesic, $typCasoveSlozkyMzdy = null, $povolitAsociativni = true)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT ' .
            static::getSelectPart($tableName) .
            " FROM $tableName, PracovniPomer" .
            " WHERE PracovniPomer.RC = $tableName.RC" .
            "   AND PracovniPomer.OsobniCislo = $tableName.OsobniCislo" .
            "   AND PracovniPomer.DatumNastupu = $tableName.DatumNastupu";

        $jedenPP = false;

        if ($idPP) {
            if (is_array($idPP)) {

                $sql .= ' AND (0=1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';

            } else {
                $jedenPP = true;
                $sql .= ' AND PracovniPomer.Id = ' . $params->addParameter($idPP);
            }
        }

        if ($mesic) {
            $sql .=
                " AND $tableName.Mesic = " . $params ->addDateParameter($mesic);
        }

        if ($typCasoveSlozkyMzdy) {
            $sql .= ' AND';

            if (is_array($typCasoveSlozkyMzdy)) {
                $sql .= ' (';

                $first = true;
                foreach ($typCasoveSlozkyMzdy as $slozka) {
                    if (!$first) {
                        $sql .= ' OR';
                    }
                    $first = false;

                    $sql .= " $tableName.TypCasoveSlozkyMzdy = " . $params ->addParameter($slozka);
                }

                $sql .= ')';

            } else {
                $sql .= " $tableName.TypCasoveSlozkyMzdy = " . $params ->addParameter($typCasoveSlozkyMzdy);
            }
        }

//        if ($filter) {
//            $sql .= ' AND ' . $filter ->sqlText;
//            $params ->appendParameters($filter ->parameters);
//        }

        $sql .= " ORDER BY PracovniPomer.Id, $tableName.Mesic, $tableName.TypCasoveSlozkyMzdy";

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani mesicnich vysledku doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            if ($povolitAsociativni  &&  $jedenPP  &&  $mesic) {
                // data jen pro jednu osobu/mesic - vraci asociativni pole
                $result[$vysledek ->typCasoveSlozkyMzdy] = $vysledek;
            } else {
                $result[] = $vysledek;
            }
        }
        $cursor ->close();

        return $result;
    }

    /**
     * Nacte mesicni vysledky. Pokud se nacitaji data jen pro jednu osobu a mesic, tak vraci asociativni pole indexovane typem.
     * @param Database $database
     * @param int|int[] $idPP
     * @param int $mesic
     * @param string|string[]|null $typCasoveSlozkyMzdy
     * @return MesicniVysledek[]
     * @throws DBException
     */
    public static function nactiMesicniVysledky($database, $idPP, $mesic, $typCasoveSlozkyMzdy = null)
    {
        return static::nactiZaznamy($database, 'MesicniVysledekVypoctu', $idPP, $mesic, $typCasoveSlozkyMzdy);
    }


    /**
     * Nacte mesicni vysledek pro zadanou osobu, obdobi a casovou slozku
     * @param Database $database
     * @param int $idPP
     * @param int $mesic
     * @param string $typCasoveSlozkyMzdy
     * @return MesicniVysledek
     * @throws DBException
     */
    public static function vratMesicniVysledek($database, $idPP, $mesic, $typCasoveSlozkyMzdy)
    {
        $data = self::nactiMesicniVysledky($database, $idPP, $mesic, $typCasoveSlozkyMzdy);

        return $data[$typCasoveSlozkyMzdy] ?? null;
    }

    /**
     * Vraci ciselnou hodnotu pro zadanou jednotku
     * @param $jednotka
     * @return float|string
     */
    public function vratHodnotu($jednotka)
    {
        switch (strtoupper($jednotka)) {
            case 'D' :
                return $this ->dny;

            case 'K' :
                return $this ->kalendarniDny;

            case 'H' :
            case '' :
                return $this ->hodiny;
        }

        return '';
    }


    /**
     * @param string $jednotka H/D/K/P
     * @param string $format
     * @return string
     */
    public function vratText($jednotka, $format = null)
    {
        switch (strtoupper($jednotka)) {
            case 'H' :
                return ($this ->hodiny) ? StringUtils::format($format ?: 'minuty', round($this ->hodiny * 60)) : '';

            case 'D' :
                return $this ->dny ? StringUtils::format($format ?: 'number_2', $this ->dny) : '';

            case 'K' :
                return $this ->kalendarniDny ? StringUtils::format($format ?: 'number_2', $this ->kalendarniDny) : '';

            case 'P' :
                return $this ->poznamka;
        }

        return '';
    }


    public function vratHint()
    {
        return $this ->lastUpdateUser
            ? $this ->lastUpdateUser . ', ' . StringUtils::format('timestamp', $this ->lastUpdateTimestamp)
            : '';
    }
}
