<?php

/**
 * Formular pro editaci a pridavani vozidel
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RCOsobyFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;


/**
 * Pole pro zobrazeni prirazeni vozidla osobam (aktualni i historie) a editaci platnosti.
 * Konfiguracni volba lzeEditovatHistorii urcuje, jestli muze uzivatel editovat i zaznamy v minulosti (posouvat platnosti a rusit)
 */
class PrirazeneOsobyVozidlaField extends FormField
{
    /** @var array */
    private $osoby;

    /** @var bool */
    private $lzeEditovatHistorii;


    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_TEXT);

        // implicitne muze editovat historii, jen kdyz ma pravo smazat existujici vozidlo
        $this ->lzeEditovatHistorii = Prava::vratPrava('vozidla') ->priznak;
    }

    protected function loadConfigurationItem($key, $value)
    {
        if ($key === 'lzeEditovatHistorii') {
            $this ->lzeEditovatHistorii = (int)$value;
        }

        parent::loadConfigurationItem($key, $value);
    }



    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);
        $result['data-lzeEditovatHistorii'] = $this ->lzeEditovatHistorii ? '1' : '0';
        return $result;
    }


    private function nactiDataProVozidlo(Database $database, $registracniZnacka)
    {
        $this ->osoby = [];

        if (($registracniZnacka ?? '') === '') {
            // neni co nacitat - neni editovane existujici vozidlo (jedna se o INSERT)
            return;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT VozidloOsoby.PlatnostOd, VozidloOsoby.PlatnostDo, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, MIN(PracovniPomer.Id)' .
            ' FROM VozidloOsoby, Osoba, PracovniPomer' .
            ' WHERE Osoba.RC = VozidloOsoby.RC' .
            '   AND PracovniPomer.RC = VozidloOsoby.RC' .
            '   AND VozidloOsoby.RegistracniZnacka = ' . $params ->addParameter($registracniZnacka) .
            ' GROUP BY VozidloOsoby.PlatnostOd, VozidloOsoby.PlatnostDo, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' .
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno, VozidloOsoby.PlatnostOd DESC',
            $params
        );

        while ($row = $cursor ->fetch()) {
            $this ->osoby[] = [
                'platnostOd' => $database ->stringToTimestamp($row[0]),
                'platnostDo' => $database ->stringToTimestamp($row[1]),
                'rc' => $row[2],
                'prijmeni' => $row[3],
                'jmeno' => $row[4],
                'titul' => $row[5],
                'idPP' => $row[6]
            ];

        }
    }


    public function loadDatabaseValue(Database $database, ?FormRowArray $rowData)
    {
        $this ->nactiDataProVozidlo($database, $rowData['field-registracniznacka']);
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $prava = Prava::vratPrava('vozidla');

        // header
        echo
            '<div class="row d-none d-sm-flex background-table-header color-table-header">' .
            '<div class="col-7"></div>' .
            '<div class="col-2 padding-all">' . TextManager::_('Platnost_od') . '</div>' .
            '<div class="col-2 padding-all">' . TextManager::_('do') . '</div>' .
            '</div>';

        foreach ($this ->osoby as $index => $item) {
            $lzeEditovat = $prava ->lzeZobrazitRC($item['rc']);

            $htmlSuffix =
                ($item['idPP'] ?? StringUtils::htmlTextConvert(str_replace(' ', '_', $item['rc']))) . // teoreticky muze byt vozidlo prirazeno osobe bez PP
                '_' . $item['platnostOd'];

            $platnostOdHtmlName = 'PlatnostOd_' . $htmlSuffix;
            $platnostOdStr = ($lzeEditovat  &&  $this ->lzeEditovatHistorii  &&  isset($_POST[$platnostOdHtmlName]))
                ? StringUtils::htmlTextConvert($_POST[$platnostOdHtmlName])
                : StringUtils::format('date', $item['platnostOd']);


            $platnostDoHtmlName = 'PlatnostDo_' . $htmlSuffix;
            $platnostDoStr = ($lzeEditovat && isset($_POST[$platnostDoHtmlName]))
                ? StringUtils::htmlTextConvert($_POST[$platnostDoHtmlName])
                : StringUtils::format('date', $item['platnostDo']);

            echo
                "<div class='vozidlo-osoby row' id='row_vozidloosoby_$index'>" .
                // jmeno osoby
                '<div class="col-12 col-sm-7">' .
                StringUtils::htmlTextConvert(($item['titul'] ? $item['titul'] . ' ' : '') .  $item['prijmeni'] . ' ' . $item['jmeno']) .
                '</div>' .

                // platnost od
                '<div class="col-3 col-sm-0 align-right padding-right-2">' . TextManager::_('Platnost_od') . '</div>' .
                '<div class="col-3 col-sm-2">' .
                ( $lzeEditovat  &&  $this ->lzeEditovatHistorii
                    ? '<input name="' . $platnostOdHtmlName . '" class="form-control date-input ' . MessageManager::getClass($platnostOdHtmlName) . '" value="' . $platnostOdStr . '"/>'
                    : $platnostOdStr
                ) .
                '</div>' .

                // platnost do
                '<div class="col-2 col-sm-0 align-right padding-right-2">' . TextManager::_('do') . '</div>' .
                '<div class="col-3 col-sm-2">' .
                ( ($lzeEditovat  &&  ($this ->lzeEditovatHistorii  ||  !$item['platnostDo']  ||  $item['platnostDo'] >= DateUtils::today()))
                    ? '<input name="' . $platnostDoHtmlName . '" class="form-control date-input ' . MessageManager::getClass($platnostDoHtmlName) . '" value="' . $platnostDoStr . '"/>'
                    : $platnostDoStr
                ) .
                '</div>' .

                // ikony
                '<div class="col-1">' .
                ( $lzeEditovat && $this ->lzeEditovatHistorii
                    ?   "<input type='hidden' name='Delete_$htmlSuffix' value='0' id='Delete_$htmlSuffix'/>" .
                        "<i class='inline-glyph glyph-button fa glyph-delete setValue toggleClass' data-value='1' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_vozidloosoby_$index' data-add-class='row-to-delete'></i>" .
                        "<i class='inline-glyph glyph-button fa glyph-undo d-none setValue toggleClass' data-value='0' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_vozidloosoby_$index' data-remove-class='row-to-delete'></i>"
                    : ''
                ) .
                '</div>' .
                '</div>';
        }
    }


    /**
     * @param Database $database
     * @param string $registracniZnacka
     * @param object $logDescription
     * @return bool
     * @throws DBException
     */
    public function zapisZmeny($database, $registracniZnacka, $logDescription)
    {
        if (!$this ->visible  ||  $this ->readOnly) {
            return true;
        }

        if (!$this ->osoby) {
            $this ->nactiDataProVozidlo($database, $registracniZnacka);
        }

        $prava = Prava::vratPrava('vozidla');
        $result = true;

        foreach ($this ->osoby as $item) {
            if ($prava->lzeZobrazitRC($item['rc'])) {
                // ma pravo editovat data teto osoby
                $htmlSuffix =
                    ($item['idPP'] ?? StringUtils::htmlTextConvert(str_replace(' ', '_', $item['rc']))) .
                    '_' . $item['platnostOd'];

                if ('1' === ($_POST["Delete_$htmlSuffix"] ?? false)) {
                    if ($this ->lzeEditovatHistorii) {
                        $recVozidloOsoby = VozidloFormular::getRecVozidloOsoby($database, $logDescription, $registracniZnacka, $item['rc'], $item['platnostOd']);
                        if ($recVozidloOsoby->loadRecord()) {
                            $recVozidloOsoby->deleteRecord();
                        }
                    }
                } else if (isset($_POST["PlatnostDo_$htmlSuffix"])) {
                    // uzivatel odeslal hodnotu pro tento input
                    try {
                        // platnost do
                        $platnostDo = $item['platnostDo'];
                        if (!$platnostDo || $platnostDo >= DateUtils::today() ||  $this ->lzeEditovatHistorii) {
                            // puvodni platnostDo neni v minulosti, nebo ma uzivatel pravo editovat historii
                            $postPlatnostDo = trim($_POST["PlatnostDo_$htmlSuffix"]);
                            $platnostDo = $postPlatnostDo === ''
                                ? null
                                : DateUtils::dateStringToTimestamp($postPlatnostDo, "PlatnostDo_$htmlSuffix");

                            if (!$this ->lzeEditovatHistorii  &&  $platnostDo  &&  $platnostDo < DateUtils::today()) {
                                // uzivatel posunul platnostDo do minulosti => nelze
                                throw new RonException(TextManager::_('Nemate_pravo_editovat_historicke_zaznamy'), "PlatnostDo_$htmlSuffix");
                            }
                        }

                        // platnost od - editovatelne jen kdyz ma uzivatel pravo editovat historii, nebo je puvodni PlatnostOd v budoucnu
                        $platnostOd = $item['platnostOd'];
                        if ($this ->lzeEditovatHistorii  ||  $item['platnostOd'] > time()) {
                            if (isset($_POST["PlatnostOd_$htmlSuffix"])) {
                                $postPlatnostOd = trim($_POST["PlatnostOd_$htmlSuffix"]);
                                $platnostOd = DateUtils::dateStringToTimestamp($postPlatnostOd, "PlatnostOd_$htmlSuffix");

                                if (!$this ->lzeEditovatHistorii  &&  $platnostOd < DateUtils::today()) {
                                    // uzivatel nema pravo editovat historii a posunul platnostOd do minulosti
                                    throw new RonException(TextManager::_('Nemate_pravo_editovat_historicke_zaznamy'), "PlatnostDo_$htmlSuffix");
                                }
                            }
                        }

                        // kontrola
                        if ($platnostDo  &&  $platnostDo < $platnostOd) {
                            throw new RonException(TextManager::_('Platnost_do_je_mensi_nez_platnost_od'), "PlatnostDo_$htmlSuffix");
                        }

                        // zapise zaznam
                        $recVozidloOsoby = VozidloFormular::getRecVozidloOsoby($database, $logDescription, $registracniZnacka, $item['rc'], $item['platnostOd']);
                        if ($this->lzeEditovatHistorii) {
                            $recVozidloOsoby['PlatnostOd'] = $platnostOd;
                        }
                        $recVozidloOsoby['PlatnostDo'] = $platnostDo;

                        if ($recVozidloOsoby->loadRecord()) {
                            $recVozidloOsoby->updateRecord();
                        }
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage($e);
                        $result = false;
                    }
                }

            }

        }

        return $result;
    }
}


class VozidloFormular extends RonPortalForm
{
    /** @var RCOsobyFormField */
    private $fieldOsoba;

    /** @var PrirazeneOsobyVozidlaField */
    private $fieldPrirazeneOsoby;

    public function __construct($database, $formAction = 'new', $dialogMode = false)
    {
        parent::__construct($database, $formAction, $dialogMode);

        $this->stornoAction = '?menuid=vozidla';

        $this->title = TextManager::_('Vozidlo');
        $this->sqlTableName = 'Vozidlo';

        $this->setXmlEventsName('VOZIDLO');

        $this->code = 'formvozidlo';

        $this->addField()
            ->setFieldName('RegistracniZnacka')
            ->setCaption('Registracni_znacka')
            ->setRequired(true) ->setPrimary(true)
            ->setMaxLength(30)
            ->setCode('field-registracniznacka');

        $this->addField()
            ->setFieldName('Popis')
            ->setCaption('Popis')
            ->setRequired(true)
            ->setMaxLength(80)
            ->setCode('field-popis');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('Aktivni')
            ->setCaption('Vozidlo_aktivni')
            ->setRequired(true)
            ->setCode('field-aktivni')
            ->setDefaultValue(1);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DruhPHM')
            ->setCaption('Druh_PHM')
            ->setSelectSql('SELECT Kod, Popis FROM DruhPHM WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setCode('field-druhphm');

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('Spotreba')
            ->setDataType('float')->addFieldCssClass('inputFloat')
            ->setCaption('Spotreba_TP')
            ->setCode('field-spotreba');

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('SpotrebaMesto')
            ->setDataType('float')->addFieldCssClass('inputFloat')
            ->setCaption('Spotreba_TP_Ve_Meste')
            ->setCode('field-spotrebamesto');

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('SpotrebaMimoMesto')
            ->setDataType('float')->addFieldCssClass('inputFloat')
            ->setCaption('Spotreba_TP_Mimo_Mesto')
            ->setCode('field-spotrebamimomesto');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('Soukrome')
            ->setCaption('Soukrome_vozidlo')
            ->setCode('field-soukrome');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('Jednostope')
            ->setCaption('Jednostope')
            ->setCode('field-jednostope')
            ->setDefaultValue(0);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('PovolenoSC')
            ->setCaption('Povoleno_pro_SC')
            ->setCode('field-povolenosc')
            ->setDefaultValue(0);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('PovolenoSPZ')
            ->setCaption('Povoleno_SPZ')
            ->setCode('field-povolenospz')
            ->setDefaultValue(1);

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka')
            ->setCaption('Poznamka');

        // Uzivatelska pole - implicitne skryta
        for ($i = 1;  $i <= 5;  $i++) {
            $this ->addField()
                ->setFieldName("Uzivatelska$i")
                ->setCaption("Vozidlo_uzivatelska_$i")
                ->setCode("field_uzivatelska$i")
                ->setVisible(false);
        }

        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osoby');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-osoby') ->setCaption('Osoby_prirazene_k_vozidlu');

        if (!$this ->isInsert()) {
            $this ->fieldPrirazeneOsoby = new PrirazeneOsobyVozidlaField();
            $this->addField($this ->fieldPrirazeneOsoby)
                ->setCode('field-prirazene-osoby');
        }

        // pridani dalsiho zamestnance
        $pravo = Prava::vratPrava('vozidla');
        $this ->fieldOsoba = new RCOsobyFormField($pravo, time());
        $this ->addField($this ->fieldOsoba)
            ->setCaption($this ->isInsert() ? 'Osoba' : 'Pridelit_vozidlo_dalsi_osobe')
            ->setCode('field-osoba')
            ->setRequired($this ->isInsert()  &&  $pravo ->prava !== Prava::PRAVA_VSE)
            ->setDefaultValue('');
    }



    public static function jePravoNaVozidlo(Database $database, string $registracniZnacka, bool $editace)
    {
        $pravo = Prava::vratPrava($editace ? 'vozidla' : 'vozidla_prohl');
        if ($pravo ->prava === Prava::PRAVA_VSE) {
            return true;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT DISTINCT RegistracniZnacka' .
            ' FROM VozidloOsoby, PracovniPomer' .
            ' WHERE RegistracniZnacka = ' . $params ->addParameter($registracniZnacka) .
            '   AND PracovniPomer.RC = VozidloOsoby.RC' .
            '   AND VozidloOsoby.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            '   AND (VozidloOsoby.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostDo IS NULL)' .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            '   AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            "\n";

        $filter = $pravo ->sqlFiltr('PracovniPomer', time(), time(), $params);
        if ($filter) {
            $sql .= ' AND (' . $filter . ')';
        }

        $cursor = $database ->select($sql, $params);
        $row = $cursor ->fetch();

        return (bool)$row;
    }


    public function loadData($usePostValues)
    {
        parent::loadData($usePostValues);

        if (!self::jePravoNaVozidlo($this ->database, $this ->data['field-registracniznacka'], true)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param Database $database
     * @param object $logDescription
     * @param ?string $registracniZnacka
     * @param ?string $rc
     * @param ?int $platnostOd
     * @return RonPortalDBRecord
     */
    public static function getRecVozidloOsoby(Database $database, $logDescription, $registracniZnacka = null, $rc = null, $platnostOd = null)
    {
        $recVozidloOsoby = new RonPortalDBRecord($database, 'VozidloOsoby', $logDescription);

        $recVozidloOsoby->addStringField('RegistracniZnacka', true, 30, $registracniZnacka);
        $recVozidloOsoby->addStringField('RC', true, 15, $rc);
        $recVozidloOsoby->addDateField('PlatnostOd', true, $platnostOd);
        $recVozidloOsoby->addDateField('PlatnostDo', true);

        if (($registracniZnacka ?? '') != ''  &&  $rc) {
            $params = $database->createParameters();
            $recVozidloOsoby->setWhere(
                'RegistracniZnacka = ' . $params ->addParameter($registracniZnacka) .
                ' AND RC = ' . $params->addParameter($rc) .
                ' AND PlatnostOd = ' . $params->addDateParameter($platnostOd),
                $params
            );
        }

        $recVozidloOsoby->setXmlEvents('VOZIDLO_OSOBY_BEFORE_POST', 'VOZIDLO_OSOBY_AFTER_POST');

        return $recVozidloOsoby;
    }



    /**
     * Kontrola, jestli je zadan existujici pracovni pomer, na ktery ma uzivatel pravo
     * @param $rec
     * @return bool
     * @throws DBException
     * @throws NeniPravoException
     */
    function beforeInsertRecord($rec)
    {
        $prava = Prava::vratPrava('vozidla');
        $osoba = $_POST['osoba'] ?? null;
        if ($osoba) {
            $pp = PracovniPomer::nactiPP($this ->database, (int)$osoba);

            if (!$prava ->lzeZobrazitRC($pp ->rc)) {
                throw new NeniPravoException();
            }
        }

        return parent::beforeInsertRecord($rec);
    }


    /**
     * Pokud uzivatel vybral novou osobu, tak ji priradi.
     * @return bool
     * @throws DBException
     */
    private function priradNovouOsobu($registracniZnacka, $zkontrolovatExistenci)
    {
        if ($_POST['osoba'] ?? null) {
            try {
                $pp = PracovniPomer::nactiPP($this->database, (int)$_POST['osoba']);

                if (!Prava::vratPrava('vozidla') ->lzeZobrazitRC($pp ->rc)) {
                    throw new RonException(TextManager::_('Pro_prirazeni_vozidla_osobe_nemate_pravo'), 'osoba');
                }

                if ($zkontrolovatExistenci) {
                    // zkontroluje, jestli uz vybrana osoba nema vozidlo k aktualnimu dni prirazeno - v tom pripade nepovoli zaznam vlozit
                    $params = $this ->database ->createParameters();
                    $cursor = $this ->database ->select(
                        'SELECT PlatnostOd, PlatnostDo' .
                        ' FROM VozidloOsoby' .
                        ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
                        '  AND RegistracniZnacka = ' . $params ->addParameter($registracniZnacka) .
                        '  AND PlatnostOd <= ' . $params ->addDateParameter(time()) .
                        '  AND (PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR PlatnostDo IS NULL)',
                        $params
                    );

                    $row = $cursor ->fetch();
                    if ($row) {
                        throw new \Exception(TextManager::_('Osoba_jiz_ma_vozidlo_k_aktualnimu_dni_prirazeno'));
                    }
                }

                // zapise vozidlo k osobe
                $recVozidloOsoby = self::getRecVozidloOsoby($this ->database, $this, $registracniZnacka, $pp ->rc, time());

                $recVozidloOsoby->insertRecord();
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e, 'osoba');
                return false;
            }
        }

        return true;
    }



    public function insertRecord()
    {
        $znacka = $_POST['RegistracniZnacka'] ?? null;
        $osoba = $_POST['osoba'] ?? null;

        if ($osoba  &&  (($znacka ?? '') != '')) {
            // zjisti, jestli uz vozidlo neexistuje a pokud ano, tak jej jen priradi (needituje)
            $params = $this ->database ->createParameters();
            $cursor = $this ->database ->select(
                'SELECT RegistracniZnacka FROM Vozidlo' .
                ' WHERE RegistracniZnacka = ' . $params ->addParameter($znacka),
                $params
            );

            $row = $cursor ->fetch();
            if ($row) {
                // vozidlo uz existuje - jen ho priradi nove osobe
                $this ->priradNovouOsobu($znacka, true);
                MessageManager::addWarning(TextManager::_('Vozidlo_jiz_existuje_bylo_pouze_prirazeno'));
                return true;
            }
        }

        return parent::insertRecord();
    }


    function afterInsertRecord($rec)
    {
        $result = $this ->priradNovouOsobu($rec['RegistracniZnacka'], false);

        return parent::afterInsertRecord($rec)  &&  $result;
    }


    function afterUpdateRecord($rec)
    {
        $result = true;

        if ($this ->fieldPrirazeneOsoby) {
            if (!$this ->fieldPrirazeneOsoby ->zapisZmeny($this ->database, $rec['RegistracniZnacka'], $this)) {
                $result = false;
            };
        }

        if (!$this ->priradNovouOsobu($rec['RegistracniZnacka'], true)) {
            $result = false;
        }

        if (!parent::afterUpdateRecord($rec)) {
            return false;
        }

        return $result;
    }




//    public function render()
//    {
//        parent::render();
//
//        $dialog = new VozidloOsobyDialog($this ->database, $this ->data['field-registracniznacka']);
//        $dialog ->render();
//    }


}
