<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Kalendar;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;
use RonPortal\User;

/**
 * Output object pro generovani reportu.
 * Class ReportOutputObject
 * @package Ron\RonPortal
 */
class ReportOutputObject implements OutputObject
{
    /** @var Report */
    var $report;

    var $format;
    //var $xmlContent; // jen obsah xml bez korenoveho elementu (<report>) a hlavicky

    // hodnoty filtru
    var $zvolenyTypObdobi;
    /** @var int    Pocatek zvoleneho obdobi (mesic nebo datum od) */
    var $obdobiOd;
    /** @var int    Konec zvoleneho obdobi */
    var $obdobiDo;
    /** @var string Nazev obdobi */
    var $nazevObdobi = null;

    /** @var PracovniPomer */
    var $pp = null;

    /** @var bool   Jestli se ma zobrazit jen jeden typ osoby */
    var $jenTypOsoby;
    /** @var int    Vybrany typ osoby */
    var $typOsoby;

    /** @var bool   Jestli se ma zobrazit jen jeden typ PP */
    var $jenTypPP;
    /** @var string Vybrany typ PP */
    var $typPP;

    /** @var bool */
    var $jenRozdeleni;
    /** @var bool */
    var $rozdeleniVcetnePodrizenych;
    /** @var Rozdeleni      Vybrane rozdeleni */
    var $rozdeleni;
    /** @var TypRozdeleni   Vybrany typ rozdeleni */
    var $typRozdeleni;

    /** @var bool */
    var $jenRozdeleni2;
    /** @var bool */
    var $rozdeleni2VcetnePodrizenych;
    /** @var Rozdeleni  Vybrane rozdeleni 2 */
    var $rozdeleni2;

    /** @var bool */
    var $jenKalendar;
    /** @var Kalendar */
    var $kalendar;

    /** @var string */
    var $order;
    /** @var string */
    var $orderName;

    /** @var string[]   Asociativni pole datumovych parametru (format j.n.Y) */
    var $dateParams = [];
    /** @var string[]   Asociativni pole parametru typu string */
    var $stringParams = [];
    /** @var int[]      Asociativni pole parametru typu integer */
    var $integerParams = [];

    /**
     * ReportOutputObject constructor.
     * @param Report $report
     * @param string $format
     * @throws \Exception
     */
    public function __construct($report = null, $format = null)
    {
        global $reportserver;

        if (!$reportserver) {
            throw new \Exception('V _appdbset.php neni nastaveno spojeni na reportserver!');
        }

        $this->report = $report;
        $this->format = $format;
    }

    /**
     * @param Database  $database
     * @param string    $reportName
     * @throws \Exception
     */
    public function loadReport($database, $reportName)
    {
        if (!$reportName) {
            throw new \Exception('Neni zadan kod sestavy!');
        }

        $this ->report = Report::loadReport($database, $reportName);

        // pokud neni nastaven format, tak nastavi prvni povoleny
        $bitMask = 0x01;
        $formaty = Report::FORMATY; // jen kvuli warningu v jIDEA
        for ($i = 0;  !$this ->format  &&  $i < count($formaty);  $i++) {
            if ($this ->report ->formaty & $bitMask) {
                $this ->format = $formaty[$i];
            }

            $bitMask *= 2;
        }
    }


    /**
     * Nacte parametry z GET (reportname, format). Ostatni parametry musi nastavit volajici kod.
     * @param Database $database
     * @param string|string[] $formName Jmeno formulare (nebo pole jmen), ke kteremu sestava patri
     * @param bool $utf8
     * @param string $deprecated_reportPrefix Pokud je zadan, zkontroluje se, ze ma nazev reportu zadany prefix
     * @throws \Exception
     */
    public function loadReportFromGet($database, $formName, $utf8 = false, $deprecated_reportPrefix = null)
    {
        $reportName = $utf8 ? StringUtils::fromUTF8($_GET['reportname']) : $_GET['reportname'];
        $this ->loadReport($database, $reportName);

        $ok = false;
        // zkontroluje, jestli zadany report patri k zadanemu formulari
        if ($formName) {
            if ($this ->report ->formular) {
                if (is_array($formName)) {
                    if (in_array($this ->report ->formular, $formName)) {
                        $ok = true;
                    }
                } else if ($formName == $this ->report ->formular) {
                    $ok = true;
                }

                if (!$ok) {
                    throw new \Exception('Zadana sestava nepatri k aktualnimu formulari!');
                }
            }
        }

        // ve starsich verzich se zobrazovaly sestavy podle zacatku kodu sestavy (ne podle prirazeneho formulare) - pro kompatibilitu jeste zkontroluje zacatek jmena
        if (!$ok  &&  $deprecated_reportPrefix) {
            if ($this ->report ->appReportName) {
                // nove sestavy uz musi mit nastaveny formName
                throw new \Exception('Sestava nema prirazen Formular! Prefix nazvu neni u sestav z manazera podporovan!');
            }

            if (0 !== stripos($reportName, $deprecated_reportPrefix)) {
                throw new \Exception('Neplatny nazev sestavy! Je ocekavan prefix ' . $deprecated_reportPrefix . '.');
            }
        }

        if (isset($_GET['format'])) {
            $this ->format =  $_GET['format'];
        }

        if ($this ->report ->appReportName) {
            ReportServer::loadReportInfo($this ->report);
        }
    }


    /**
     * @param int $typObdobi    3=datum od-do, 0=obdobi aplikace
     * @param $obdobiOd
     * @param null $obdobiDo
     * @param null $nazevObdobi
     */
    public function nastavObdobi($typObdobi, $obdobiOd, $obdobiDo, $nazevObdobi = null)
    {
        $this ->zvolenyTypObdobi = $typObdobi;
        $this ->obdobiOd = $obdobiOd;
        $this ->obdobiDo = $obdobiDo;
        $this ->nazevObdobi = $nazevObdobi;
    }

    /**
     * Prida datumove promenne
     * @param $datumOd
     * @param $datumDo
     */
    public function nastavObdobiOdDo($datumOd, $datumDo)
    {
        $this ->zvolenyTypObdobi = 3;
        $this ->obdobiOd = $datumOd;
        $this ->obdobiDo = $datumDo;
    }

    /**
     * @param PracovniPomer $pp
     * @param bool $zkontrolovatPrava
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function nastavPP($pp, $zkontrolovatPrava = true)
    {
        $this ->pp = $pp;

        if ($pp  &&  $zkontrolovatPrava) {
            if ($this ->obdobiOd) {
                Prava::nastavObdobiProPrava($this ->obdobiOd, $this ->obdobiDo);
            }

            //$prava = Prava::nactiPravaUzivatele($this ->database, User::getRC(), $this ->report ->prava ?: 'prohlizeni');
            $prava = Prava::vratPrava($this ->report ->prava ?: 'prohlizeni');

            if (!$prava->lzeZobrazitPP($this ->pp ->idPP)) {
                throw new NeniPravoException("Pro zobrazeni sestavy nemate pravo!");
            }
        }
    }

    /**
     * @param bool $jenTypOsoby
     * @param int $typOsoby
     */
    public function nastavTypOsoby($jenTypOsoby, $typOsoby = null)
    {
        $this ->jenTypOsoby = $jenTypOsoby;
        $this ->typOsoby = $jenTypOsoby ? $typOsoby : null;
    }


    /**
     * @param bool $jenTypPP
     * @param string $typPP
     */
    public function nastavTypPP($jenTypPP, $typPP = null)
    {
        $this ->jenTypPP = $jenTypPP;
        $this ->typPP = $jenTypPP ? $typPP : null;
    }


    /**
     * @param bool $jenRozdeleni
     * @param Rozdeleni $rozdeleni
     * @param bool $vcetnePodrizenych
     * @param TypRozdeleni $typRozdeleni
     */
    public function nastavRozdeleni($jenRozdeleni, $rozdeleni, $vcetnePodrizenych, $typRozdeleni = null)
    {
        $this ->jenRozdeleni = $jenRozdeleni;
        $this ->rozdeleni = $rozdeleni;
        $this ->rozdeleniVcetnePodrizenych = $vcetnePodrizenych;
        $this ->typRozdeleni = $typRozdeleni;
    }

    /**
     * @param bool  $jenRozdeleni
     * @param Rozdeleni $rozdeleni
     * @param bool  $vcetnePodrizenych
     */
    public function nastavRozdeleni2($jenRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
        $this ->jenRozdeleni2 = $jenRozdeleni;
        $this ->rozdeleni2 = $rozdeleni;
        $this ->rozdeleni2VcetnePodrizenych = $vcetnePodrizenych;
    }

    /**
     * @param bool $jenKalendar
     * @param string $kalendar
     */
    public function nastavKalendar($jenKalendar, $kalendar = null)
    {
        $this ->jenKalendar = $jenKalendar;
        $this ->kalendar = $jenKalendar ? $kalendar : null;
    }


    /**
     * @param string $order
     * @param $orderName
     * @return ReportOutputObject
     */
    public function nastavRazeni($order, $orderName)
    {
        $this->order = $order;
        $this->orderName = $orderName;

        return $this;
    }



    /**
     * Pokud jsou parametry zadany jako string (pouzivalo se ve starem kodu), tak je prevede na asociativni pole
     */
    private function processParams()
    {
        if ($this ->stringParams  &&   is_string($this ->stringParams)) {
            $result = [];
            $params = explode(';', $this ->stringParams);
            foreach ($params as $paramStr) {
                $pom = explode('=', $paramStr, 2);
                $result[$pom[0]] = $pom[1];
            }
            $this ->stringParams = $result;
        }

        if ($this ->dateParams  &&   is_string($this ->dateParams)) {
            $result = [];
            $params = explode(';', $this ->dateParams);
            foreach ($params as $paramStr) {
                $pom = explode('=', $paramStr, 2);
                $result[$pom[0]] = $pom[1];
            }
            $this ->dateParams = $result;
        }
    }


    /**
     * @param int $typObdobi
     * @param int $datumOd
     * @param int $datumDo
     * @param string $nazevObdobi
     * @return string
     */
    public static function getXMLParamsObdobi($typObdobi, $datumOd, $datumDo, $nazevObdobi)
    {
        $nazevTypuObdobiDo = '';
        $nazevObdobiDo = '';
        $pocatekPredchozihoObdobi = 0;

        switch ($typObdobi) {
            case 1 :
                // ctvrtleti
                $nazevTypuObdobi = \Ron\ronText('Ctvrtleti');
                $nazevObdobi = DateUtils::quarterDescription($datumOd);
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -3);
                break;

            case 2 :
                // rok
                $nazevTypuObdobi = \Ron\ronText('Rok');
                $nazevObdobi = date('Y', $datumOd);
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -12);
                break;

            case 3 :
                // datum od - do
                $nazevTypuObdobi = \Ron\ronText('Obdobi_od');
                $nazevTypuObdobiDo = \Ron\ronText('Do');

                $nazevObdobi = StringUtils::format('date', $datumOd);
                $nazevObdobiDo = StringUtils::format('date', $datumDo);
                break;

            case 0 :
            default :
                $nazevTypuObdobi = \Ron\ronText('Obdobi');
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -1);
                break;
        }

        return
            "    <reportparam name='TypObdobi' type='INT' value='" . $typObdobi . "'/>\n" .
            "    <reportparam name='NazevTypuObdobi' value='" . StringUtils::xmlTextConvert($nazevTypuObdobi) . "'/>\n" .
            "    <reportparam name='NazevTypuObdobiDo' value='" . StringUtils::xmlTextConvert($nazevTypuObdobiDo) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $datumOd) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $datumOd) . "'/>\n" .
            "    <reportparam name='KonecZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $datumDo) . "'/>\n" .
            "    <reportparam name='KonecZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $datumDo) . "'/>\n" .
            "    <reportparam name='NazevZvolenehoObdobi' value='" . StringUtils::xmlTextConvert($nazevObdobi) . "'/>\n" .
            "    <reportparam name='NazevZvolenehoObdobiDo' value='" . StringUtils::xmlTextConvert($nazevObdobiDo) . "'/>\n" .
            "    <reportparam name='PocatekPredchozihoObdobi' type='DATE' value='" . date('j.n.Y', $pocatekPredchozihoObdobi) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoMesice' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfMonth($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoCtvrtleti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfQuarter($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoPololeti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfHalfYear($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoRoku' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfYear($datumOd)) . "'/>\n" .
            '';
    }


    /**
     * @param PracovniPomer $pp
     * @return string
     */
    public static function getXMLParamsOsoba($pp)
    {
        return
            "    <reportparam name='Jen_Zvolena_Osoba' type='INT' value='" . ($pp ? '1' : '0') . "'/>\n" .
            "    <reportparam name='Osoba_RC' value='" . StringUtils::xmlTextConvert($pp ? $pp->rc : '') . "'/>\n" .
            "    <reportparam name='Osoba_OsobniCislo' value='" . StringUtils::xmlTextConvert($pp ? $pp->osobniCislo : '') . "'/>\n" .
            "    <reportparam name='Osoba_DatumNastupu' type='DATE' value='" . ($pp ? date('j.n.Y', $pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
            "    <reportparam name='Osoba_DatumNastupu_SQL_Const' type='SQLDATE' value='" . ($pp ? date('j.n.Y', $pp->datumNastupu) : '1.1.1970') . "'/>\n";
    }

    public function getXML()
    {
        $xmlContent =
            "  <printreport kod='" . StringUtils::xmlTextConvert($this ->report ->kod) ."' format='" . StringUtils::xmlTextConvert($this ->format) . "'>\n";

        // razeni
        if ($this ->report ->zobrazitPanelRazeni  &&  $this ->order  &&  isset($this ->report ->orderFieldsList[$this ->order])) {
            $xmlContent .=
                "    <reportparam name='ORDERING' value='" . StringUtils::htmlTextConvert($this ->order) . "'/>" .
                "    <reportparam name='ORDERING_NAME' value='" . StringUtils::htmlTextConvert($this ->report ->orderFieldsList[$this ->order]) . "'/>";
        }

        if ($this ->report ->zobrazitPanelObdobi) {
            // ... doplni promenne podle obdobi
            $xmlContent .= self::getXMLParamsObdobi((int)$this ->zvolenyTypObdobi, $this ->obdobiOd, $this ->obdobiDo, $this ->nazevObdobi);
        }

        if ($this ->report ->zobrazitPanelOsoba) {
            $xmlContent .=
                self::getXMLParamsOsoba($this ->pp) .
                "    <reportparam name='Jen_Typ_Osoby' type='INT' value='" . (int)$this->jenTypOsoby . "'/>\n" .
                "    <reportparam name='Typ_Osoby' type='INT' value='" . ($this->typOsoby ?: 0) . "'/>\n" .
                // POZOR! Ve mzdach je samostatny panel pro Typ PP a v nem jsou trochu jine promenne!
                "    <reportparam name='Jen_Typ_PP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
                "    <reportparam name='Typ_PP' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelRozdeleni) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvolene_Rozdeleni' type='INT' value='" .  ($this ->jenRozdeleni ? '1' : '0') ."'/>\n" .
                "    <reportparam name='Vcetne_Podrizenych_Rozdeleni' type='INT' value='" . ($this ->rozdeleniVcetnePodrizenych ? '1' : '0') . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Kod' type='INT' value='" . ($this ->rozdeleni ? $this ->rozdeleni ->typ : 0) . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Popis' value='" .
                ($this ->typRozdeleni
                    ? StringUtils::xmlTextConvert($this ->typRozdeleni ->popis)
                    : ($this ->rozdeleni ? $this ->rozdeleni ->typ : 0)) .
                "'/>\n" .
                "    <reportparam name='Rozdeleni_Kod' value='" . ($this ->rozdeleni ? StringUtils::xmlTextConvert($this ->rozdeleni ->kod) : '') . "'/>\n" .
                "    <reportparam name='Rozdeleni_Popis' value='" . ($this ->rozdeleni ? StringUtils::xmlTextConvert($this ->rozdeleni ->popis) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvolene_RozdeleniDo' type='INT' value='0'/>\n" .
                "    <reportparam name='RozdeleniDo_Kod' value=''/>\n" .
                "    <reportparam name='RozdeleniDo_Popis' value=''/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelRozdeleni2) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvolene_Rozdeleni2' type='INT' value='" .  ($this ->jenRozdeleni2 ? '1' : '0') ."'/>\n" .
                "    <reportparam name='Vcetne_Podrizenych_Rozdeleni2' type='INT' value='" . ($this ->rozdeleniVcetnePodrizenych ? '1' : '0') . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Kod2' type='INT' value='" . ($this ->rozdeleni2 ? $this ->rozdeleni2 ->typ : 0) . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Popis2' value='" . StringUtils::xmlTextConvert(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_NADPIS)) . "'/>\n" .
                "    <reportparam name='Rozdeleni_Kod2' value='" . ($this ->rozdeleni2 ? StringUtils::xmlTextConvert($this ->rozdeleni2 ->kod) : '') . "'/>\n" .
                "    <reportparam name='Rozdeleni_Popis2' value='" . ($this ->rozdeleni2 ? StringUtils::xmlTextConvert($this ->rozdeleni2 ->popis) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvolene_RozdeleniDo2' type='INT' value='0'/>\n" .
                "    <reportparam name='RozdeleniDo_Kod2' value=''/>\n" .
                "    <reportparam name='RozdeleniDo_Popis2' value=''/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelKalendar) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvoleny_Kalendar' type='INT' value='" . ($this ->kalendar ? '1' : '0') . "'/>\n" .
                "    <reportparam name='KalendarOd_Kod' value='" . ($this ->kalendar ? StringUtils::xmlTextConvert($this ->kalendar ->kod) : '') . "'/>\n" .
                "    <reportparam name='KalendarOd_Popis' value='" . ($this ->kalendar ? StringUtils::xmlTextConvert($this ->kalendar ->popis) : '') . "'/>\n" .
                // TODO: dodelat nastaveni "kalendar do"
                "    <reportparam name='Jen_Zvoleny_KalendarDo' type='INT' value='0'/>\n" .
                "    <reportparam name='KalendarDo_Kod' value=''/>\n" .
                "    <reportparam name='KalendarDo_Popis' value=''/>\n";
        }

        // neimplementovane panely
        $xmlContent .=
            // vycet osob
            "    <reportparam name='Jen_Zvoleny_Vycet_Osob' type='INT' value='0'/>\n" .
            //"    <reportparam name='_Zvoleny_Vycet_Osob_SQL_Filter' value='0 = 0'/>\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v1' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v1' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v1_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni 2
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v2' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v2' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v2_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            '';

        // neimplementovane panely z mezd
        if (!strcasecmp($this ->report ->applicationId, 'MZDY') ||  !strcasecmp($this ->report ->applicationId, 'PERSON')) {

//            "    <reportparam name='Jen_Typ_PP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
//            "    <reportparam name='Typ_PP' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .


            $xmlContent .=
                // typ PP
                "    <reportparam name='Jen_Zvoleny_TypPP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
                "    <reportparam name='TypPPOd_Kod' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvoleny_TypPPDo' type='INT' value='0'/>\n" .
                "    <reportparam name='TypPPDo_Kod' value=''/>\n" .
                // evidencni stav
                "    <reportparam name='Jen_Zvoleny_EvidStav' type='INT' value='0'/>\n" .
                "    <reportparam name='EvidStavOd_Kod' value=''/>\n" .
                "    <reportparam name='Jen_Zvoleny_EvidStavDo' type='INT' value='0'/>\n" .
                "    <reportparam name='EvidStavDo_Kod' value=''/>\n" .
                '';
        }

        if (!$this ->report ->appReportName) {
            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
            $xmlContent .=
                "    <reportparam name='RC' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->rc : '') . "'/>\n" .
                "    <reportparam name='osobniCislo' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->osobniCislo : '') . "'/>\n" .
                "    <reportparam name='datumNastupu' type='DATE' value='" . ($this->pp ? date('j.n.Y', $this->pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
                // u starych sestav se pouzivala promenna LoginName
                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
        }

        $xmlContent .=
            "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";

        foreach ($this ->integerParams as $paramName => $value) {
            $value = (int)$value; // pro jistotu
            $xmlContent .=
                "    <reportparam name='$paramName' type='INT' value='$value'/>\n";
        }

        foreach ($this ->stringParams as $paramName => $value) {
            $xmlContent .=
                "    <reportparam name='$paramName' value='" . StringUtils::xmlTextConvert($value) . "'/>\n";
        }

        foreach ($this ->dateParams as $paramName => $value) {
            $xmlContent .=
                "    <reportparam name='$paramName' type='DATE' value='$value'/>\n";
        }


        $xmlContent .=
            "  </printreport>\n";

//        echo "<pre>" . StringUtils::htmlTextConvert($xmlContent) . "</pre>";
//        throw new \Exception("test");

        return $xmlContent;
    }


    /**
     * Nastavi parametry (dateParams a stringParams) podle zadanych hodnot - pouzije se pro stare sestavy
     */
    public function setParams()
    {
        // pokud jsou parametry zadany jako retezec, tak je prevede do pole
        $this ->processParams();

        if (!isset($this ->dateParams['pocatekZvolenehoObdobi'])) {
            $this->dateParams['pocatekZvolenehoObdobi'] = date('j.n.Y', $this->obdobiOd);
            $this->dateParams['konecZvolenehoObdobi'] = date('j.n.Y', $this->obdobiDo);
        }

        if (!isset($this ->stringParams['TypOsoby'])) {
            $this->stringParams['TypOsoby'] = $this->jenTypOsoby ? $this->typOsoby : '';
        }

        if (!isset($this ->stringParams['TypPP'])) {
            $this->stringParams['TypPP'] = $this->jenTypPP ? $this->typPP : '';
        }

        if (!isset($this->stringParams['Rozdeleni'])) {
            $this->stringParams['TypRozdeleni'] = $this->rozdeleni ? $this->rozdeleni->typ : 0;
            $this->stringParams['Rozdeleni'] = $this->rozdeleni ? $this->rozdeleni->kod : '';
        }

        if (!isset($this->stringParams['Rozdeleni2'])) {
            $this->stringParams['TypRozdeleni2'] = $this->rozdeleni2 ? $this->rozdeleni2->typ : 0;
            $this->stringParams['Rozdeleni2'] = $this->rozdeleni2 ? $this->rozdeleni2->kod : '';
        }
    }

    /**
     * @throws \Exception
     */
    public function output()
    {
        error_reporting( E_ERROR | E_PARSE );

        $server = ReportServer::createInstance();
        try {
            if (!$this ->report ->appReportName) {
                $this ->setParams();
            }
//            if ($this ->report ->appReportName) {
            $this ->processParams(); // upravi parametry nastavene starym kodem (jako string)

            $server->sendXmlRequest($this->getXML(), $this ->report ->applicationId);

            if (isset($_GET['test'])  &&  $_GET['test'] === 'redirect') {
                HttpResponse::setHeader('Content-Type', 'text/html; charset=' . TextManager::getInstance() ->getCharSet());
                HttpResponse::setHeader(HttpHeaders::LOCATION, $_SERVER['REQUEST_URI']);
                HttpResponse::getInstance()->renderHeaders();

                $server ->downloadContent();
            } else {

                HttpResponse::getInstance()->getHeaders()->addHeadersForContent($this->report->kod . '.' . ReportServer::getFileExtension($this->format), 0);
                HttpResponse::getInstance()->renderHeaders();

                $server->outputContent();
            }
//            } else {
//                // ve stare verzi printserveru se musely puvodni reporty generovat jinym prikazem ...
//                $this ->setParams();
//                $server ->outputReport($this ->report ->kod, $this ->format, $this ->pp, $this ->stringParams, $this ->dateParams, $this ->integerParams);
//            }
        } catch (\Exception $e) {
            $server ->close();
            throw $e;
        }
    }


    /**
     * Vyexportovani reportu
     * @param PracovniPomer $pp
     * @param string[]  $stringParams
     * @param string[]  $dateParams
     * @param int[]     $integerParams
     * @return string
     * @throws \Exception
     */
    function download($pp, $stringParams, $dateParams, $integerParams)
    {
        $server = ReportServer::createInstance();
        try {
//            if ($this ->report ->appReportName) {
            $this ->stringParams = $stringParams ?: [];
            $this ->dateParams = $dateParams ?: [];
            $this ->integerParams = $integerParams ?: [];

            $server->sendXmlRequest($this->getXML());
            return $server ->downloadContent();
//            } else {
//                $server->createReport($this->report->kod, $this->format, $pp, $stringParams, $dateParams, $integerParams);
//                return $server->downloadContent();
//            }
        } catch (\Exception $e) {
            $server ->close();
            throw $e;
        }
    }


    public static function downloadReport($report, $format, $pp, $stringParams, $dateParams, $integerParams)
    {
        $outputObject = new ReportOutputObject($report, $format);
        return $outputObject ->download($pp, $stringParams, $dateParams, $integerParams);
    }
}

