<?php

namespace RonPortal\XML;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\Http\HttpHeaders;
use Ron\StringUtils;
use XMLReader;



class XMLElementAttribute
{
    public $prefix;
    public $name;
    public $value;

    /**
     * XMLElementAttribute constructor.
     * @param $prefix
     * @param $name
     * @param $value
     */
    public function __construct($prefix, $name, $value)
    {
        $this->prefix = $prefix;
        $this->name = $name;
        $this->value = $value;
    }


}


/**
 * Obecny predek pro vsechny elementy. Pri generovani HTML kopiruje svuj obsah na vystup
 * Class XMLFormElement
 */
class XMLFormElement
{
    /** @var RonPortalXMLForm */        // TODO: docasne, pak by se to melo prehodit jen na XMLForm
    public $xmlForm;
    /* @var XMLFormElement */
    public $parent;
    /** @var string */
    public $elementName;
    /* @var XMLFormElement[] */
    public $elements = [];
    /* @var string Prefix tagu v xml, napr html:... */
    public $prefix;

    public $isEmpty; // true, pokud element v XML nema zadny obsah

    /* @var XMLElementAttribute[] */
    public $attributes = [];


    /**
     * @param XMLForm $xmlForm
     * @param XMLFormElement $parent
     */
    public function __construct($xmlForm, $parent)
    {
        $this->xmlForm = $xmlForm;
        $this->parent = $parent;
    }


    /**
     * Hodnotu parametru 0/1/yes/no prevede na boolean
     * @param string $attributeValue
     * @param bool   $defaultValue  Pouzije se v pripade, ze je $attributeValue prazdne nebo null
     * @return bool
     */
    public static function getBoolValue($attributeValue, $defaultValue = false)
    {
        if ($attributeValue === 'yes' || $attributeValue === 'true') {
            return true;
        }

        if (!isset($attributeValue)  ||  $attributeValue === '') {
            return $defaultValue;
        }

        return (int)$attributeValue === 1;
    }


    public function getAttributeValue($name, $defaultValue = null)
    {
        foreach ($this->attributes as $attribute) {
            if ($attribute->name === $name  &&  !$attribute->prefix) {
                return $attribute->value;
            }
        }

        return $defaultValue;
    }


    public function getBoolAttribute($name, $defaultValue = false)
    {
        $value = $this ->getAttributeValue($name);
        if (!isset($value)) {
            return $defaultValue;
        }

        return self::getBoolValue($value, $defaultValue);
    }

    /**
     * Vraci vlastni prefix, nebo prefix parenta, pokud prefix nema
     * @return string
     */
    function getCurrentPrefix()
    {
        if ($this->prefix) {
            return $this->prefix;
        }

        return $this->parent ? $this->parent->getCurrentPrefix() : '';
    }


    /**
     * @param XMLReader $xmlReader
     * @throws Exception
     */
    protected function loadXMLChildren($xmlReader)
    {
        while ($xmlReader->nodeType !== XMLReader::END_ELEMENT) {
            $child = $this ->xmlForm ->createXMLFormElement($this, $xmlReader);

            if ($child) $this->elements[] = $child;
        }

        if ($xmlReader->localName !== $this->elementName) {
            throw new Exception('Chyba v XML formulari. Je ocekavan konec tagu ' . $this->elementName);
        }

        $xmlReader->read(); // preskoci END_ELEMENT
    }


    /* @param XMLReader $xmlReader */
    protected function loadXMLAttributes($xmlReader)
    {
        $this->attributes = [];
        while ($xmlReader->moveToNextAttribute()) {
            $attribute = new XMLElementAttribute($xmlReader->prefix, $xmlReader->localName, $xmlReader->value);
            $this->attributes[$attribute->name] = $attribute;
        }
    }


    /**
     * nacte hlavicku
     * @param XMLReader $xmlReader
     * @throws Exception
     */
    protected function loadXMLElement($xmlReader)
    {
        $this->isEmpty = $xmlReader->isEmptyElement; // musi se zjistit pred zacatkem cteni atributu!

        $this->prefix = $xmlReader->prefix;
        $this->elementName = $xmlReader->localName;

        $this->loadXMLAttributes($xmlReader);

        // prejde za atributy
        if (!$xmlReader->read()) {
            throw new Exception('Chyba XML v tagu ' . $this ->elementName . '!');
        }
    }


    /**
     * @param XMLReader $xmlReader
     * @return string
     */
    function loadXMLValue($xmlReader)
    {
        if ($this->isEmpty) return '';

        $value = '';

        while ($xmlReader->nodeType != XMLReader::END_ELEMENT && $xmlReader->nodeType != XMLReader::NONE) {
            if ($xmlReader->nodeType != XMLReader::COMMENT) {
                $value .= $xmlReader->value;
            }
            $xmlReader->read();
        }

        return $value;
    }


    /**
     * Nacte hlavicku a podrizene elementy
     * @param XMLReader $xmlReader
     */
    function loadXML($xmlReader)
    {
        $this->loadXMLElement($xmlReader);
        if (!$this->isEmpty) {
            $this->loadXMLChildren($xmlReader);
        }

        //$xmlReader ->read(); // prejde na zacatek dalsiho elementu
    }


    /**
     * @param string $name
     * @return XMLFormElement
     */
    function findChildByName($name)
    {
        foreach ($this ->elements as $child) {
            if ($child ->elementName == $name) {
                return $child;
            }
        }

        return null;
    }

    /**
     * @param string $name
     * @return XMLFormElement[]
     */
    function findChildrenByName($name)
    {
        $result = [];

        foreach ($this ->elements as $child) {
            if ($child ->elementName == $name) {
                $result[] = $child;
            }
        }

        return $result;
    }


    /**
     * vyhleda nejblizsi nadrazeny element daneho typu (podle nazvu xml tagu)
     * @param string $name
     * @return XMLFormElement
     */
    function findParentByName($name)
    {
        $parent = $this->parent;

        while ($parent && $parent->elementName != $name) {
            $parent = $parent->parent;
        }

        return $parent;
    }


    /**
     * vyhleda nejblizsi nadrazeny element dane tridy (nebo potomka dane tridy)
     * @param string $className
     * @return XMLFormElement
     */
    function findParentByClass($className)
    {
        $parent = $this->parent;

        while ($parent) {
            if ( is_a($parent, $className) ) return $parent;

            $parent = $parent->parent;
        }

        return null;
    }


    /**
     * Vraci text tohoto elementu - tj. u textovych nodu vraci obsah, u ostatni prazdny text
     * @return string
     */
    function getOwnText()
    {
        return '';
    }

    /**
     * Vraci text tohoto elementu - tj. zretezeny obsah textovych nodu. Pokud je recursive true,
     * tak pripoji i obsah vnorenych elementu.
     *
     * @param bool $recursive
     * @return string
     */
    function getTextContent($recursive)
    {
        $result = $this->getOwnText();

        foreach ($this->elements as $element) {
            $result .= $recursive
                ? $element->getTextContent(true)
                : $element->getOwnText();
        }

        return $result;
    }

    /**
     * @param string $name
     * @return mixed
     */
    function variableValue($name)
    {
        return $this->xmlForm->getVariableValue($name);
    }


    /**
     * @param string $name
     * @param mixed $value
     * @param mixed $index
     */
    function setVariableValue($name, $value, $index = null)
    {
        if (isset($index) && $index !== '') {
            // pole
            $pole = $this->xmlForm->getVariableValue($name);

            if (is_array($pole)) {
                if ($index === '[]') {
                    $pole[] = $value;
                } else {
                    $pole[$index] = $value;
                }
                $this->xmlForm->setVariable($name, $pole);
            } else if ($pole instanceof \ArrayAccess) {
                // dbrecord, globals
                $pole[$index] = $value;
            } else if (is_string($pole)  &&  is_string($value)  &&  strlen($value) === 1) {
                // nahrada jednoho znaku v retezci
                $pole[$index] = $value;
                $this->xmlForm->setVariable($name, $pole);
            } else {
                $pole = [($index === '[]' ? 0 : $index) => $value];
                $this->xmlForm->setVariable($name, $pole);
            }
        } else {
            $this->xmlForm->setVariable($name, $value);
        }

        return $this;
    }


    function setVariableValueWithFormat($name, $format, $value)
    {
        // nastavi hodnotu promenne
        $this->xmlForm->setVariable($name, $value);

        try {
            if (strcmp($value, '')) {
                switch ($format) {
                    case 'date' :
                    case 'datetime' :
                        if ($value === '') {
                            $this->xmlForm->setVariable($name . '_TIMESTAMP', null);
                        } else {
                            $pomts = DateUtils::datetimeStringToTimestamp($value);
                            $this->xmlForm->setVariable($name . '_TIMESTAMP', $pomts);
                        }
                        break;

                    case 'time' :
                        $pomts = DateUtils::timeStringToTimestamp($value);
                        $this->xmlForm->setVariable($name . '_TIMESTAMP', $pomts); // pridat jeste _FLOAT ?
                        break;

                    case 'integer' :
                    case 'int' :
                        if (!StringUtils::isInteger($value)) {
                            throw new Exception(\Ron\ronText('Bad_number_format'));
                        }
                        break;

                    case 'positive' :
                        if (!StringUtils::isInteger($value)) {
                            throw new Exception(\Ron\ronText('Bad_number_format'));
                        }
                        if ($value <= 0) {
                            throw new Exception(\Ron\ronText('Cislo_neni_vetsi_nez_0'));
                        }
                        break;

                    case 'float' :
                        if (!StringUtils::isFloat($value)) {
                            throw new Exception(\Ron\ronText('Bad_number_format'));
                        }
                        $this->xmlForm->variables[$name . '_FLOAT'] = StringUtils::stringToFloat($value);
                        break;

                    default :
                        break;
                }
            }
        } catch (Exception $e) {
            $this->xmlForm->addErrorMsg($e->getMessage(), $name); // \Ron\ronText('Bad_date_format')
        }
    }


    /**
     * @param $str
     * @param string $formatovani  Muze byt html, sql, cmd. Jakakoliv jina hodnota se interpretuje jako html
     * @return string
     */
    function nahradPromenne($str, $formatovani = '')
    {
        return $this->xmlForm->nahradPromenne($str, $formatovani);
    }

    function nahradPromenneSQL($str)
    {
        return $this->xmlForm->nahradPromenne($str, 'sql');
    }

    /**
     * Formatuje sql hodnotu na text, ale pro timestamp vraci cislo!
     * @param string $format
     * @param mixed $value
     * @return int|null|string
     */
    function formatSqlValue($format, $value)
    {
        if ($value === null) {
            return '';  // SE 16.4.2025
        }

        switch ( $format ) {
            case 'timestamp' :
                return isset($value) ? $this ->xmlForm ->database ->stringToTimestamp($value) : '';

            default :
                return DBUtils::getFormatedValue($this ->xmlForm ->database, $format, $value);
        }
    }


    /**
     * Doplni (nebo nahradi) do zadaneho pole vsechny atributy s prefixem html (maji vetsi prioritu)
     * @param string[] $attributesArray
     * @return string[]
     */
    function setHTMLAttributes($attributesArray)
    {
        foreach ($this->attributes as $attribute) {
            if ($attribute->prefix === 'htmladd') {
                $attrValue = StringUtils::fromUTF8($attribute->value);
                $attrValue = $this->nahradPromenne($attrValue);
                $attributesArray[$attribute->name] = isset($attributesArray[$attribute->name])
                    ? $attributesArray[$attribute->name] . ' ' . $attrValue
                    : $attrValue;
            }

            if ($attribute->prefix === 'html') {
                $attrValue = StringUtils::fromUTF8($attribute->value);
                $attrValue = $this->nahradPromenne($attrValue);
                $attributesArray[$attribute->name] = $attrValue;
            }
        }

        return $attributesArray;
    }


    /**
     * @param string[] $attributesarray
     * @return string
     * prevede pole atributu (name => hodnota) na HTML text name="hodnota"
     */
    function getHTMLAttributes($attributesarray)
    {
        $result = '';
        foreach ($attributesarray as $attrname => $attrvalue) {
            //$attrs_str .= ' '.$attrname.'="'.questform_convert($attrvalue).'"';
            $result .= ' ' . $attrname . '="' . $attrvalue . '"';
        }
        return $result;
    }


    /** @return string */
    function getElementsHTMLOutput()
    {
        $result = '';

        foreach ($this->elements as $element) {
            if ($this->xmlForm->exit) {
                // dale nepokracuje
                return $result;
            }

            $result .= $element->getHTMLOutput();
        }

        return $result;
    }


    /**
     * @return string
     */
    function getHTMLOutput()
    {
        $selfClosing = false;
        $generateNonce = false;

        switch (strtolower($this->elementName)) {
            case 'area' :
            case 'base' :
            case 'basefont' :
            case 'br' :
            case 'hr' :
            case 'input' :
            case 'img' :
            case 'link' :
            case 'meta' :
                $selfClosing = true;
                break;

            case 'style' :
            case 'script' :
                $generateNonce = true;
                break;
        }

        $result = '<' . $this->elementName;

        foreach ($this->attributes as $attribute) {
            $attrValue = StringUtils::fromUTF8($attribute->value);

            if ($attribute->prefix === 'html') {
                $attrValue = $this->nahradPromenne($attrValue);
            }

            $result .= ' ' . $attribute->name . '="' . $attrValue . '"';

            if ($attribute ->name === 'nonce') {
                $generateNonce = false;
            }
        }

        if ($generateNonce) {
            $result .= ' nonce="' . HttpHeaders::getNonce() . '"';
        }

        if ($selfClosing && !count($this->elements)) {
            $result .= '/>';
        } else {
            $result .= '>';

            $result .= $this->getElementsHTMLOutput();


            $result .= '</' . $this->elementName . '>';
        }

        return $result;
    }


    /**
     * Zapise obsah vnorenych elementu.
     * @param \XMLWriter $xmlWriter
     * @return void
     */
    function writeXMLElements($xmlWriter)
    {
        foreach ($this->elements as $element) {
            if ($this->xmlForm->exit) {
                // dale nepokracuje
                return;
            }

            $element->writeXMLOutput($xmlWriter);
        }
    }

    /**
     * Zapise obsah atributu elementu.
     * @param \XMLWriter $xmlWriter
     * @param array $attributes
     * @return void
     */
    function writeXMLAttributes($xmlWriter, $attributes = null)
    {
        $newAttrs = [];

        foreach ($this->attributes as $attribute) {
            //$attrvalue = iconv('utf-8', 'windows-1250', $attribute -> value); NEPREKLADAT! xmlWriter pozaduje UTF8!
            $newAttrs[($attribute->prefix ? $attribute->prefix . ':' : '') . $attribute->name] = $attribute->value;
        }

        if ($attributes) {
            // doplni a pripadne prepise atributy zadane parametrem
            foreach ($attributes as $name => $value) {
                $newAttrs[$name] = StringUtils::toUTF8($value);
            }
        }

        foreach ($newAttrs as $name => $value) {
            $xmlWriter->writeAttribute($name, $value);
        }
    }

    /**
     * @param \XMLWriter $xmlWriter
     * @return void  Zapise obsah tohoto elementu. Pouze kopiruje vstup, potomek muze generovat neco jineho
     */
    function writeXMLOutput($xmlWriter)
    {
        $xmlWriter->startElement(
            ($this->prefix ? $this->prefix . ':' : '') .
            $this->elementName
        );

        if ($this->attributes) {
            $this->writeXMLAttributes($xmlWriter);
        }

        $this->writeXMLElements($xmlWriter);

        $xmlWriter->endElement();
    }


    /**
     * @param \XMLWriter $xmlWriter
     * @return void  Zapise obsah vnorenych elementu, ale neprovadi je (jenom kopiruje)
     */
    function copyXMLElements($xmlWriter)
    {
        foreach ($this->elements as $element) {
            $element->copyXML($xmlWriter);
        }
    }


    /**
     * @param \XMLWriter $xmlWriter
     * @return void  Kopiruje obsah tohoto elementu. Kopiruje i vnorene elementy, ale neprovadi je.
     */
    function copyXML($xmlWriter)
    {
        $xmlWriter->startElement(
            ($this->prefix ? $this->prefix . ':' : '') .
            $this->elementName
        );

        if ($this->attributes) {
            $this->writeXMLAttributes($xmlWriter);
        }

        $this->copyXMLElements($xmlWriter);

        $xmlWriter->endElement();
    }


    /**
     * Pomocna metoda - zapise do XML tag variable
     * @param \XMLWriter $xmlWriter
     * @param string $name
     * @param string $value
     */
    function writeXmlVariable($xmlWriter, $name, $value)
    {
        $xmlWriter->startElement('variable');
        $xmlWriter->writeAttribute('name', $name);
        $xmlWriter->writeAttribute('value', StringUtils::toUTF8($value));
        $xmlWriter->endElement();
    }


    function elementsEvent()
    {
        foreach ($this->elements as $element) {
            if ($this->xmlForm->exit) {
                // dale nepokracuje
                return;
            }

            $element->event();
        }
    }

    /**
     * Provede udalost - tj. negeneruje vystup, jen provadi prikazy
     */
    function event()
    {
        $this->elementsEvent();
    }



    function processElements()
    {
        foreach ($this->elements as $element) {
            if ($this->xmlForm->exit) {
                // dale nepokracuje
                return;
            }

            $element->process();
        }
    }

    /**
     * Zpracovani elementu pri vyplnovani datoveho XML
     */
    function process()
    {
        $this ->processElements();
    }


    function loadElementsData()
    {
        foreach ($this->elements as $element) {
            $element->loadData();
        }
    }

    function loadData()
    {
        $this ->loadElementsData();
    }
}
