<?php

namespace RonPortal\Zamestnanci;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\System\RecordNotFoundException;

class NahlasenaZmena
{
    const STAV_NEZPRACOVANO = 0;
    const STAV_ZPRACOVANO = 1;
    const STAV_ODMITNUTO = 2;

    // hodnoty nactene z typu nahlasene zmeny
    /** @var string     Kod typu nahlasene zmeny */
    public $typZmeny;
    /** @var string     Popis typu nahlasene zmeny */
    public $popisTypuZmeny;
    /** @var int        Priznak, jestli se u tohoto typu zmeny vyzaduje schvaleni */
    public $vyzadovatSchvaleni;
    /** @var int        Priznak, jestli je povoleno nahlasit tuto zmenu v Portalu */
    public $povolitVIntrawebu;

    // hodnoty nactene z konkretni nahlasene zmeny
    /** @var int        Id zaznamu */
    public $id;
    /** @var string     RC osoby, ktere se zmena tyka */
    public $rc;
    /** @var int        Cas nahlaseni zmeny */
    public $datumCas;

    /** @var int        Stav nahlasene zmeny 0=nezpracovano, 1=zpracovano, 2=odmitnuto */
    public $stav;
    /** @var int        Cas zpracovani zmeny */
    public $datumCasZpracovani;

    /** @var string     RC uzivatele, ktery zmenu zpracoval */
    public $zpracoval;
    /** @var string     RC uzivatele, ktery zmenu nahlasil */
    public $nahlasil;

    /** @var string     Poznamka zadna uzivatelem */
    public $poznamka;

    /** @var string[]   Asociativni pole hodnot */
    public $hodnoty = [];

    /**
     * Nacte zaznam nahlasene zmeny se zadanym Id
     * @param Database $database
     * @param int $id
     * @return NahlasenaZmena
     * @throws RecordNotFoundException
     * @throws DBException
     */
    public static function nactiZmenu(Database $database, int $id):NahlasenaZmena
    {
        $params = $database ->createParameters();
        $row = DBUtils::getSqlValue($database,
            'SELECT TypNahlaseneZmeny.Kod, TypNahlaseneZmeny.Popis, TypNahlaseneZmeny.VyzadovatSchvaleni, TypNahlaseneZmeny.PovolitVIntrawebu,' . // 0..3
            ' NahlasenaZmena.RC, NahlasenaZmena.DatumCas, NahlasenaZmena.Stav, NahlasenaZmena.Hodnoty,' . // 4..7
            ' NahlasenaZmena.DatumCasZpracovani, NahlasenaZmena.Zpracoval, NahlasenaZmena.Nahlasil, NahlasenaZmena.Poznamka,' . // 8..11
            ' NahlasenaZmena.Id' . // 12
            ' FROM NahlasenaZmena, TypNahlaseneZmeny' .
            ' WHERE NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod' .
            '   AND NahlasenaZmena.Id = ' . $params ->addParameter($id),
            $params, true);

        if (!$row) {
            throw new RecordNotFoundException('Zaznam zmeny nenalezen!');
        }

        $zmena = new static();
        $zmena ->loadRow($database, $row);
        return $zmena;
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param bool $pouzeNezpracovane
     * @return NahlasenaZmena[]
     * @throws DBException
     */
    public static function nactiZmeny(Database $database, string $rc, $pouzeNezpracovane)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT TypNahlaseneZmeny.Kod, TypNahlaseneZmeny.Popis, TypNahlaseneZmeny.VyzadovatSchvaleni, TypNahlaseneZmeny.PovolitVIntrawebu,' . // 0..3
            ' NahlasenaZmena.RC, NahlasenaZmena.DatumCas, NahlasenaZmena.Stav, NahlasenaZmena.Hodnoty,' . // 4..7
            ' NahlasenaZmena.DatumCasZpracovani, NahlasenaZmena.Zpracoval, NahlasenaZmena.Nahlasil, NahlasenaZmena.Poznamka,' . // 8..11
            ' NahlasenaZmena.Id' . // 12
            ' FROM NahlasenaZmena, TypNahlaseneZmeny' .
            ' WHERE NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod' .
            '   AND NahlasenaZmena.RC = ' . $params ->addParameter($rc) .
            ($pouzeNezpracovane ? ' AND NahlasenaZmena.Stav = 0' : '') .
            'ORDER BY DatumCas',
            $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $zmena = new NahlasenaZmena();
            $zmena ->loadRow($database, $row);
            $result[] = $zmena;
        }

        return $result;
    }

    
    private function loadRow($database, $row)
    {
        $this ->id = (int)$row[12];

        $this ->typZmeny = $row[0];
        $this ->popisTypuZmeny = $row[1];
        $this ->vyzadovatSchvaleni = (int)$row[2];
        $this ->povolitVIntrawebu = (int)$row[3];

        $this ->rc = $row[4];
        $this ->datumCas = $database ->stringToTimestamp($row[5]);
        $this ->stav = $row[6];

        $this ->datumCasZpracovani = $database ->stringToTimestamp($row[8]);
        $this ->zpracoval = $row[9];
        $this ->nahlasil = $row[10];
        $this ->poznamka = $row[11];

        $this ->hodnoty = [];
        $hodnotyRows = explode("\n", $row[7]);
        foreach ($hodnotyRows as $hodnotaRow) {
            $pom = explode("=", $hodnotaRow, 2);
            if ($pom[0] ?? null) {
                $this ->hodnoty[trim($pom[0])] = trim($pom[1] ?? '');
            }
        }
    }


    /**
     * Nacte sablonu zmeny (zaznam z tabulky TypZmeny).
     * @param Database $database
     * @param string $kodTypu
     * @return NahlasenaZmena
     * @throws RecordNotFoundException|DBException
     */
    public static function nactiTypZmeny(Database $database, string $kodTypu):NahlasenaZmena
    {
        $params = $database ->createParameters();
        $row = DBUtils::getSqlValue($database,
            'SELECT Kod, Popis, VyzadovatSchvaleni, PovolitVIntrawebu' . // 0..3
            ' FROM TypNahlaseneZmeny' .
            ' WHERE Kod = ' . $params ->addParameter($kodTypu),
            $params, true);

        if (!$row) {
            throw new RecordNotFoundException('Zaznam typu zmeny nenalezen!');
        }

        $zmena = new static();
        $zmena ->typZmeny = $row[0];
        $zmena ->popisTypuZmeny = $row[1];
        $zmena ->vyzadovatSchvaleni = (int)$row[2];
        $zmena ->povolitVIntrawebu = (int)$row[3];

        return $zmena;
    }



    /**
     * Suffixy pro kody uzivatelsky prav pro prohlizeni jednotlivych sekci v osobnich udajich a hlaseni zmen
     * podle kodu typu zmeny. Pokud zde neni zadan, pouzije se primo kod typu zmeny (v pripadech kdy je max. 11 znaku)
     */
    const KODY_PRAV = [
        'penzijnipripojisteni' => 'penzijni',
        'rodinnyprislusnik' =>  'prislusnik',
        'zivotnipojisteni' =>   'zivotni',
        'pracovnimista' =>      'pracmista',
        'velikostpomucek' =>    'velikosti',
        'novyclenrodiny' =>     'novyclen',
        'emailprovyplatnici' => 'emailvl'
        // ucetosoby, trvale, prechodne, prukaz, rodinnystav, pojistovna, vzdelani, pomucky, osobni
    ];


    /**
     * Zjisti, jestli ma uzivatel zadane pravo ($typPrava) na zadany typ zmeny pro zadanou osobu
     * @param string $typZmeny  Kod typu zmeny
     * @param string $typPrava  Kod typu prava, ktere se ma zkontrolovat (prohl_osobni, ou_zmeny_schval nebo ou_zmeny_nahlaseni)
     * @param string $rc        Id osoby
     * @return bool
     * @throws \Exception
     */
    public static function jePravoNaTypZmeny($typZmeny, $typPrava, $rc)
    {
        //prohl_osobni          + max 17 znaku
        //ou_zmeny_nahlaseni    + max 11 znaku
        //ou_zmeny_schval       + max 14 znaku

        // nektere kody prav jsou zkracene (suffix muze mit max 11 znaku)
        $suffix = self::KODY_PRAV[$typZmeny] ?? $typZmeny;

        return Prava::vratPrava($typPrava, $typPrava . '_' . $suffix) ->lzeZobrazitRC($rc);
    }


    /**
     * Zkontroluje, jestli ma uzivatel pravo na prohlizeni/editaci zadaneho typu zmeny
     * @param string $typZmeny  Kod typu zmeny
     * @param string $rc        RC osoby (z tabulky Osoba.RC)
     * @param bool $editace     Pokud je true, musi mit pravo na editaci.
     * @param bool $schvaleni   Pokud je true, musi mit pravo na schvaleni
     * @return bool
     * @throws \Exception
     */
    public static function jePravoNaZmenu($typZmeny, $rc, $editace, $schvaleni = false)
    {
        //$suffix = self::KODY_PRAV[$typZmeny] ?? $typZmeny;

        if (!$editace  &&  !$schvaleni) {
            if (self::jePravoNaTypZmeny($typZmeny, 'prohl_osobni', $rc)) {
                // OK, muze prohlizet tuto zmenu
                return true;
            }
            // pokracuje dale, protoze muze zmenu zobrazit, kdyz ma na ni pravo editace nebo schvaleni
        }

        //if (Prava::vratPrava('ou_zmeny_schval', 'ou_zmeny_schval_' . $typZmeny) ->lzeZobrazitRC($rc)) {
        if (self::jePravoNaTypZmeny($typZmeny, 'ou_zmeny_schval', $rc)) {
            // muze zmenu schvalit => takze ji muze i zobrazit a editovat
            return true;
        } else {
            // nemuze schvalovat
            if ($schvaleni) {
                return false;
            }
        }

        // zkontroluje pravo na editaci
        //if (Prava::vratPrava('ou_zmeny_nahlaseni', 'ou_zmeny_nahlaseni_' . $typZmeny)->lzeZobrazitRC($rc)) {
        if (self::jePravoNaTypZmeny($typZmeny, 'ou_zmeny_nahlaseni', $rc)) {
            // OK, muze nahlasit tuto zmenu => ma pravo prohlizet a editovat
            return true;
        }

        return false;
    }


    public static function zkontrolujPravoNaZapsanouZmenu(Database $database, int $idZmeny, $editace = false, $schvaleni = false)
    {
        $zmena = self::nactiZmenu($database, $idZmeny);
        if (!self::jePravoNaZmenu($zmena ->typZmeny, $zmena ->rc, $editace, $schvaleni)) {
            throw new NeniPravoException();
        }
    }
}
